;
;	Part 6 Event Handler Routines - Linearity Tables Building & Management
;
;---------------------------------------------------------------------------
pro P6_Select_Make_New_Tables, Event
	;
	;	just a button sensitivity manager
	;
	;	if selected, turn on droplist for nights, turn off other top buttons
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	l_ptr = PTR_struct.PTR_Linearity
	;
	;	turn off current three top buttons
	;
	for iw = 0, 2 do P6_button_sensitize, Event, iw, 0
	;
	;	turn on the top (night) droplist	( which turns on other droplists )
	;
	WIDGET_CONTROL, (*s_ptr).P6_droplist_wid[0], SENSITIVE=1
	;
	;	and populate it
	;
	P6_droplist_populate, Event, 0
	;
	;	clear the group image counters
	;
	(*l_ptr).n_lin_list = 0L
	(*l_ptr).n_bias_list = 0L
	;
	;	identify this linearity table
	;
	this_lin = (*l_ptr).this_lin + 1
	(*l_ptr).this_lin = this_lin
	(*l_ptr).lin_id[this_lin] = this_lin
end
;
;-----------------------------------------------
pro P6_Select_Night, Event
	;
	MSP_BDP_print_text, Event, "Night Selection Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	;
	wid = (*s_ptr).P6_droplist_wid[0]
	;
	night_index = Event.index
	;
	;	index is not enough, as we might not have done all the previous parts
	;	So, need to get the night designation from the list
	;
	WIDGET_CONTROL, wid, GET_VALUE = vector_night_desig
	selected_night_desig = vector_night_desig[night_index]
	;
	night_desig = selected_night_desig
	night_text = STRING(format='(I8.8)',night_desig)
	;
	;	read in that night's save file
	;
	MSP_BDP_print_text, Event, "Opening Night Summary for night = "+night_text
	;
	MSP_BDP_Create_Night_File_Path, Event, night_desig, night_file_path
	MSP_BDP_Restore_Night_File, Event, night_file_path, description, ierr
	;
	if(ierr ne 0) then begin
		MSP_BDP_print_text, Event, "Could not restore night "+night_text+" - skipping"
		goto, P6SN_end
	endif
	;
	;	must redo pointers
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	raw_root = (*s_ptr).root_raw
	;
	;	capture number of groups
	;
	ngroups = (*n_ptr).num_groups
	ngroups_text = STRING(format='(I)',ngroups)
	MSP_BDP_print_text, Event, "Restored "+night_text+" with "+ngroups_text+" groups"
	;
	;	Turn on droplists below this one
	;
	for iw = 1, N_ELEMENTS( (*s_ptr).P6_droplist_wid)-1 do begin
		WIDGET_CONTROL, (*s_ptr).P6_droplist_wid[iw], SENSITIVE=1
	endfor
	;
	;	and populate them
	;
	;	method 1 tab entries:
	;
	;	20101224 - DPC - turned off to deprecate this method
	;
	;P6_Linearity_Droplist_Populate, Event, 1
	;P6_Bias_Droplist_Populate, Event, 2
	;
	;	method 2 tab entries:
	;
	P6_Linearity_Droplist_Populate, Event, 3
	P6_Bias_Droplist_Populate, Event, 5
	P6_Linearity_Droplist_Populate, Event, 4
	P6_Bias_Droplist_Populate, Event, 6
	;
	this_lin = (*l_ptr).this_lin
	(*l_ptr).lin_night[this_lin] = night_desig
	(*l_ptr).lin_format[this_lin] = 'Full'				; for now
	;
	;	and turn on the "Read" step 4 button, in case we are returning
	;
	P6_button_sensitize, Event, 6, 1
	;
	P6SN_end:
end
;
;-------------------------------
pro P6_List_All_Tables, Event
	;
	;	list the currently registered linearity tables
	;
	MSP_BDP_print_text, Event, "List Registered Linearity Tables Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	;
	n_lin_tables = (*l_ptr).n_lin_tables
	;
	;	if none selected, say so
	;
	if(n_lin_tables eq 0) then begin
		n_lin_tables = 1
		no_tables = 1
	endif else no_tables = 0
	;
	col_headers = ["Reg. No.","__Night__", "_________Fully Qualified Pathname to First Linearity Table__________"]
	;
	format = strarr(3,n_lin_tables)
	format[0:2,*] = '(a)'
	;
	color_array = strarr(3,n_lin_tables)
	;
	row_headers = strarr(n_lin_tables)
	data_line = {reg_no: '', night: '', path: ''}
	data_array = REPLICATE(data_line, n_lin_tables)			; array of structures
	;
	nc = 0
	;
	for ii = 0, n_lin_tables-1 do begin
			;
			row_headers[ii] = STRING(format='(i3.1)',ii+1)
			color_array[*,ii] = 'yellow'
			;
			if(no_tables) then begin
				data_array[ii].reg_no = ''
				data_array[ii].night = ''
				data_array[ii].path = 'No Linearity Tables are registered yet'
			endif else begin
				data_array[ii].reg_no = STRTRIM(STRING(format='(I3.1)',ii+1),2)
				file_name = FILE_BASENAME( (*l_ptr).lin_table_0_path[ii] )
				night_desig_text = STRMID(file_name,0,8)
				data_array[ii].night = night_desig_text
				data_array[ii].path  = (*l_ptr).lin_table_0_path[ii]
			endelse
			;
	endfor
	;
	r_string = "Registered Linearity Tables "
  if( (*s_ptr).default_font ne 'None') then begin
	 new_struct = Dialog_Big_Table(Title="Registered Linearity Tables",Label=r_string, $
					Group_Leader=Event.top, Col_Headers = col_headers, $
					Format=format, Row_headers = row_headers,$
					Data_array = data_array, Font='Helvetica*16', $
					color_array=color_array)
	endif else begin
	  new_struct = Dialog_Big_Table(Title="Registered Linearity Tables",Label=r_string, $
	    Group_Leader=Event.top, Col_Headers = col_headers, $
	    Format=format, Row_headers = row_headers,$
	    Data_array = data_array, Font='None', $
	    color_array=color_array)
	endelse
	;
end
;
;------------------------------------------------------------
pro P6_Import_Existing_Table, Event
	;
	;	registers a linearity table into the list of such tables
	;
	MSP_BDP_print_text, Event, "Register a Linearity Table Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	;
	n_lin_tables = (*l_ptr).n_lin_tables
	;
	;	get the pathname
	;
	MSP_BDP_print_text, Event, "Select the pathname to the Lin_0 (first) linearity table"
	;
	start_dir = (*s_ptr).root_reduced
	lin_path = Dialog_Pickfile(Title='Select Lin_0 Table',FILTER='*_Lin_0.fits',PATH=start_dir)
	;
	if(lin_path eq '') then begin
		MSP_BDP_print_text, Event, "No Path Selected. Try again"
		goto, PIET_end
	endif
	(*l_ptr).lin_table_0_path[n_lin_tables] = lin_path
	(*l_ptr).n_lin_tables++
	;
	PIET_end:
end
;
;---------------------------------------------------------------------
;
;	METHOD 1 - Multiple Images - Programs
;
;-----------------------------------------------------------------------
pro P6_Select_Linearity_Group, Event, droplist_number
	;
	MSP_BDP_print_text, Event, "Linearity Group Selection Requested"
	;
	if(droplist_number eq 1) then begin
		MSP_BDP_print_text, Event, "This method deprecated - use Method 2"
		;
		goto, P6_SLG_deprecated
	endif
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	;
	wid = (*s_ptr).P6_droplist_wid[droplist_number]
	;
	group_index = Event.index
	;
	WIDGET_CONTROL, wid, GET_VALUE = vector_group_desig
	selected_group_desig = vector_group_desig[group_index]
	;
	;	the form of the entry is "Grp_" then number, then space
	;
	space = STRPOS(selected_group_desig,' ')
	;
	num = STRMID(selected_group_desig, 4, (space-4))
	READS,num, format='(I)',ID
	;
	;	save in linearity structure
	;
	if(droplist_number eq 1) then (*l_ptr).m1_lin_grp = ID
	if(droplist_number eq 3) then begin
		(*l_ptr).m2_lin_first_grp = ID
		print, "Found a first linearity group, number = ",ID
	endif
	if(droplist_number eq 4) then begin
		(*l_ptr).m2_lin_last_grp = ID
		print, "Found a last linearity group, number - ",ID
	endif
	;
	group_text = STRTRIM(STRING(format='(I)',ID),2)
	;
	;	if method 1:
	;
	if(droplist_number eq 1) then begin
		;
		;	read in that group's image list
		;
		n_images = (*n_ptr).groups[ID].group_images
		n_images_text = STRTRIM(STRING(format='(I)',n_images),2)
		(*l_ptr).n_lin_list = n_images
		;
		lin_list = (*n_ptr).groups[ID].group_list[0:n_images-1]
		(*l_ptr).lin_list[0:n_images-1] = lin_list
		MSP_BDP_print_text, Event, "Group "+group_text+" has "+n_images_text+" linearity images"
		;
		;	Turn on buttons below this one if bias group selected
		;
		if( (*l_ptr).n_lin_list ne 0 and (*l_ptr).n_bias_list ne 0) then begin
			P6_button_sensitize, Event, 3, 1
		endif
		;
		image_0 = (*n_ptr).groups[ID].group_start
		ind = WHERE(image_0 eq (*n_ptr).image_list[*],count)
		if(count ne 0) then begin
			UT = (*n_ptr).header_list[ind[0]].ut
			this_lin = (*l_ptr).this_lin
			(*l_ptr).lin_UT[this_lin] = UT
		endif
	endif else begin
		;
		;	if Method 2, and bias groups already selected, turn on button 11
		;
		if( ((*l_ptr).m2_lin_first_grp ne -1) and ((*l_ptr).m2_lin_last_grp ne -1) $
		   and ((*l_ptr).m2_bias_first_grp ne -1) and ((*l_ptr).m2_bias_last_grp ne -1)) then begin
		   		P6_button_sensitize, Event, 11, 1
		endif
		;
	endelse
	;
	P6_SLG_deprecated:
end
;
;------------------------------------------------------------------
pro P6_Select_Bias_Group, Event, droplist_number
	;
	MSP_BDP_print_text, Event, "Bias Group Selection Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	;
	wid = (*s_ptr).P6_droplist_wid[droplist_number]
	;
	bias_index = Event.index
	;
	WIDGET_CONTROL, wid, GET_VALUE = vector_bias_desig
	selected_bias_desig = vector_bias_desig[bias_index]
	;
	;	the form of the entry is "Grp_" then number, then space
	;
	space = STRPOS(selected_bias_desig,' ')
	;
	num = STRMID(selected_bias_desig, 4, (space-4))
	READS,num, format='(I)',ID
	;
	;	save in linearity structure
	;
	if(droplist_number eq 2) then (*l_ptr).m1_bias_grp = ID
	if(droplist_number eq 5) then begin
		(*l_ptr).m2_bias_first_grp = ID
		print, "Found a first bias group, number = ",ID
	endif
	if(droplist_number eq 6) then begin
		(*l_ptr).m2_bias_last_grp = ID
		print, "Found a last bias group, number - ",ID
	endif
	;
	bias_text = STRTRIM(STRING(format='(I)',ID),2)
	;
	;	if method 1:
	;
	if(droplist_number eq 2) then begin;
		;
		;	read in that group's image list
		;
		n_images = (*n_ptr).groups[ID].group_images
		n_images_text = STRTRIM(STRING(format='(I)',n_images),2)
		(*l_ptr).n_bias_list = n_images
		;
		bias_list = (*n_ptr).groups[ID].group_list[0:n_images-1]
		(*l_ptr).bias_list[0:n_images-1] = bias_list
		MSP_BDP_print_text, Event, "Group "+bias_text+" has "+n_images_text+" bias images"
		;
		;	Turn on buttons below this one if linearity group selected
		;
		if( (*l_ptr).n_lin_list ne 0 and (*l_ptr).n_bias_list ne 0) then begin
			P6_button_sensitize, Event, 3, 1
		endif
	endif else begin
		;
		;	if Method 2, and bias groups already selected, turn on button 11
		;
		if( ((*l_ptr).m2_lin_first_grp ne -1) and ((*l_ptr).m2_lin_last_grp ne -1) $
		   and ((*l_ptr).m2_bias_first_grp ne -1) and ((*l_ptr).m2_bias_last_grp ne -1)) then begin
		   		P6_button_sensitize, Event, 11, 1
		endif
		;
	endelse
	;
end
;
;------------------------------------------------------------
pro P6_Make_Image_Matches, Event
	;
	;	replacement for "make_linearity_list.pro" this program
	;	generates lists of fully qualified pathnames for linearity
	;	steps and bias
	;
	MSP_BDP_print_text, Event, "Make Image Match Lists Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	;
	night_desig = (*n_ptr).night_desig
	night_desig_text = STRING(format='(I8.8)',night_desig)
	;
	;	do bias list first
	;
	n_bias_paths = 0L
	n_bias_list = (*l_ptr).n_bias_list
	for ib = 0, n_bias_list-1 do begin
		;
		;	fetch the image number
		;
		image_no = (*l_ptr).bias_list[ib]
		image_no_text = STRTRIM(STRING(format='(i5.3)',image_no),2)
		;
		;	convert to CDS image name
		;
		CDS_name = night_desig_text + '.' + image_no_text + '.fits'
		;
		;	get raw directory index by match to night's list of images
		;
		ind = WHERE(image_no eq (*n_ptr).image_list[*],count)
		if(count eq 0) then begin
			MSP_BDP_print_text, Event, "ERROR - no match of image "+image_no_text+" for night "+night_desig_text
			goto, b_loop_end
		endif
		;
		ii = ind[0]
		;
		image_index = (*n_ptr).image_root_index[ii]
		;
		if(image_index eq 0) then begin
			base_dir = (*s_ptr).root_raw
		endif else begin
			base_dir = (*s_ptr).root_reduced
		endelse
		;
		;	recover CDS, RAW1 directories
		;
		CDS_dir = base_dir + (*n_ptr).paths_cds_root[image_index]
		;RAW1_dir = (*n_ptr).paths_raw1_root[image_index]
		;
		;	form full pathname to CDS file
		;
		path = fix_path( FILEPATH(CDS_name, ROOT_DIR=CDS_dir) )
		(*l_ptr).bias_paths[n_bias_paths] = path
		n_bias_paths++
		(*l_ptr).n_bias_paths = n_bias_paths
		;
		b_loop_end:
	endfor
	MSP_BDP_print_text, Event, "Created full pathnames for "+STRING(format='(i)',n_bias_paths)+" bias frames "
	;
	;	and for all linearity images, separated by integration times
	;
	;-----------------
	;	begin by making list of integration times
	;
	expts = dblarr(2000)
	ind_expts = lonarr(2000)
	n_expts = 0L
	n_lin_list = (*l_ptr).n_lin_list
	for il = 0, n_lin_list-1 do begin
		image_no = (*l_ptr).lin_list[il]
		image_no_text = STRTRIM(STRING(format='(i5.3)',image_no),2)
		;
		;	convert to CDS image name
		;
;		CDS_name = night_desig_text + '.' + image_no_text + '.fits'
		;
		;	find the corresponding image no in the night list, then get its integration time
		;
		ind = WHERE( image_no eq (*n_ptr).image_list[*],count)
		if(count eq 0) then begin
			MSP_BDP_print_text, Event, "ERROR - no match of image "+image_no_text+" for night "+night_desig_text
			goto, l_loop_end
		endif
		;
		;	get integration time
		;
		expts[n_expts] = (*n_ptr).image_exptime[ind[0]]		; expt time
		ind_expts[n_expts] = ind[0]							; index to image in night_struct arrays
		n_expts++
		;
		l_loop_end:
	endfor
	;
	;	trim expts array
	;
	expts = expts[0:n_expts-1]
	;
	;	now, build the list of unique, sorted exposure times
	;
	us_expts = expts[UNIQ(expts,SORT(expts))]
	;
	n_lin_expt = N_ELEMENTS(us_expts)
	(*l_ptr).n_lin_expt = n_lin_expt
	(*l_ptr).lin_expt[0:n_lin_expt-1] = us_expts[*]
	(*l_ptr).lin_expt[n_lin_expt:N_ELEMENTS((*l_ptr).lin_expt)-1] = 99.9		; the rest of the array
	;
	;	integration time list completed, now collect full pathnames for each image for each time
	;
	jnd_expts = SORT(expts)			; indices to sorted list
	last_expt = -1.0
	t_counter = -1
	i_counter = -1
	for it = 0, n_expts-1 do begin
		ind = ind_expts[jnd_expts[it]]
		this_expt = (*n_ptr).image_exptime[ind]
		if(this_expt ne last_expt) then begin
			t_counter++
			i_counter = 0
			last_expt = this_expt
		endif
		;
		;	get image number, and create full pathname
		;
		image_no = (*n_ptr).image_list[ind]
		image_no_text = STRTRIM(STRING(format='(i5.3)',image_no),2)
		;
		;	convert to CDS image name
		;
		CDS_name = night_desig_text + '.' + image_no_text + '.fits'
		;
		image_index = (*n_ptr).image_root_index[ind]
		if(image_index eq 0) then begin
			base_dir = (*s_ptr).root_raw
		endif else begin
			base_dir = (*s_ptr).root_reduced
		endelse
		;
		;	recover CDS, RAW1 directories
		;
		CDS_dir = base_dir + (*n_ptr).paths_cds_root[image_index]
		;RAW1_dir = (*n_ptr).paths_raw1_root[image_index]
		;
		;	form full pathname to CDS file
		;
		path = fix_path( FILEPATH(CDS_name, ROOT_DIR=CDS_dir) )
		;
		(*l_ptr).lin_paths[t_counter, i_counter] = path
		i_counter++
		(*l_ptr).n_lin_paths[t_counter] = i_counter
	endfor
	;
	MSP_BDP_print_text, Event, "Created full pathnames for "+STRING(format='(i)',n_expts)+" linearity frames "
	;
	;	turn on next button (view/edit)
	;
	P6_button_sensitize, Event, 4, 1
	;
end
;
;------------------------------------------------------------
pro P6_View_Image_Match_Table, Event
	;
	;	put up table of matches and pathnames, bias first, then linearity, sorted by times
	;
	MSP_BDP_print_text, Event, "View Image Match Lists Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	;
	;	build the table structure
	;
	col_headers = ["Image No.","Expos. Time","_____Fully Qualified Pathname (including redirects)_____"]
	;
	;	number of lines = n_bias + n_lin
	;	load all the info into one big string array
	;
	n_bias = (*l_ptr).n_bias_paths
	lin_stuff = strarr(3,2000)
	n_lin_stuff = 0L
	for ib = 0, n_bias-1 do begin
		lin_stuff[0,n_lin_stuff] = STRTRIM(STRING(format='(i)',ib+1),2)
		lin_stuff[1,n_lin_stuff] = 'Bias'
		lin_stuff[2,n_lin_stuff] = (*l_ptr).bias_paths[ib]
		n_lin_stuff++
	endfor
	;
	n_expts = (*l_ptr).n_lin_expt
	;
	for ie = 0, n_expts-1 do begin
		;
		;	load the lin_stuff array
		;
		n_images = (*l_ptr).n_lin_paths[ie]
		for ii = 0, n_images-1 do begin
			lin_stuff[0,n_lin_stuff] = STRTRIM(STRING(format='(i)',ii+1),2)
			lin_stuff[1,n_lin_stuff] = STRING(format='(f10.6)',(*l_ptr).lin_expt[ie])
			lin_stuff[2,n_lin_stuff] = (*l_ptr).lin_paths[ie,ii]
			n_lin_stuff++
		endfor
	endfor
	n_lines = n_lin_stuff
	lin_stuff = lin_stuff[*,0:n_lines-1]
	;
	format = strarr(3,n_lines)
	format[0,*] = '(a)'
	format[1,*] = '(a)'
	format[2,*] = '(a)'
	;
	color_array = strarr(3,n_lines)
	;
	row_headers = strarr(n_lines)
	data_line = {image_no: '', expt: '', path: ''}
	data_array = REPLICATE(data_line, n_lines)			; array of structures
	;
	dummy = 0.0
	last_expt = STRING(format='(f10.6)',dummy)
	nc = 0
	;
	for ii = 0, n_lines-1 do begin
			;
			;	load structure for this image
			;
			if(ii lt n_bias) then begin
				row_headers[ii] = STRING(format='("B_",i3.3)',ii)
				color_array[*,ii] = 'yellow'
			endif else begin
				this_expt = lin_stuff[1,ii]
				if(this_expt ne last_expt) then begin
					last_expt = this_expt
					nc++
					if(nc ge 6) then nc=0
				endif
				row_headers[ii] = STRING(format='("L_",i3.3)',ii)
				case nc of
					0: n_color = 'orange'
					1: n_color = 'skyblue'
					2: n_color = 'lawngreen'
					3: n_color = 'goldenrod light'
					4: n_color = 'rosybrown light'
					5: n_color = 'slateblue'
					else: n_color = 'white'
				endcase
				color_array[*,ii] = n_color
			endelse
			;
			data_array[ii].image_no = lin_stuff[0,ii]
			data_array[ii].expt = lin_stuff[1,ii]
			data_array[ii].path  = lin_stuff[2,ii]
			;
	endfor
	;
	r_string = STRING(format='("Linearity Images for Night ",I8.8)', (*n_ptr).night_desig)
	if( (*s_ptr).default_font ne 'None') then begin
	 new_struct = Dialog_Big_Table(Title="Bias and Linearity Images",Label=r_string, $
					Group_Leader=Event.top, Col_Headers = col_headers, $
					Format=format, Row_headers = row_headers,$
					Data_array = data_array, Font='Helvetica*16', $
					color_array=color_array)
	endif else begin
	 new_struct = Dialog_Big_Table(Title="Bias and Linearity Images",Label=r_string, $
	    Group_Leader=Event.top, Col_Headers = col_headers, $
	    Format=format, Row_headers = row_headers,$
	    Data_array = data_array, Font='None', $
	    color_array=color_array)
	endelse
	;
	;	load new table data back into structure
	;
	nc = N_TAGS(new_struct)
	nr = N_ELEMENTS(new_struct)
	;
	;	turn on next button
	;
	P6_button_sensitize, Event, 5, 1
end
;
;------------------------------------------------------------
pro P6_Combine_Image_Matches, Event
	;
	MSP_BDP_print_text, Event, "Combine Images in Match Lists Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	b_ptr = PTR_struct.PTR_BPM
	;
	(*s_ptr).P6_LED_colors[*] = 'black'
	n_coadds = (*l_ptr).n_lin_expt + 1
	LED_scale = float(n_coadds) / float(N_ELEMENTS( (*s_ptr).P6_LED_colors ))
	if (LED_scale lt 1.) then LED_scale = 1.0
	(*l_ptr).LED_scale = LED_scale
	;
	this_LED = 0
	(*l_ptr).this_LED = this_LED
	(*s_ptr).P6_LED_colors[this_LED] = 'yellow'
	WCT_LED_stack, (*s_ptr).P6_LED_wid[0], (*s_ptr).P6_LED_colors
	;
	;	start by creating directory for storing coadded images
	;
	root_reduced = (*s_ptr).root_reduced
	this_lin = (*l_ptr).this_lin
	;
	;	night designation
	;
	night_desig = (*n_ptr).night_desig
	night_desig_text = STRING(format='(I8.8)',night_desig)
	;
	lin_directory = '6_Linearity_Tables'+PATH_SEP()+night_desig_text ; subdirectory of night desig
	coadd_subdirectory = "coadded_images_method_1"
	;
	lin_path = root_reduced + lin_directory + PATH_SEP() + coadd_subdirectory
	FILE_MKDIR, lin_path
	;
	(*l_ptr).lin_directory[this_lin] = fix_path( root_reduced + lin_directory )
	;
	MSP_BDP_print_text, Event, "Created directory "+lin_path
	;
	;	next, fetch the appropriate bad pixel mask, based on night and UT
	;	proximity
	;
	;	start by fetching BPM directory list
	;
	MSP_BDP_Read_BPM_Headers, Event
	;
	;	get night, UT for Linearity Data
	;
	this_lin = (*l_ptr).this_lin
	lin_night = (*l_ptr).lin_night[this_lin]
	lin_ut = (*l_ptr).lin_UT[this_lin]
	;
	n_bpm = (*b_ptr).bpm_n_BPM
	d_UT_min = 1.0e6
	jbpm = 0
	for ibpm = 0, n_bpm-1 do begin
		if( lin_night eq (*b_ptr).bpm_night[ibpm]) then begin
			d_UT = abs(lin_ut - (*b_ptr).bpm_UT[ibpm])
			if(d_UT lt d_UT_min) then begin
				d_UT_min = d_UT
				jbpm = ibpm
			endif
		endif
	endfor
	;
	;print, "Closest night-UT BPM match is number ",jbpm
	;print, "  smallest d_UT = ",d_UT_min
	;
	BPM_path = (*b_ptr).bpm_image_name[jbpm]
	(*l_ptr).BPM_path = BPM_path
	;
	;print, "BPM path = ",BPM_path
	;
	;---------------------------------------------------------
	;
	;	collect list of paths for bias image coaddition
	;
	n_bias_list = (*l_ptr).n_bias_list
	bias_list = (*l_ptr).bias_paths[0:n_bias_list-1]
	;
	;	and RAW1 list, as well
	;
	raw1_list = strarr(n_bias_list)
	for ib = 0, n_bias_list-1 do begin
		raw1_list[ib] = CDS_to_RAW1_pathname(bias_list[ib])
		;
		;print, "creating raw1 path = ",raw1_list[ib]
	endfor
	;
	;	start coadder
	;
	if(lin_night gt (*s_ptr).row513_date) then row513_flag = 0 else row513_flag = 1
	;
	;print, "lin_night, row513_flag = ",lin_night, row513_flag
	;
	outpath = fix_path( FILEPATH('Bias_coadd.fits',ROOT_DIR=lin_path) )
	;
	MSP_BDP_print_text, Event, "  CDS Coadding started."
	;
	LED_wid = (*s_ptr).P6_LED_wid[0]
	colors = (*s_ptr).P6_LED_colors
	coadd_median_with_bad_sub, Event, BPM_path, bias_list, outpath, $
								row513_flag, LED_wid, this_LED, colors
	;
	;	and RAW1 version
	;
	outpath = fix_path( FILEPATH('raw1_bias',ROOT_DIR=lin_path) )
	MSP_BDP_print_text, Event, "  RAW1 Coadding started."
	coadd_median_with_bad_RAW1_sub, Event, BPM_path, raw1_list, outpath, $
									row513_flag, LED_wid, this_LED, colors
	;
	this_LED = 0
	(*s_ptr).P6_LED_colors[this_LED] = 'green'
	WCT_LED_stack, (*s_ptr).P6_LED_wid[0], (*s_ptr).P6_LED_colors
	;---------------------------------------------------------------
	;	next, linearity files
	;
	n_lin_times = (*l_ptr).n_lin_expt
	for itime = 0, n_lin_times-1 do begin
		;
		;	LED set up
		;
		this_LED = (itime+1) / LED_scale
		this_LED = (0 > this_LED) < (N_ELEMENTS((*s_ptr).P6_LED_colors )-1)
		(*l_ptr).this_LED = this_LED
		(*s_ptr).P6_LED_colors[this_LED] = 'orange'
		WCT_LED_stack, (*s_ptr).P6_LED_wid[0], (*s_ptr).P6_LED_colors
		;
		;	get the time and convert to 10^6 multiplied text
		;
		expt = (*l_ptr).lin_expt[itime]
		expt_mil = long(1.0d6 * expt)
		expt_mil_text = STRING(format='(I8.8)',expt_mil)
		;
		;	create filenames for outputs
		;
		cds_file = 'Lin_'+expt_mil_text+".fits"
		raw1_file = 'raw1_'+expt_mil_text
		;
		MSP_BDP_print_text, Event, "Starting coadding for expt = "+STRING(format='(f10.4)',expt)+' seconds.'
		;
		;	and paths for same
		;
		cdspath = fix_path( FILEPATH(cds_file, ROOT_DIR=lin_path) )
		raw1path = fix_path( FILEPATH(raw1_file, ROOT_DIR=lin_path) )
		;
		;	load CDS list
		;
		n_lin_list = (*l_ptr).n_lin_paths[itime]
		lin_list = (*l_ptr).lin_paths[itime,0:n_lin_list-1]
		;
		;	and RAW1 list, as well
		;
		raw1_lin_list = strarr(n_lin_list)
		for ib = 0, n_lin_list-1 do begin
			raw1_lin_list[ib] = CDS_to_RAW1_pathname(lin_list[ib])
			;
			;print, "creating raw1 path = ",raw1_list[ib]
		endfor
		;
		;	start coadder
		;
		if(lin_night gt (*s_ptr).row513_date) then row513_flag = 0 else row513_flag = 1
		;
		;print, "cdspath, lin_night, row513_flag = ",cdspath, lin_night, row513_flag
		;
		MSP_BDP_print_text, Event, "  CDS Coadding started."
		;
		LED_wid = (*s_ptr).P6_LED_wid[0]
		colors = (*s_ptr).P6_LED_colors
		coadd_median_with_bad_sub, Event, BPM_path, lin_list, cdspath, $
									row513_flag, LED_wid, this_LED, colors
		;
		;	and RAW1 version
		MSP_BDP_print_text, Event, "  RAW1 Coadding started."
		;
		coadd_median_with_bad_RAW1_sub, Event, BPM_path, raw1_lin_list, raw1path, $
										row513_flag, LED_wid, this_LED, colors
		;
		(*s_ptr).P6_LED_colors[this_LED] = 'green'
		WCT_LED_stack, (*s_ptr).P6_LED_wid[0], (*s_ptr).P6_LED_colors
	endfor
	;
	;	turn on other buttons
	;
	for ib = 6, 9 do P6_button_sensitize, Event, ib, 1
	;
	;	and update the run summary that we have completed a major portion of this step
	;
	num_nights = (*s_ptr).num_nights
	ind = WHERE((*n_ptr).night_desig eq (*s_ptr).night_date[0:num_nights-1], count)
	if(count ne 0) then night_index = ind[0]
	(*s_ptr).night_step[night_index] = 6
	(*s_ptr).night_comment[night_index] = "Images Coadded"
	;
	P1_Write_Summary_File, Event
end
;
;------------------------------------------------------------------------------
;
;	METHOD 2 - Multiple Runs - Programs
;
;------------------------------------------------------------------------------
;-------------------------------------------------------------
pro P6_Make_Image_Matches_Method_2, Event
	;
	;	Method 2
	;
	MSP_BDP_print_text, Event, "Make Match Lists (Method 2) Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	b_ptr = PTR_struct.PTR_BPM
	;
	;	establish that first, last linearity and bias groups were identified
	;
	test_1 = ((*l_ptr).m2_lin_first_grp eq -1)
	test_2 = ((*l_ptr).m2_lin_last_grp eq -1)
	test_3 = ((*l_ptr).m2_bias_first_grp eq -1)
	test_4 = ((*l_ptr).m2_bias_last_grp eq -1)
	;
	if( test_1 or test_2 or test_3 or test_4 ) then begin
			;
			MSP_BDP_print_text, Event, "ERROR - Must select first and last linearity and bias groups. Retry selections"
			if(test_1) then MSP_BDP_print_text, Event, "  -> missing First Linearity Group"
			if(test_2) then MSP_BDP_print_text, Event, "  -> missing Last Linearity Group"
			if(test_3) then MSP_BDP_print_text, Event, "  -> missing First Bias Group"
			if(test_4) then MSP_BDP_print_text, Event, "  -> missing Last Bias Group"
			;
			goto, P6MIMM_end
	endif
	;
	;	compute number of linearity and bias groups to consider
	;
	;---------------------- linearity first --------------------------------
	WIDGET_CONTROL, (*s_ptr).P6_droplist_wid[3], GET_VALUE=lin_grp_list
	n_lin_grp_list = N_ELEMENTS(lin_grp_list)
	selected_lin_grp_list = lonarr(n_lin_grp_list)
	;
	i_lin_first_grp = (*l_ptr).m2_lin_first_grp
	i_lin_last_grp = (*l_ptr).m2_lin_last_grp
	;
	ilin = 0L
	for ig = 0, n_lin_grp_list-1 do begin
		;
		space = STRPOS(lin_grp_list[ig],' ')
		;
		num = STRMID(lin_grp_list[ig], 4, (space-4))
		READS,num, format='(I)',ID
		;
		if( (ID ge i_lin_first_grp) and (ID le i_lin_last_grp)) then begin
			selected_lin_grp_list[ilin] = ID
			ilin++
		endif
	endfor
	;
	(*l_ptr).m2_n_lin_grps = ilin
	if(ilin gt 0) then begin
		(*l_ptr).m2_lin_grps[0:ilin-1] = selected_lin_grp_list[0:ilin-1]
		max_lin_grps = N_ELEMENTS((*l_ptr).m2_lin_grps)
		if(ilin lt max_lin_grps-1) then (*l_ptr).m2_lin_grps[ilin:max_lin_grps-1] = -1
	endif
	;
	MSP_BDP_print_text, Event, "Found "+STRING(format='(i5.1)',ilin)+" linearity groups to process"
	;---------------------- bias second -----------------------------------------
	WIDGET_CONTROL, (*s_ptr).P6_droplist_wid[5], GET_VALUE=bias_grp_list
	n_bias_grp_list = N_ELEMENTS(bias_grp_list)
	selected_bias_grp_list = lonarr(n_bias_grp_list)
	;
	i_bias_first_grp = (*l_ptr).m2_bias_first_grp
	i_bias_last_grp = (*l_ptr).m2_bias_last_grp
	;
	ibias = 0L
	for ig = 0, n_bias_grp_list-1 do begin
		;
		space = STRPOS(bias_grp_list[ig],' ')
		;
		num = STRMID(bias_grp_list[ig], 4, (space-4))
		READS,num, format='(I)',ID
		;
		if( (ID ge i_bias_first_grp) and (ID le i_bias_last_grp)) then begin
			selected_bias_grp_list[ibias] = ID
			ibias++
		endif
	endfor
	;
	(*l_ptr).m2_n_bias_grps = ibias
	if(ibias gt 0) then begin
		(*l_ptr).m2_bias_grps[0:ibias-1] = selected_bias_grp_list[0:ibias-1]
		max_bias_grps = N_ELEMENTS((*l_ptr).m2_bias_grps)
		if(ibias lt max_bias_grps-1) then (*l_ptr).m2_bias_grps[ibias:max_bias_grps-1] = -1
	endif
	;
	MSP_BDP_print_text, Event, "Found "+STRING(format='(i5.1)',ibias)+" bias groups to process"
	;--------------------------------------------------------------------------------------------
	;
	;	for every linearity group, fetch the integration time of every image, add it to the list
	;
	n_all_lin_images = 0L
	for ig = 0, ilin-1 do begin
		this_group = (*l_ptr).m2_lin_grps[ig]
		;
		n_this_group_images = (*n_ptr).groups[this_group].group_images
		n_all_lin_images = n_all_lin_images + n_this_group_images
	endfor
	;
	lin_exp_times = fltarr(n_all_lin_images)
	lin_CDS_paths = strarr(n_all_lin_images)
	lin_RAW1_paths = strarr(n_all_lin_images)
	;
	n_night_max_images = (*n_ptr).num_images
	i_all = 0L
	for ig = 0, ilin-1 do begin
		this_group = (*l_ptr).m2_lin_grps[ig]
		;
		n_this_group_images = (*n_ptr).groups[this_group].group_images
		;
		for ii = 0, n_this_group_images-1 do begin
			;
			;	get image number from group list
			;
			image_number = (*n_ptr).groups[this_group].group_list[ii]
			;
			;	and match it to the night's image list
			;
			n_image = WHERE(image_number eq (*n_ptr).image_list[0:n_night_max_images-1],count)
			if(count ne 1) then begin
				MSP_BDP_print_text, Event, "ERROR - no image match in M2 linearity matching. Skipping"
				goto, P6MIMM_end
			endif
			n_image_index = n_image[0]		; image index, not number now..
			;
			;	fetch exposure time for this image
			;
			lin_exp_times[i_all] = (*n_ptr).image_exptime[n_image_index]
			;
			;	and full pathnames to the CDS and RAW1 images
			;
			lin_CDS_paths[i_all] = get_CDS_pathname(Event, n_image_index)
			lin_RAW1_paths[i_all] = get_RAW1_pathname(Event, n_image_index)
			;
			i_all++
		endfor
	endfor
	;
	;	full lists built, get sort indices
	;
	expt_sort_index = SORT(lin_exp_times)					; the full sort indices
	expt_uniq_index = UNIQ(lin_exp_times[expt_sort_index])	; the indices of unique values, pointing to last of each group
	n_subgroups = N_ELEMENTS(expt_uniq_index)				; the number of groups
	;
	(*l_ptr).m2_n_exp_times = n_subgroups
	;
	;	sort into subgroups, then image pathname sets
	;
	ilast = 0L
	for isg = 0, n_subgroups-1 do begin
		first = ilast
		last = expt_uniq_index[isg]
		;
		(*l_ptr).m2_exp_times[isg] = lin_exp_times[expt_sort_index[expt_uniq_index[isg]]]
		;
		n_this_exp_time = last - first + 1
		(*l_ptr).m2_exp_n_paths[isg] = n_this_exp_time
		;
;		print, "For exposure time = ",(*l_ptr).m2_exp_times[isg]," CDS file pathnames are:"
		;
		ik0 = first
		ik1 = first + n_this_exp_time-1
		;
		CDS_short = lin_CDS_paths[expt_sort_index[ik0:ik1]]
		RAW1_short = lin_RAW1_paths[expt_sort_index[ik0:ik1]]
		CDS_file = FILE_BASENAME(CDS_short)
		RAW1_file = FILE_BASENAME(RAW1_short)
		;
		(*l_ptr).m2_exp_CDS_pathnames[isg, 0:(n_this_exp_time-1)] = CDS_short(SORT(CDS_file))
		(*l_ptr).m2_exp_RAW1_pathnames[isg, 0:(n_this_exp_time-1)] = RAW1_short(SORT(RAW1_file))
		;
;		for ii = ik0, ik1 do begin
;			ik = ii - ik0
;			print, 'ii=', ii,'  ',(*l_ptr).m2_exp_CDS_pathnames[isg, ik]
;		endfor
        ;
        ilast = last+1
		;
	endfor
	;
	MSP_BDP_print_text, Event, "Finished sorting linearity images into "+STRING(format='(I5.1)',n_subgroups)+" subgroups"
	;
	;----------------------- linearity list built ---------------------------------------
	;
	;	build bias list
	;
	;		for each bias group, determine coadd numbers, starting UT, pathnames
	;
	bias_UT_times = dblarr(200)
	;
	for ig = 0, (*l_ptr).m2_n_bias_grps-1 do begin
		this_group = (*l_ptr).m2_bias_grps[ig]
		;
		n_this_group_images = (*n_ptr).groups[this_group].group_images
		;
		(*l_ptr).m2_bias_n_paths[ig] = n_this_group_images
		;
		for ii = 0, n_this_group_images-1 do begin
			;
			image_number = (*n_ptr).groups[this_group].group_list[ii]
			;
			;	and match it to the night's image list
			;
			n_image = WHERE(image_number eq (*n_ptr).image_list[0:n_night_max_images-1],count)
			if(count ne 1) then begin
				MSP_BDP_print_text, Event, "ERROR - no image match in M2 bias matching. Skipping"
				goto, P6MIMM_end
			endif
			n_image_index = n_image[0]		; image index, not number now..
			;
			;	fetch UT for this image
			;
			bias_UT_times[ii] = (*n_ptr).header_list[n_image_index].ut
			;
			;	and full pathnames to the CDS and RAW1 images
			;
			(*l_ptr).m2_bias_CDS_pathnames[ig, ii] = get_CDS_pathname(Event, n_image_index)
			(*l_ptr).m2_bias_RAW1_pathnames[ig, ii] = get_RAW1_pathname(Event, n_image_index)
			;
		endfor
		;
		;	compute average UT for this group
		;
		ut_moment = MOMENT(bias_UT_times[0:n_this_group_images-1])
		(*l_ptr).m2_bias_ut_times[ig] = ut_moment[0]
		;
	endfor
	;
	MSP_BDP_print_text, Event, "Finished collecting bias group information for "+STRING(format='(I5.1)',(*l_ptr).m2_n_bias_grps)+' subgroups'
	;
	;	turn on button to allow viewing these new groups and subgroups
	;
	P6_button_sensitize, Event, 12, 1
	;
	P6MIMM_end:
end
;
;-------------------------------------------------------------
pro P6_View_Image_Match_Table_Method_2, Event
	;
	;	put up table of matches and biases
	;
	MSP_BDP_print_text, Event, "View Image Match Lists (M2) Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	;
	;	build the table structure
	;
	col_headers = ["SubGroup","Time (UT or ExpT)","N_Images","_____Filename of First Image_____"]
	;
	;	number of lines = n_bias + n_lin
	;	load all the info into one big string array
	;
	n_bias = (*l_ptr).m2_n_bias_grps
	n_lin = (*l_ptr).m2_n_exp_times
	;
	n_lines = n_bias + n_lin
	n_cols = N_ELEMENTS(col_headers)
	;
	format = strarr(n_cols,n_lines)
	format[0:n_cols-1,*] = '(a)'
	;
	color_array = strarr(n_cols,n_lines)
	;
	row_headers = strarr(n_lines)
	data_line = {subgroup: '', time: '', nimages: '', path: ''}
	data_array = REPLICATE(data_line, n_lines)			; array of structures
	;
	for ii = 0, n_lines-1 do begin
			;
			;	load structure for this image
			;
			if(ii lt n_bias) then begin
				row_headers[ii] = STRING(format='("B_",i3.3)',ii)
				color_array[*,ii] = 'yellow'
				;
				data_array[ii].subgroup = 'Bias_Grp_' + STRTRIM(STRING(format='(i5.3)',ii),2)
				data_array[ii].time = STRTRIM(STRING(format='(f12.6)',(*l_ptr).m2_bias_ut_times[ii]),2)+' UT'
				data_array[ii].nimages = STRTRIM(STRING(format='(i5.1)',(*l_ptr).m2_bias_n_paths[ii]),2)
				data_array[ii].path = FILE_BASENAME((*l_ptr).m2_bias_CDS_pathnames[ii,0])
			endif else begin
				ig = ii - n_bias
				;
				row_headers[ii] = STRING(format='("Lin_",i3.3)',ig)
				color_array[*,ii] = 'green'
				;
				data_array[ii].subgroup = 'Lin_Grp_' + STRTRIM(STRING(format='(i5.3)',ig),2)
				data_array[ii].time = STRTRIM(STRING(format='(f12.6)',(*l_ptr).m2_exp_times[ig]),2)
				data_array[ii].nimages = STRTRIM(STRING(format='(i5.1)',(*l_ptr).m2_exp_n_paths[ig]),2)
				data_array[ii].path = FILE_BASENAME((*l_ptr).m2_exp_CDS_pathnames[ig,0])
			endelse

			;
	endfor
	;
	r_string = STRING(format='("Linearity Images for Night ",I8.8)', (*n_ptr).night_desig)
	if( (*s_ptr).default_font ne 'None') then begin
	 new_struct = Dialog_Big_Table(Title="Bias and Linearity Images",Label=r_string, $
					Group_Leader=Event.top, Col_Headers = col_headers, $
					Format=format, Row_headers = row_headers,$
					Data_array = data_array, Font='Helvetica*16', $
					color_array=color_array)
	endif else begin
	 new_struct = Dialog_Big_Table(Title="Bias and Linearity Images",Label=r_string, $
	    Group_Leader=Event.top, Col_Headers = col_headers, $
	    Format=format, Row_headers = row_headers,$
	    Data_array = data_array, Font='None', $
	    color_array=color_array)
	endelse
	;
	;	load new table data back into structure
	;
	nc = N_TAGS(new_struct)
	nr = N_ELEMENTS(new_struct)
	;
	;	turn on next button
	;
	P6_button_sensitize, Event, 13, 1
	;
end
;
;------------------------------------------------------------
pro P6_Combine_Image_Matches_Method_2, Event
	;
	MSP_BDP_print_text, Event, "Combine Image Matches (M2) Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	b_ptr = PTR_struct.PTR_BPM
	f_ptr = PTR_struct.PTR_FITS
	;
	;	set up LED progress bar
	;
	n_subgroups = (*l_ptr).m2_n_bias_grps + (*l_ptr).m2_n_exp_times
	n_LEDs = N_ELEMENTS((*s_ptr).P6_LED_colors)
	LED_scale = float(n_subgroups) / float(n_LEDs)
	if(LED_scale lt 1.0) then LED_scale = 1.0
	(*l_ptr).LED_scale = LED_scale
	;
	(*s_ptr).P6_LED_colors[*] = 'yellow'
	if(n_subgroups lt n_LEDs) then (*s_ptr).P6_LED_colors[n_subgroups:n_LEDs-1] = 'black'
	WCT_LED_stack, (*s_ptr).P6_LED_wid[3], (*s_ptr).P6_LED_colors
	;
	;	start by coadding bias images, saving raw1 averages in FITS files on
	;	"coadded_images_method_2" directory
	;
	root_reduced = (*s_ptr).root_reduced
	;
	this_lin = (*l_ptr).this_lin
	;
	;	fetch BPM info
	;
	MSP_BDP_Read_BPM_Headers, Event
	;
	for ib = 0, (*l_ptr).m2_n_bias_grps-1 do begin
		;
		this_LED = ROUND(float(ib) / LED_scale)
		this_LED = 0 > ((n_LEDs-1) < (this_LED))
		(*s_ptr).P6_LED_colors[this_LED] = 'blue'
		WCT_LED_stack, (*s_ptr).P6_LED_wid[3], (*s_ptr).P6_LED_colors
		;
		;	load the images and coadd the RAW1s
		;
		;	start by creating directory for storing coadded images
		;
		;	night designation
		;
		night_desig = (*n_ptr).night_desig
		night_desig_text = STRING(format='(I8.8)',night_desig)
		;
		lin_directory = '6_Linearity_Tables'+PATH_SEP()+night_desig_text ; subdirectory of night desig
		coadd_subdirectory = "coadded_images_method_2"
		;
		lin_path = fix_path( root_reduced + lin_directory + PATH_SEP() + coadd_subdirectory )
		test = FILE_INFO(lin_path)
		if(NOT(test.exists) or NOT(test.directory)) then begin
			FILE_MKDIR, lin_path
			MSP_BDP_print_text, Event, "Created directory "+lin_path
		endif
		;
		(*l_ptr).lin_directory[this_lin] = root_reduced + lin_directory
		;
		;	next, fetch the appropriate bad pixel mask, based on night and UT
		;	proximity
		;
		;	start by fetching BPM directory list
		;
		;MSP_BDP_Read_BPM_Headers, Event
		;
		;	get night, UT for Bias Data
		;
		this_lin = (*l_ptr).this_lin
		lin_night = (*l_ptr).lin_night[this_lin]
		;
		;	mean UT of bias images in this subgroup
		;
		lin_ut = (*l_ptr).m2_bias_ut_times[ib]
		;
		n_bpm = (*b_ptr).bpm_n_BPM
		d_UT_min = 1.0e6
		jbpm = 0
		for ibpm = 0, n_bpm-1 do begin
			if( lin_night eq (*b_ptr).bpm_night[ibpm]) then begin
				d_UT = abs(lin_ut - (*b_ptr).bpm_UT[ibpm])
				if(d_UT lt d_UT_min) then begin
					d_UT_min = d_UT
					jbpm = ibpm
				endif
			endif
		endfor
		;
		BPM_path = (*b_ptr).bpm_image_name[jbpm]
		(*l_ptr).BPM_path = BPM_path
		;
		print, "For Bias Coadding - Closest night-UT BPM match is number ",jbpm," for BPM file = ",$
					FILE_BASENAME(BPM_path)
		print, "  smallest d_UT = ",d_UT_min
		;
		;print, "BPM path = ",BPM_path
		;
		;---------------------------------------------------------
		;
		;	collect list of paths for bias image coaddition
		;
		n_bias_list = (*l_ptr).m2_bias_n_paths[ib]
		raw1_list = (*l_ptr).m2_bias_RAW1_pathnames[ib,0:n_bias_list-1]
		;
		;	check for any duds
		;
		ind = WHERE(raw1_list eq '',count)
		if(count ne 0) then begin
			print, "Found a blank filename on raw1_list at position ",ind
			print, "  n_bias_list = ",n_bias_list
			for inb = 0, n_bias_list-1 do begin
				print, "  raw1 file = ",FILE_BASENAME(raw1_list[inb])
			endfor
			;
			;	throw and error to halt here with debug
			;
			a = b / c
		endif
		;
		;	start RAW1 coadder
		;
		if(lin_night gt (*s_ptr).row513_date) then row513_flag = 0 else row513_flag = 1
		;
		;print, "lin_night, row513_flag = ",lin_night, row513_flag
		;
		;outpath = FILEPATH('Bias_RAW1_coadd_'+STRTRIM(STRING(format='(I3.3)',ib),2)+'.fits',ROOT_DIR=lin_path)
		temp_path = fix_path( FILEPATH('temp_raw1',ROOT_DIR=lin_path) )
		;
		MSP_BDP_print_text, Event, "Bias RAW1 Image Coadding started."
		;
		LED_wid = (*s_ptr).P6_LED_wid[3]
		colors = (*s_ptr).P6_LED_colors
		coadd_median_with_bad_RAW1_sub, Event, BPM_path, raw1_list, temp_path, $
								row513_flag, LED_wid, this_LED, colors
		;
		;	read in the image
		;
		bias_image = read_raw1(temp_path)
		;
		;	and the FITS header for the first image
		;
		bias_header = HEADFITS((*l_ptr).m2_bias_CDS_pathnames[ib,0])
		;
		outpath = fix_path( FILEPATH('Bias_RAW1_coadd_'+STRTRIM(STRING(format='(I3.3)',ib),2)+'.fits',ROOT_DIR=lin_path) )
		;
		;	add JD to this header
		;
		image_JD = get_image_JD(Event, (*l_ptr).m2_bias_CDS_pathnames[ib,0])
		;
		FXADDPAR, bias_header, "JD", image_jd, "MSP-BDP Part 6 computed Julian date", FORMAT='(f20.7)'
		FXADDPAR, bias_header, "FILENAME", FILE_BASENAME(outpath),"MSP-BDP created filename"
		;
		;	also add the list of images in the coadd
		;
		FXADDPAR, bias_header, "HISTORY", "Coadded RAW1 Bias images on "+SYSTIME()
		FXADDPAR, bias_header, "HISTORY", "  Reduced by "+(*s_ptr).who
		for inb = 0, n_bias_list-1 do begin
			FXADDPAR, bias_header, "HISTORY", "  input Bias RAW1 file = "+FILE_BASENAME(raw1_list[inb])
		endfor
		;
		;	and write it out
		;
		writefits, outpath, bias_image, bias_header
		;
		MSP_BDP_print_text, Event, "Wrote Image "+FILE_BASENAME(outpath)+" to disk"
		;
		;	and delete the temp file
		;
		FILE_DELETE, temp_path
		;
		(*s_ptr).P6_LED_colors[this_LED] = 'green'
		WCT_LED_stack, (*s_ptr).P6_LED_wid[3], (*s_ptr).P6_LED_colors
		;
		;	save info in structure
		;
		(*l_ptr).m2_bias_coadd_pathnames[ib] = outpath
		(*l_ptr).m2_bias_coadd_JD[ib] = image_JD
		;
	endfor
	(*l_ptr).m2_bias_n_coadd = (*l_ptr).m2_n_bias_grps
	;
	MSP_BDP_print_text, Event, "Finished coadding "+STRING(format='(I5.1)',(*l_ptr).m2_n_bias_grps)+" bias groups"
	;
	;	then, go through each linearity subgroup, building images for coadding, and do the
	;	coadding
	;
	night_desig = (*n_ptr).night_desig
	night_desig_text = STRING(format='(I8.8)',night_desig)
	;
	for isg = 0, (*l_ptr).m2_n_exp_times-1 do begin
		;
		this_LED = ROUND(float((*l_ptr).m2_n_bias_grps+isg) / LED_scale)
		this_LED = 0 > ((n_LEDs-1) < (this_LED))
		(*s_ptr).P6_LED_colors[this_LED] = 'blue'
		WCT_LED_stack, (*s_ptr).P6_LED_wid[3], (*s_ptr).P6_LED_colors
		;
		;	number of images in this subgroup
		;
		n_sg_images = (*l_ptr).m2_exp_n_paths[isg]
		;
		;	pick one BPM for the subgroup
		;
		image_JD = get_image_JD(Event, (*l_ptr).m2_exp_CDS_pathnames[isg,0])
		BPM_path = get_closest_BPM(Event, image_JD)
		BPM_image = readfits(BPM_path, BPM_header, /silent)
		;
		print, "For sub-group ",isg," first image JD = ",image_JD," matching to BPM_image = ",FILE_BASENAME(BPM_path)
		;
		ib0_open = -2L
		ib1_open = -2L
		;
		temp_paths = strarr(n_sg_images)
		;
		for ii = 0, n_sg_images-1 do begin
			;
			;	for each image, read in CDS and RAW1 images
			;
			CDS_image = double(readfits( (*l_ptr).m2_exp_CDS_pathnames[isg,ii],CDS_header,/silent))
			RAW1_image = double(read_raw1( (*l_ptr).m2_exp_RAW1_pathnames[isg,ii]))
			;
			;	get JD
			;
			CDS_JD = get_image_JD(Event, (*l_ptr).m2_exp_CDS_pathnames[isg,ii])
			;
			print, " for image ",ii," CDS file = ",FILE_BASENAME((*l_ptr).m2_exp_CDS_pathnames[isg,ii])," with JD = ",$
					CDS_JD
			print, "                   RAW1 file = ",FILE_BASENAME((*l_ptr).m2_exp_RAW1_pathnames[isg,ii])
			;
			;	compare to JDs of biases - find closest two that bracket the observation
			;
			low_index = WHERE(CDS_JD gt (*l_ptr).m2_bias_coadd_JD[0:(*l_ptr).m2_bias_n_coadd-1],count)
			if(count gt 0) then begin
				d_low_JD = abs(CDS_JD - (*l_ptr).m2_bias_coadd_JD[low_index])
				min_low_index = WHERE(MIN(d_low_JD) eq d_low_JD,count)
				if(count gt 0) then begin
					min_low_index = min_low_index[0]
					low_bias_index = low_index[min_low_index]
				endif else begin
					low_bias_index = -1
				endelse
			endif else begin
				low_bias_index = -1
			endelse
			;
			print," best match low_bias_index = ",low_bias_index
			;
			high_index = WHERE(CDS_JD lt (*l_ptr).m2_bias_coadd_JD[0:(*l_ptr).m2_bias_n_coadd-1],count)
			if(count gt 0) then begin
				d_high_JD = abs(CDS_JD - (*l_ptr).m2_bias_coadd_JD[high_index])
				min_high_index = WHERE(MIN(d_high_JD) eq d_high_JD,count)
				if(count gt 0) then begin
					min_high_index = min_high_index[0]
					high_bias_index = high_index[min_high_index]
				endif else begin
					high_bias_index = -1
				endelse
			endif else begin
				high_bias_index = -1
			endelse
			;
			print," best match high_bias_index = ",high_bias_index
			;
			;	check to see if these are currently in memory
			;
			if((ib0_open ne low_bias_index) and (low_bias_index ne -1)) then begin
				ib0_image = readfits((*l_ptr).m2_bias_coadd_pathnames[low_bias_index],ib0_header, /silent)
				ib0_open = low_bias_index
				ib0_JD = (*l_ptr).m2_bias_coadd_JD[low_bias_index]
				;
				print,"  loaded new low bias image ",FILE_BASENAME((*l_ptr).m2_bias_coadd_pathnames[low_bias_index]),$
						" with JD = ",ib0_JD
				;
			endif else begin
				if(low_bias_index eq -1) then ib0_JD = -1.0
			endelse
			;
			if((ib1_open ne high_bias_index) and (high_bias_index ne -1)) then begin
				ib1_image = readfits((*l_ptr).m2_bias_coadd_pathnames[high_bias_index],ib1_header, /silent)
				ib1_open = high_bias_index
				ib1_JD = (*l_ptr).m2_bias_coadd_JD[high_bias_index]
				;
				print,"  loaded new high bias image ",FILE_BASENAME((*l_ptr).m2_bias_coadd_pathnames[high_bias_index]),$
						" with JD = ",ib1_JD
				;
			endif else begin
				if(high_bias_index eq -1) then ib1_JD = -1.0
			endelse
			;
			;	interpolation factor
			;
			if((ib0_JD ne -1.0) and (ib1_JD ne -1.0)) then begin
				dJD = double(ib1_JD - ib0_JD)
				;
				;	check for zero JD difference - would be a problem
				;
				if(dJD eq 0.0) then begin
					MSP_BDP_print_text, Event, "ERROR - zero JD difference found"
					print, "ib0_JD, ib1_JD = ",ib0_JD, ib1_JD
					;
					dJD = (1.0 / 60.0)/ 24.0		; one minute difference forced
				endif
				xJD = double(CDS_JD - ib0_JD)
				;
				xb1 = xJD / dJD						; 1 - x so that small delta = high interpol factor
				xb0 = 1.0d0 - xb1
			endif else begin
				xb0 = 0.0
				xb1 = 0.0
				if(ib0_JD ne -1.0) then begin
					xb0 = 1.0
				endif
				if(ib1_JD ne -1.0) then begin
					xb1 = 1.0
				endif
			endelse
			;
			print, "image ",ii," interpolation factors = ",xb0,xb1
			;
			;	subtract interpolated bias raw1 values to R2 image
			;
			if(xb0 ne 0.0) then scaled_b0_image = safe_math(ib0_image, xb0, BPM_image, '*')
			if(xb1 ne 0.0) then scaled_b1_image = safe_math(ib1_image, xb1, BPM_image, '*')
			;
			if(xb0 ne 0.0 and xb1 ne 0.0) then begin
				bias_raw1_image = safe_math(scaled_b0_image, scaled_b1_image, BPM_image, '+')
			endif else begin
				bias_raw1_image = 0.0
				if(xb0 ne 0.0) then begin
					bias_raw1_image = ib0_image
				endif
				if(xb1 ne 0.0) then begin
					bias_raw1_image = ib1_image
				endif
				n_pix = N_ELEMENTS(bias_raw1_image)
				if(n_pix eq 1) then begin
					MSP_BDP_print_text, Event, "ERROR - no bias images bracket this linearity image. "
					print, "found no bracketing bias images - using zeroed image"
					bias_raw_image = 0.0 * BPM_image		; a zero filled image
				endif
			endelse
			;
			;	do the math
			;
			temp_image = safe_math(CDS_image,RAW1_image,BPM_image,'-')		; minus because CDS = D1 - D2
			;
			D2_RR_image = safe_math(temp_image, bias_raw1_image, BPM_image, '+')	; to make B1 - D2 = RR - D2
			;
			;	write this to disk, for later coaddition
			;
			lin_directory = '6_Linearity_Tables'+PATH_SEP()+night_desig_text ; subdirectory of night desig
			coadd_subdirectory = "coadded_images_method_2"
			;
			lin_path = fix_path( root_reduced + lin_directory + PATH_SEP() + coadd_subdirectory )
			test = FILE_INFO(lin_path)
			if(NOT(test.exists) or NOT(test.directory)) then begin
				FILE_MKDIR, lin_path
				MSP_BDP_print_text, Event, "Created directory "+lin_path
			endif
			;
			temp_paths[ii] = fix_path( FILEPATH('temp_D2_RR_'+STRTRIM(STRING(format='(I3.3)',ii),2)+'.fits',$
					ROOT_DIR=lin_path) )
			;
			writefits, temp_paths[ii], D2_RR_image, CDS_header
		endfor
		;
		(*s_ptr).P6_LED_colors[this_LED] = 'violet'
		WCT_LED_stack, (*s_ptr).P6_LED_wid[3], (*s_ptr).P6_LED_colors
		;
		;	perform coaddition for this subgroup
		;
		if(lin_night gt (*s_ptr).row513_date) then row513_flag = 0 else row513_flag = 1
		;
		;	get the time and convert to 10^6 multiplied text
		;
		n_header = N_ELEMENTS(CDS_header)
		(*s_ptr).raw_header[0:n_header-1] = CDS_header
		MSP_BDP_Parse_FITS_Header, Event
		;
		expt = (*f_ptr).exptime
		expt_mil = long(1.0d6 * expt)
		expt_mil_text = STRING(format='(I8.8)',expt_mil)
		;
		;	create filenames for outputs
		;
		cds_file = 'Lin_'+expt_mil_text+".fits"
		;
		outpath = fix_path( FILEPATH(cds_file,ROOT_DIR=lin_path) )
		;
		MSP_BDP_print_text, Event, "Coadding of "+cds_file+" started."
		;
		LED_wid = (*s_ptr).P6_LED_wid[3]
		colors = (*s_ptr).P6_LED_colors
		coadd_median_with_bad_sub, Event, BPM_path, temp_paths, outpath, $
								row513_flag, LED_wid, this_LED, colors
		;
		;	delete temp images
		;
		FILE_DELETE, temp_paths
		;
		(*s_ptr).P6_LED_colors[this_LED] = 'green'
		WCT_LED_stack, (*s_ptr).P6_LED_wid[3], (*s_ptr).P6_LED_colors
	endfor
	;
	MSP_BDP_print_text, Event, "Finished coadding all Linearity Images"
	;
	;	turn on other buttons
	;
	for ib = 6, 9 do P6_button_sensitize, Event, ib, 1
	;
	;	and update the run summary that we have completed a major portion of this step
	;
	num_nights = (*s_ptr).num_nights
	ind = WHERE((*n_ptr).night_desig eq (*s_ptr).night_date[0:num_nights-1], count)
	if(count ne 0) then night_index = ind[0]
	(*s_ptr).night_step[night_index] = 6
	(*s_ptr).night_comment[night_index] = "Images Coadded"
	;
	P1_Write_Summary_File, Event
end
;
;---------------------------------------------------------------------------------
;
;	Programs common to both Methods (to be run AFTER Method 1 or Method 2)
;
;---------------------------------------------------------------------------------
;------------------------------------------------------------
pro P6_Read_Directory, Event
	;
	MSP_BDP_print_text, Event, "Read Directories of Coadded Linearity Images Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	b_ptr = PTR_struct.PTR_BPM
	;
	;	there are two possible directories - method 1 and method 2
	;
	;	for method 1, need to do some math along the way
	;	for method 2, don't need new math
	;
	;	get the directory
	;
	root_reduced = (*s_ptr).root_reduced
	this_lin = (*l_ptr).this_lin
	;
	night_desig = (*n_ptr).night_desig
	night_desig_text = STRING(format='(I8.8)',night_desig)
	;
	lin_directory = '6_Linearity_Tables' + PATH_SEP() + night_desig_text
	coadd_sub_method_1 = "coadded_images_method_1"
	coadd_sub_method_2 = "coadded_images_method_2"
	;
	lin_path_1 = root_reduced + lin_directory + PATH_SEP() + coadd_sub_method_1
	lin_path_2 = root_reduced + lin_directory + PATH_SEP() + coadd_sub_method_2
	;
	(*l_ptr).lin_directory[this_lin] = fix_path( root_reduced + lin_directory )
	;
	MSP_BDP_print_text, Event, "Examining directory "+lin_path_1
	;
	;	do a filesearch on fits files
	;
	dir_list_1 = FILE_SEARCH(lin_path_1,'*.fits', COUNT=n_dir_list_1)
	if(n_dir_list_1 ne 0) then begin
		MSP_BDP_print_text, Event, "Found "+STRING(format='(I)',n_dir_list_1)+" coadded images in method 1 directory"
		method_1 = 1
	endif else begin
		MSP_BDP_print_text, Event, "Warning - method 1 coadding image directory was empty."
		method_1 = 0
	endelse
	;
	MSP_BDP_print_text, Event, "Examining directory "+lin_path_2
	;
	dir_list_2 = FILE_SEARCH(lin_path_2,'Lin_*.fits', COUNT=n_dir_list_2)
	if(n_dir_list_2 ne 0) then begin
		MSP_BDP_print_text, Event, "Found "+STRING(format='(I)',n_dir_list_2)+" coadded images in method 2 directory"
		method_2 = 1
	endif else begin
		MSP_BDP_print_text, Event, "Warning - method 2 coadding image directory was empty."
		method_2 = 0
	endelse
	;
	;	test for a viable method
	;
	if( method_1 ) then begin
		if( NOT(method_2) ) then begin
			(*l_ptr).method = 1
			dir_list = dir_list_1
			n_dir_list = n_dir_list_1
		endif else begin
			(*l_ptr).method = -1
			MSP_BDP_print_text, Event, "ERROR - linearity coadds for BOTH methods present - fix this, then retry."
		endelse
	endif else begin
		if( method_2) then begin
			(*l_ptr).method = 2
			dir_list = dir_list_2
			n_dir_list = n_dir_list_2
		endif else begin
			(*l_ptr).method = -1
			MSP_BDP_print_text, Event, "ERROR - no linearity coadds exist for either method - must coadd first."
		endelse
	endelse
	;
	if((*l_ptr).method eq -1) then goto, list_end
	;
	;	save the list in the linearity structure
	;
	ind_no_bias = WHERE(FILE_BASENAME(dir_list[*]) ne 'Bias_coadd.fits',count)
	if((*l_ptr).method eq 1) then begin
		ind_bias = WHERE(FILE_BASENAME(dir_list[*]) eq 'Bias_coadd.fits',count)
		if(count ne 1) then begin
			MSP_BDP_print_text, Event, "ERROR - Bias_coadd.fits not found"
			goto, list_end
		endif
	endif
	(*l_ptr).n_coadd = n_dir_list
	if((*l_ptr).method eq 1) then (*l_ptr).n_coadd--		; to account for bias
	(*l_ptr).coadd_path_list = dir_list[ind_no_bias]
	if((*l_ptr).method eq 1) then (*l_ptr).coadd_path_bias = dir_list[ind_bias]
	;
	MSP_BDP_print_text, Event, "Directory successfully read."
	;
	; turn on button below this one
	;
	P6_button_sensitize, Event, 7, 1
	;
	list_end:
end
;
;------------------------------------------------------------
pro P6_View_Coadded_Images, Event
	;
	;	show coadding details
	;
	MSP_BDP_print_text, Event, "List Images Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	b_ptr = PTR_struct.PTR_BPM
	;
	;	get the list
	;
	n_dir_list = (*l_ptr).n_coadd 		; to account for bias
	lin_path_list = (*l_ptr).coadd_path_list[0:n_dir_list-1]
	if ((*l_ptr).method eq 1) then bias_path_list = (*l_ptr).coadd_path_bias
	;
	;
	;	build the table structure
	;
	col_headers = ["__Image__", "___Filename___", "___ExpTime___"]
	;
	n_lines = n_dir_list
	if((*l_ptr).method eq 1) then n_lines++		; to account for the bias image
	format = strarr(3,n_lines)
	format[0,*] = '(a)'
	format[1,*] = '(a)'				; UT converted to string
	format[2,*] = '(a)'
	;
	color_array = strarr(3,n_lines)
	;
	row_headers = strarr(n_lines)
	data_line = {image_type: '', filename: '', expt: ''}
	data_array = REPLICATE(data_line, n_lines)			; array of structures
	;
	for i = 0, n_lines-1 do begin
			;
			if((i eq 0) and ((*l_ptr).method eq 1)) then begin
				filename = FILE_BASENAME(bias_path_list)
				data_array[i].filename=filename
				data_array[i].image_type="Bias"
				data_array[i].expt = '0.000000'
				row_headers[i] = 'Bias'
				color_array[*,i] = 'orange'
			endif else begin
				if((*l_ptr).method eq 1) then i1 = (i-1) else i1=i
				;
				filename = FILE_BASENAME(lin_path_list[i1])
				;
				expt_text = STRMID(filename, 4, 8)
				expt_long = 0L
				reads, expt_text, expt_long
				expt = double(expt_long) / 1.0d6
				(*l_ptr).lin_expt[i1] = expt
				row_headers[i] = 'Lin_'+STRING(format='(I3.1)',i1)
				data_array[i].filename=filename
				data_array[i].image_type='Lin.'
				data_array[i].expt = expt
				color_array[*,i] = 'yellow'
			endelse
			;
	endfor
	;
	r_string = STRING(format='("Linearity Coadd Image List for Night ",I8.8)', (*n_ptr).night_desig)
	if( (*s_ptr).default_font ne 'None') then begin
    new_struct = Dialog_Big_Table(Title="Coadded Image Table",Label=r_string, $
					Group_Leader=Event.top, Col_Headers = col_headers, $
					Format=format, Row_headers = row_headers,$
					Data_array = data_array, Font='Helvetica*16', $
					color_array=color_array)
	endif else begin
	 new_struct = Dialog_Big_Table(Title="Coadded Image Table",Label=r_string, $
	    Group_Leader=Event.top, Col_Headers = col_headers, $
	    Format=format, Row_headers = row_headers,$
	    Data_array = data_array, Font='None', $
	    color_array=color_array)
	endelse
	;
	;	load new table data back into structure
	;
	nc = N_TAGS(new_struct)
	nr = N_ELEMENTS(new_struct)
	;
	;	note - deletion of images not allowed
	;
;	for ir = 0, nr-1 do begin
;		(*n_ptr).image_object[ir] = 	new_struct[ir].object
;		(*n_ptr).image_type[ir] = 		new_struct[ir].type
;		(*n_ptr).image_filter[ir] = 	new_struct[ir].filter
;		(*n_ptr).image_exptime[ir] = 	new_struct[ir].expt
;		(*n_ptr).image_HWP_angle[ir] = 	new_struct[ir].HWP
;		(*n_ptr).image_ski_jump[ir] = 	new_struct[ir].SJ
;		(*n_ptr).image_frame[ir] = 		new_struct[ir].frame
;		(*n_ptr).image_camera[ir] = 	new_struct[ir].cam
;		(*n_ptr).image_root_index[ir] = new_struct[ir].Pind
;		(*n_ptr).image_comment[ir] = 	new_struct[ir].comment
;	endfor
	;
	;	all the BPM groups built, populate, then show, table
	;
	;	at the end of the show/edit cycle, build the list of pathnames to
	;	the CDS images, resolving all redirect indices
	;
	;---------------------------------
	;
	P6_button_sensitize, Event, 8, 1
end
;
;------------------------------------------------------------
pro P6_Read_Coadded_Images, Event
	;
	;	get the coadded CDS and RAW1 images into a new structure and pointer
	;
	MSP_BDP_print_text, Event, "Read Coadded Images Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	b_ptr = PTR_struct.PTR_BPM
	;
	;	jump to correct method processor
	;
	iskip = 0
	case (*l_ptr).method of
		1:	P6_Read_Images_Method_1, Event
		2:	P6_Read_Images_Method_2, Event
		else: begin
				MSP_BDP_print_text, Event, "No images to read. skipping"
				iskip = 1
			  end
	endcase
	;
	if(iskip eq 0) then begin
		;
		for ib = 9,10 do P6_button_sensitize, Event, ib, 1
		;
		;	and text windows
		;
		for ib = 0, 1 do WIDGET_CONTROL, (*s_ptr).P6_text_wid[ib], SENSITIVE=1
		;
		P6_Draw_Ready, Event, TEXT='All Images Read In'
	endif
end
;
;-----------------------------------------------------------
pro P6_Read_Images_Method_1, Event
	;
	;	get the coadded CDS and RAW1 images into a new structure and pointer
	;
	MSP_BDP_print_text, Event, "Read Coadded Images using Method 1 Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	b_ptr = PTR_struct.PTR_BPM
	;
	P6_LED_sensitize, Event, 1, 1
	LED_wid = (*s_ptr).P6_LED_wid[1]
	colors = strarr(10)
	colors[*] = 'yellow'
	WCT_LED_stack, (*s_ptr).P6_LED_wid[1], colors
	;
	;	number of images
	;
	n_dir_list = (*l_ptr).n_coadd
	;
	LED_scale = float(n_dir_list) / 10.0
	;
	lin_path_list = (*l_ptr).coadd_path_list[0:n_dir_list-1]
	bias_path_list = (*l_ptr).coadd_path_bias
	;
	;	image size
	;
	bias_image = readfits(bias_path_list, header, /SILENT)
	image_size = SIZE(bias_image)
	nc = image_size[1]
	nr = image_size[2]
	bias_raw1_dir = FILE_DIRNAME(bias_path_list)
	bias_raw1_path = fix_path( bias_raw1_dir + PATH_SEP()+'raw1_bias' )
	bias_raw1_image = double(read_raw1(bias_raw1_path, nc, nr))
	;
	;	bad pixel mask
	;
	MSP_BDP_Read_BPM_Headers, Event
	;
	;	get night, UT for Linearity Data
	;
	this_lin = (*l_ptr).this_lin
	lin_night = (*l_ptr).lin_night[this_lin]
	lin_ut = (*l_ptr).lin_UT[this_lin]
	;
	n_bpm = (*b_ptr).bpm_n_BPM
	d_UT_min = 1.0e6
	jbpm = 0
	for ibpm = 0, n_bpm-1 do begin
		if( lin_night eq (*b_ptr).bpm_night[ibpm]) then begin
			d_UT = abs(lin_ut - (*b_ptr).bpm_UT[ibpm])
			if(d_UT lt d_UT_min) then begin
				d_UT_min = d_UT
				jbpm = ibpm
			endif
		endif
	endfor
	;
	;print, "Closest night-UT BPM match is number ",jbpm
	;print, "  smallest d_UT = ",d_UT_min
	;
	BPM_path = (*b_ptr).bpm_image_name[jbpm]
	;
	(*l_ptr).BPM_path = BPM_path
	;
	;	read it in
	;
	BPM_image = long(readfits(BPM_path, BPM_header, /SILENT))
	;
	lin_data = {	n_images:	n_dir_list+1,	$						; including bias image
					bias_image:	bias_image,		$						; the bias CDS image
					bias_raw1_image: bias_raw1_image, $					; the bias raw1 image
					BPM_image: BPM_image,		$						; the bad pixel mask
					lin_header:		strarr(200),$						; FITS header template
					n_header:				0L,	$						; number of header lines
					lin_expt:	(*l_ptr).lin_expt[0:n_dir_list-1],	$	; the exposure time list
					lin_image_0: fltarr(n_dir_list,nc,nr) $				; the CDS-RAW1+Bias_raw1 images
				}
	;
	(*l_ptr).lin_data_ptr_0 = PTR_NEW(lin_data,/NO_COPY)	; create heap variable of this structure
	;
	;	loop on reading linearity CDS images, subtracting their RAW1 images and adding in the bias-raw1 images
	;
	;	This will generate a data cube of RR-D2 images (ie, second read referenced to reset release)
	;
	for il = 0, n_dir_list-1 do begin
		;
		this_LED = ROUND( float(il) / LED_scale)
	    this_LED = 0 > (this_LED < 9)
	    ;
	    colors[this_LED] = 'orange'
	    WCT_LED_stack, (*s_ptr).P6_LED_wid[1], colors
        ;
		lin_image = readfits(lin_path_list[il],header,/SILENT)
		;
		colors[this_LED] = 'blue'
    	WCT_LED_stack, (*s_ptr).P6_LED_wid[1], colors
    	;
		if(il eq 0) then begin
			;
			;	capture the header
			;
			(*(*l_ptr).lin_data_ptr_0).n_header = N_ELEMENTS(header)
			(*(*l_ptr).lin_data_ptr_0).lin_header[0:N_ELEMENTS(header)-1] = header
		endif
		;
		;	and raw1 image
		;
		raw1_name = STRMID(FILE_BASENAME(lin_path_list[il]),4,8)
		raw1_path = fix_path( bias_raw1_dir + PATH_SEP()+'raw1_' + raw1_name )
		raw1_image = double(read_raw1(raw1_path, nc, nr))
		;
		colors[this_LED] = 'violet'
    	WCT_LED_stack, (*s_ptr).P6_LED_wid[1], colors
		;
		;	do the math
		;
		temp_image = safe_math(lin_image,raw1_image,BPM_image,'-')		; minus because CDS = D1 - D2
		;
		D2_RR_image = safe_math(temp_image, bias_raw1_image, BPM_image, '+')	; to make B1 - D2 = RR - D2
		(*(*l_ptr).lin_data_ptr_0).lin_image[il,*,*] = float(D2_RR_image[*,*])
		;
		colors[this_LED] = 'green'
    	WCT_LED_stack, (*s_ptr).P6_LED_wid[1], colors
    	;
	endfor
	;
	MSP_BDP_print_text, Event, "Read in all CDS and RAW1 coadded images"
	;
end
;
;-----------------------------------------------------------
pro P6_Read_Images_Method_2, Event
	;
	;	get the coadded CDS and RAW1 images into a new structure and pointer
	;
	MSP_BDP_print_text, Event, "Read Coadded Images Method 2 Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	b_ptr = PTR_struct.PTR_BPM
	;
	P6_LED_sensitize, Event, 1, 1
	LED_wid = (*s_ptr).P6_LED_wid[1]
	colors = strarr(10)
	colors[*] = 'yellow'
	WCT_LED_stack, (*s_ptr).P6_LED_wid[1], colors
	;
	;	number of images
	;
	n_dir_list = (*l_ptr).n_coadd
	;
	LED_scale = float(n_dir_list) / 10.0
	;
	lin_path_list = (*l_ptr).coadd_path_list[0:n_dir_list-1]
	;bias_path_list = (*l_ptr).coadd_path_bias
	;
	;	image size
	;
	lin_image = readfits(lin_path_list[0], header, /SILENT)
	image_size = SIZE(lin_image)
	nc = image_size[1]
	nr = image_size[2]
;	bias_raw1_dir = FILE_DIRNAME(bias_path_list)
;	bias_raw1_path = bias_raw1_dir + PATH_SEP()+'raw1_bias'
;	bias_raw1_image = double(read_raw1(bias_raw1_path, nc, nr))
	bias_image = 'none'
	bias_raw1_image = 0L
	;
	;	bad pixel mask
	;
	MSP_BDP_Read_BPM_Headers, Event
	;
	;	get night, UT for Linearity Data
	;
	this_lin = (*l_ptr).this_lin
	lin_night = (*l_ptr).lin_night[this_lin]
	lin_ut = (*l_ptr).lin_UT[this_lin]
	;
	n_bpm = (*b_ptr).bpm_n_BPM
	d_UT_min = 1.0e6
	jbpm = 0
	for ibpm = 0, n_bpm-1 do begin
		if( lin_night eq (*b_ptr).bpm_night[ibpm]) then begin
			d_UT = abs(lin_ut - (*b_ptr).bpm_UT[ibpm])
			if(d_UT lt d_UT_min) then begin
				d_UT_min = d_UT
				jbpm = ibpm
			endif
		endif
	endfor
	;
	;print, "Closest night-UT BPM match is number ",jbpm
	;print, "  smallest d_UT = ",d_UT_min
	;
	BPM_path = (*b_ptr).bpm_image_name[jbpm]
	;
	(*l_ptr).BPM_path = BPM_path
	;
	;	read it in
	;
	BPM_image = long(readfits(BPM_path, BPM_header, /SILENT))
	;
;	lin_data = {	n_images:	n_dir_list+1,	$						; including bias image
;					bias_image:	bias_image,		$						; the bias CDS image
;					bias_raw1_image: bias_raw1_image, $					; the bias raw1 image
;					BPM_image: BPM_image,		$						; the bad pixel mask
;					lin_header:		strarr(200),$						; FITS header template
;					n_header:				0L,	$						; number of header lines
;					lin_expt:	(*l_ptr).lin_expt[0:n_dir_list-1],	$	; the exposure time list
;					lin_image: fltarr(n_dir_list,nc,nr) $				; the CDS-RAW1+Bias_raw1 images
;				}
;	;
;	(*l_ptr).lin_data_ptr = PTR_NEW(lin_data,/NO_COPY)	; create heap variable of this structure
	;
	;
	HELP, /MEMORY
	;
	print, "lin_image request size = ",n_dir_list," x ",nc," x ",nr
	;
	;	if more than 26 in the directory list, must also allocate the auxillary arrays
	;
	if(n_dir_list le 26) then begin
		n_dir_list_0 = n_dir_list
		n_dir_list_1 = 0
		n_dir_list_2 = 0
		n_dir_list_3 = 0
	endif else begin
		if(n_dir_list le 51) then begin
			n_dir_list_0 = 26
			n_dir_list_1 = n_dir_list - 26
			n_dir_list_2 = 0
			n_dir_list_3 = 0
		endif else begin
			if(n_dir_list le 76) then begin
				n_dir_list_0 = 26
				n_dir_list_1 = 25
				n_dir_list_2 = n_dir_list - 26 - 25
				n_dir_list_3 = 0
			endif else begin
				if(n_dir_list le 101) then begin
					n_dir_list_0 = 26
					n_dir_list_1 = 25
					n_dir_list_2 = 25
					n_dir_list_3 = n_dir_list - 26 - 25 - 25
				endif else begin
					print, "error - too many images in linearity run - no more than 101 allowed"
					stop
				endelse
			endelse
		endelse
	endelse
	;
	(*l_ptr).lin_data_ptr_0 = PTR_NEW({	n_images:	n_dir_list+1,	$						; including bias image
										bias_image:	bias_image,		$						; the bias CDS image
										bias_raw1_image: bias_raw1_image, $					; the bias raw1 image
										BPM_image: BPM_image,		$						; the bad pixel mask
										lin_header:		strarr(200),$						; FITS header template
										n_header:				0L,	$						; number of header lines
										lin_expt:	(*l_ptr).lin_expt[0:n_dir_list-1],	$	; the exposure time list
										lin_image_0: fltarr(n_dir_list_0,nc,nr) $			; the CDS-RAW1+Bias_raw1 images
										},/NO_COPY)	; create heap variable while creating the structure
	if(n_dir_list_1 gt 0) then begin
		(*l_ptr).lin_data_ptr_1 = PTR_NEW({lin_image_1: fltarr(n_dir_list_1,nc,nr)},/NO_COPY)	; create heap variable while creating the structure
	endif
	;
	if(n_dir_list_2 gt 0) then begin
		(*l_ptr).lin_data_ptr_2 = PTR_NEW({lin_image_2: fltarr(n_dir_list_2,nc,nr)},/NO_COPY)	; create heap variable while creating the structure
	endif
	;
	if(n_dir_list_3 gt 0) then begin
		(*l_ptr).lin_data_ptr_3 = PTR_NEW({lin_image_3: fltarr(n_dir_list_3,nc,nr)},/NO_COPY)	; create heap variable while creating the structure
	endif
	;
	;	loop on reading linearity RR-D2 images (ie, second read referenced to reset release)
	;
	for il = 0, n_dir_list-1 do begin
		;
		this_LED = ROUND( float(il) / LED_scale)
	    this_LED = 0 > (this_LED < 9)
	    ;
	    colors[this_LED] = 'orange'
	    WCT_LED_stack, (*s_ptr).P6_LED_wid[1], colors
        ;
		lin_image = readfits(lin_path_list[il],header,/SILENT)
		;
		colors[this_LED] = 'blue'
    	WCT_LED_stack, (*s_ptr).P6_LED_wid[1], colors
    	;
		if(il eq 0) then begin
			;
			;	capture the header
			;
			(*(*l_ptr).lin_data_ptr_0).n_header = N_ELEMENTS(header)
			(*(*l_ptr).lin_data_ptr_0).lin_header[0:N_ELEMENTS(header)-1] = header
		endif
		;
		;	and raw1 image
		;
;		raw1_name = STRMID(FILE_BASENAME(lin_path_list[il]),4,8)
;		raw1_path = bias_raw1_dir + PATH_SEP()+'raw1_' + raw1_name
;		raw1_image = double(read_raw1(raw1_path, nc, nr))
		;
		colors[this_LED] = 'violet'
    	WCT_LED_stack, (*s_ptr).P6_LED_wid[1], colors
		;
		;	do the math
		;
;		temp_image = safe_math(lin_image,raw1_image,BPM_image,'-')		; minus because CDS = D1 - D2
;		;
;		D2_RR_image = safe_math(temp_image, bias_raw1_image, BPM_image, '+')	; to make B1 - D2 = RR - D2
;
		D2_RR_image = lin_image
		;
		if(il le 25) then begin
			(*(*l_ptr).lin_data_ptr_0).lin_image_0[il,*,*] = float(D2_RR_image[*,*])
		endif
		;
		if(il gt 25 and il le 50) then begin
			(*(*l_ptr).lin_data_ptr_1).lin_image_1[il-26,*,*] = float(D2_RR_image[*,*])
		endif
		;
		if(il gt 50 and il le 75) then begin
			(*(*l_ptr).lin_data_ptr_2).lin_image_2[il-51,*,*] = float(D2_RR_image[*,*])
		endif
		;
		if(il gt 75 and il le 100) then begin
			(*(*l_ptr).lin_data_ptr_3).lin_image_3[il-76,*,*] = float(D2_RR_image[*,*])
		endif
		;
		colors[this_LED] = 'green'
    	WCT_LED_stack, (*s_ptr).P6_LED_wid[1], colors
    	;
	endfor
	;
	MSP_BDP_print_text, Event, "Read in D2-RR coadded images"
	;
end
;------------------------------------------------------------
pro P6_Show_Pixel_Plot, Event
	;
	MSP_BDP_print_text, Event, "Show Pixel Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	b_ptr = PTR_struct.PTR_BPM
	r_ptr = PTR_struct.PTR_RAW1
	;
	;	number of images
	;
	n_dir_list = (*l_ptr).n_coadd
	;
	;	pixel coords
	;
	xx = (*l_ptr).xx
	yy = (*l_ptr).yy
	;
	;print, "xx, yy = ",xx,yy
	;
	;	pointers to data
	;
	d_ptr_0 = (*l_ptr).lin_data_ptr_0
	d_ptr_1 = (*l_ptr).lin_data_ptr_1
	d_ptr_2 = (*l_ptr).lin_data_ptr_2
	d_ptr_3 = (*l_ptr).lin_data_ptr_3
	;
	x_plot = (*d_ptr_0).lin_expt[0:n_dir_list-1]
	y_plot = REFORM( (*d_ptr_0).lin_image_0[*,xx,yy])
	;
	if(n_dir_list gt 26) then begin
		n_dir_list_1 = (25 < (n_dir_list - 26))
		y_plot = [y_plot, REFORM( (*d_ptr_1).lin_image_1[*,xx,yy])]
	endif
	;
	if(n_dir_list gt 51) then begin
		n_dir_list_1 = (25 < (n_dir_list - 51))
		y_plot = [y_plot, REFORM( (*d_ptr_2).lin_image_2[*,xx,yy])]
	endif
	;
	if(n_dir_list gt 76) then begin
		n_dir_list_1 = (25 < (n_dir_list - 76))
		y_plot = [y_plot, REFORM( (*d_ptr_3).lin_image_3[*,xx,yy])]
	endif
	;
	;	check for deleted data
	;
	npts = N_ELEMENTS(y_plot)
	ind = WHERE(y_plot[*] ne -1.0e6, count)
	if(count ne npts) then begin
		if(count gt 1) then begin
			;
			;	trim to good ones, but write message
			;
			x_plot = x_plot[ind]
			y_plot = y_plot[ind]
			MSP_BDP_print_text, Event,"Warning - some data were deleted in this plot"
		endif else begin
			;
			;	all deleted, send message
			;
			P6_Draw_Done, Event, TEXT="All deleted data"
		endelse
	endif
	;
	x_text = STRING(format='(I4.1)',xx)
	y_text = STRING(format='(I4.1)',yy)
	;
	;	plot into widget draw window
	;
	device, decomposed=0
	color_red = REFORM((*r_ptr).color_rgb[0,*])
	color_green = REFORM((*r_ptr).color_rgb[1,*])
	color_blue = REFORM((*r_ptr).color_rgb[2,*])
	;
	tvlct, color_red, color_green, color_blue, 0
	;
	win_wid = (*s_ptr).P6_draw_window_id
	WSET, win_wid
	;
	plot, x_plot, y_plot,yrange=[0.0,max(y_plot)],xrange=[-1.0,max(x_plot)],xstyle=1,psym=-4,$
				color=1, background=0, thick=1, LINESTYLE=0, Ytitle="CDS ADUs", $
				Xtitle = "Exp. Time", TITLE="Pixel  ("+x_text+','+y_text+')', $
				;POSITION=(*b_ptr).position,/DEVICE,
				xtickinterval=1.0, $
				xminor=4
	;
	;	DPC 20100708 - clean the ramp, showing replaced pixels
	;
	y_in = y_plot
	;
	clean_ramp, x_plot, y_plot
	;
	ind_bad = WHERE(y_in ne y_plot, nbad)
	if(nbad gt 0) then begin
		oplot, x_plot[ind_bad], y_plot[ind_bad],psym=6,color=2,thick=2
	endif
	;
	a = 0.0
end
;
;-------------------------------------------------------------
pro P6_Start_Linearity_Calculation, Event
	;
	;	launch linearity table maker
	;
	MSP_BDP_print_text, Event, "Make Linearity Tables Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	l_ptr = PTR_struct.PTR_Linearity
	;
	make_linearity_tables, Event
	;
	;	release the pointer heap data
	;
	PTR_FREE, (*l_ptr).lin_data_ptr_0
	PTR_FREE, (*l_ptr).lin_data_ptr_1
	PTR_FREE, (*l_ptr).lin_data_ptr_2
	PTR_FREE, (*l_ptr).lin_data_ptr_3
	;
	num_nights = (*s_ptr).num_nights
	ind = WHERE((*n_ptr).night_desig eq (*s_ptr).night_date[0:num_nights-1], count)
	if(count ne 0) then night_index = ind[0]
	;
	(*s_ptr).night_step[night_index] = 6
	(*s_ptr).night_comment[night_index] = "Lin. Tables Made"
    ;
    ;	this table gets registered in the list by make_linearity_tables
	;
	P1_Write_Summary_File, Event
	;
	P6_Draw_Done, Event
end
;
;-----------------------------------------------------------------
pro P6_Save_Night_Summary, Event
	;
	;	saves an updated night summary
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	l_ptr = PTR_struct.PTR_Linearity
	;
	this_night = (*n_ptr).night_desig
	;
	;	which one in the summary?
	;
	num_nights = (*s_ptr).num_nights
	ind = WHERE(this_night eq (*s_ptr).night_date[0:num_nights-1], count)
	if(count ne 0) then night_index = ind[0] else begin
		MSP_BDP_print_text, Event, "Error - No matching night in P5..."
		goto, P5SNS_end
	endelse
	;
	num_nights = (*s_ptr).num_nights
	ind = WHERE(this_night eq (*s_ptr).night_date[0:num_nights-1], count)
	if(count ne 0) then night_index = ind[0]
	(*s_ptr).night_who[night_index] = (*s_ptr).who
	(*s_ptr).night_step[night_index] = 6
	(*s_ptr).night_comment[night_index] = "Lin Table Made"
	;
	P1_Write_Summary_File, Event
	;
	P6_Draw_Done, Event, TEXT=text
	P5SNS_end:
end
;
;--------------------------------------
pro P6_Draw_Done, Event, TEXT=text
	;
	;	clears draw space and writes "Done !" in center
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	l_ptr = PTR_struct.PTR_Linearity
	;
	WSET, (*s_ptr).P6_draw_window_id
	ERASE
	;
	device, decomposed=0
	color_red = REFORM((*r_ptr).color_rgb[0,*])
	color_green = REFORM((*r_ptr).color_rgb[1,*])
	color_blue = REFORM((*r_ptr).color_rgb[2,*])
	;
	tvlct, color_red, color_green, color_blue, 0
	;
	window_id = (*s_ptr).P6_draw_window_id
	;
	WSET, window_id
	;
	x = [0., 1.]
	y = [0., 1.]
	;
	plot, x, y, color=0, xrange=[0.,1.], yrange=[0.,1.], xstyle=1, ystyle=1, $
				background=0, thick=1, LINESTYLE=0, $
				;POSITION=(*b_ptr).position,
				/DEVICE,/NODATA
	;
	XYoutS, 0.5, 0.5, "Done !", ALIGNMENT=0.5, color=2, charthick=2, charsize=4.0
	;
	nt = N_ELEMENTS(TEXT)
	if(nt eq 1) then XYoutS, 0.5, 0.25, text, ALIGNMENT=0.5, color=2, charthick=2, charsize=4.0
	;
end
;
;------------------------------------------------------
pro P6_Draw_Ready, Event, TEXT=text
	;
	;	clears draw space and writes "Ready" in center
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	l_ptr = PTR_struct.PTR_Linearity
	;
	WSET, (*s_ptr).P6_draw_window_id
	ERASE
	;
	device, decomposed=0
	color_red = REFORM((*r_ptr).color_rgb[0,*])
	color_green = REFORM((*r_ptr).color_rgb[1,*])
	color_blue = REFORM((*r_ptr).color_rgb[2,*])
	;
	tvlct, color_red, color_green, color_blue, 0
	;
	window_id = (*s_ptr).P6_draw_window_id
	;
	WSET, window_id
	;
	x = [0., 1.]
	y = [0., 1.]
	;
	plot, x, y, color=0, xrange=[0.,1.], yrange=[0.,1.], xstyle=1, ystyle=1, $
				background=0, thick=1, LINESTYLE=0, $
				;POSITION=(*b_ptr).position,
				/DEVICE,/NODATA
	;
	XYoutS, 0.5, 0.5, "Ready", ALIGNMENT=0.5, color=2, charthick=2, charsize=3.0
	;
	nt = N_ELEMENTS(TEXT)
	if(nt eq 1) then XYoutS, 0.5, 0.25, text, ALIGNMENT=0.5, color=2, charthick=2, charsize=1.5
	;
end