;
;	Part 7 Event Handler Routines - Make Ave RAW1 images
;
;	[this version *also* creates FITS files for each raw1 image,
;		where useful information is saved in the FITS headers.]
;
;-----------------------------------------------------------------
;
pro P7_Select_Night, Event
	;
	MSP_BDP_print_text, Event, "Night Selection Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	b_ptr = PTR_struct.PTR_BPM
	;
	wid = (*s_ptr).P7_droplist_night_wid[0]
	;
	night_index = Event.index
	;
	;	index is not enough, as we might not have done all the part 2
	;	stuff.... So, need to get the night designation from the list
	;
	WIDGET_CONTROL, wid, GET_VALUE = vector_night_desig
	selected_night_desig = vector_night_desig[night_index]
	;
	night_desig = selected_night_desig
	night_text = STRING(format='(I8.8)',night_desig)
	;
	;	reset buttons, and LEDs
	;
	for iw = 0, N_ELEMENTS( (*s_ptr).P7_button_wid)-1 do begin
		if(iw le 0) then sense = 1 else sense = 0
		P7_button_sensitize, Event, iw, sense
		;if(iw le 1) then P5_base_sensitize, Event, iw, 0
		if(iw eq 0) then P7_LED_sensitize, Event, iw, 0
	endfor
	;
	;	read in that night's save file
	;
	MSP_BDP_print_text, Event, "Opening Night Summary for night = "+night_text
	;
	MSP_BDP_Create_Night_File_Path, Event, night_desig, night_file_path
	MSP_BDP_Restore_Night_File, Event, night_file_path, description, ierr
	;
	if(ierr ne 0) then begin
		MSP_BDP_print_text, Event, "Could not restore night "+night_text+" - skipping"
		goto, P7SN_end
	endif
	;
	;	must redo pointers
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	raw_root = (*s_ptr).root_raw
	;
	;	capture number of groups
	;
	ngroups = (*n_ptr).num_groups
	ngroups_text = STRING(format='(I)',ngroups)
	MSP_BDP_print_text, Event, "Restored "+night_text+" with "+ngroups_text+" groups"
	;
	;	find all the bias groups
	;
	P7_Find_Bias_Groups, Event, n_ave_raw1
	;
	;	turn on first button
	;
	if(n_ave_raw1 gt 0) then P7_button_sensitize, Event, 0, 1 else P7_button_sensitize, Event, 0, 0
	;
	;WIDGET_CONTROL, (*s_ptr).P5_droplist_night_wid[1], SENSITIVE=0		; TBD off for now
	;
	P7SN_end:
end
;
;------------------------------------------------------
pro P7_Find_Bias_Groups, Event, n_ave_raw1
	;
	;	show the table of BIAS groups for Ave RAW1 image creation
	;
	MSP_BDP_print_text, Event, "Find Groups for Ave RAW1 creation requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	b_ptr = PTR_struct.PTR_BPM
	;
	night_desig = (*n_ptr).night_desig
	night_desig_text = STRING(format='(I8.8)',night_desig)
	;
	root_raw = (*s_ptr).root_raw
	;
	n_images = (*n_ptr).num_images
	file_list = (*n_ptr).image_list[0:n_images-1]
	image_type = (*n_ptr).image_type[0:n_images-1]
	image_frame = (*n_ptr).image_frame[0:n_images-1]
	;
	input_pathname_list = strarr(n_images)
	;
	;	1. Find and save the info for Bias/Dark groups in structure
	;
	n_groups = (*n_ptr).num_groups
	n_ave_raw1 = 0L
	;
	for ig = 0, n_groups-1 do begin
		;
		;	is this group a BIAS group? (or a 0 sec Dark?)
		;
		this_group_type = (*n_ptr).groups[ig].group_type
		this_group_exp = (*n_ptr).groups[ig].group_exptime
		tgt_UC = STRUPCASE(this_group_type)
		test_1 = (tgt_UC  eq 'BIAS')			; save bias groups, or
		test_2 = (tgt_UC eq 'DARK')				; darks with 0 sec times
		test_3 = this_group_exp eq 0.0
		test_23 = test_2 && test_3
		if(NOT(test_1) and NOT(test_23)) then goto, group_loop_end
		;
		;	this group is a bias (or 0s Dark) group, add its info to the ave_raw1 structure
		;
		group_id = (*n_ptr).groups[ig].group_id
		group_images = (*n_ptr).groups[ig].group_images
		image_list = (*n_ptr).groups[ig].group_list[0:group_images-1]
		;
		coadds = (*n_ptr).groups[ig].group_coadds
		;
		;	UT
		;
		image_no = image_list[0]
		ind = WHERE( (*n_ptr).image_list[0:n_images-1] eq image_no, count)
		if(count ne 1) then begin
			MSP_BDP_print_text, Event, "ERROR - no match of image no in P7 UT load. image_no = "+STRING(format='(i)',image_no)
			goto, group_loop_end
		endif
		;
		ut = (*n_ptr).header_list[ind[0]].ut
		(*a_ptr).group[n_ave_raw1].ut = ut
		;
		;print, "first call to UT = ",ut
		;
		(*a_ptr).n_groups = n_ave_raw1 + 1
		(*a_ptr).this_group_no = n_ave_raw1
		(*a_ptr).group[n_ave_raw1].serial_no = n_ave_raw1
		(*a_ptr).group[n_ave_raw1].type = this_group_type
		(*a_ptr).group[n_ave_raw1].output_FITS_name = 'FITS_Ave_RAW1_'+night_desig_text+"."+STRTRIM(STRING(format='(i4.1)',n_ave_raw1),2)+'.fits'
		(*a_ptr).group[n_ave_raw1].output_RAW1_name = 'Ave_RAW1_'+night_desig_text+'.'+STRTRIM(STRING(format='(i4.1)',n_ave_raw1),2)+'.raw1'
		(*a_ptr).group[n_ave_raw1].night_desig = night_desig
		(*a_ptr).group[n_ave_raw1].group_id = group_id
		(*a_ptr).group[n_ave_raw1].n_images = group_images
		(*a_ptr).group[n_ave_raw1].coadds = coadds
		;
		;	image pathnames, resolved as to raw directory
		;
		for ii = 0, group_images-1 do begin
			;
			ind = WHERE( (*n_ptr).image_list[0:n_images-1] eq image_list[ii],count)
			if(count ne 1) then begin
				MSP_BDP_print_text, Event, "ERROR - no match of image no in P7 pathname load. image_no = "+STRING(format='(i)',image_list[ii])
				goto, group_loop_end
			endif
			;
			index = (*n_ptr).image_root_index[ind[0]]
			if(index eq 0) then begin
				base_dir = (*s_ptr).root_raw
			endif else begin
				base_dir = (*s_ptr).root_reduced
			endelse
			;
			cds_directory = base_dir + (*n_ptr).paths_cds_root[index]
			raw1_directory = base_dir + (*n_ptr).paths_raw1_root[index]
			;
			image_no_text = STRTRIM(STRING(format='(i5.3)',image_list[ii]),2)
			cds_path = fix_path( FILEPATH(night_desig_text+"."+image_no_text+".fits", ROOT_DIR=cds_directory) )
			raw1_path = fix_path( FILEPATH('raw1_'+image_no_text,ROOT_DIR=raw1_directory) )
			;
			;print, "CDS path = ",cds_path
			;print, "RAW1 path = ",raw1_path
			;
			(*a_ptr).group[n_ave_raw1].image_CDS_pathname[ii] = cds_path
			(*a_ptr).group[n_ave_raw1].image_RAW1_pathname[ii] = raw1_path
		endfor
		;
		;	increment counter
		;
		n_ave_raw1++
		;
		MSP_BDP_print_text, Event, "For Ave RAW1 "+STRING(format='(i)',n_ave_raw1)+$
				" found "+STRING(format='(i)',group_images)+" images"
		;
		group_loop_end:
	endfor
	;
	(*a_ptr).n_groups = n_ave_raw1
	;
	;	if there are no images to build, then write summary and exit
	;
	if(n_ave_raw1 eq 0) then begin
		MSP_BDP_print_text, Event, "No Bias groups found to build Ave RAW1 images from."
		num_nights = (*s_ptr).num_nights
		ind = WHERE(night_desig eq (*s_ptr).night_date[0:num_nights-1], count)
		if(count ne 0) then night_index = ind[0] else begin
			MSP_BDP_print_text, Event, "Error - No matching night in P7..."
			goto, P7_ST_end
		endelse
		;
		(*s_ptr).night_step[night_index] = 7
		text = "No Ave RAW1s Made"
		(*s_ptr).night_comment[night_index] = text
		;
		P1_Write_Summary_File, Event
		;
		goto, P7_ST_end
	endif
	;
	;	OK, we are going to build images. Check existence of output directory
	;	and create it if needed.
	;
	(*a_ptr).output_directory = '7_Ave_RAW1_Images'
	ar1_dir_path = fix_path( (*s_ptr).root_reduced + (*a_ptr).output_directory )
	;
	;	see if this directory exists
	;
	test_dir = FILE_INFO( ar1_dir_path )
	if( NOT(test_dir.exists) or NOT(test_dir.directory) ) then begin
		MSP_BDP_print_text, Event, "Creating new Ave RAW1 directory: "+ar1_dir_path
		FILE_MKDIR, ar1_dir_path
	endif
	;
	P7_ST_end:
end
;
;------------------------------------------------------
pro P7_View_Group_Table, Event
	;
	;	show the table of BIAS groups for Ave RAW1 image creation
	;
	MSP_BDP_print_text, Event, "Show/Edit Groups for Ave RAW1 creation requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	b_ptr = PTR_struct.PTR_BPM
	;
	;	build the table structure
	;
	col_headers = ['Group', 'Type', '____UT____', 'N_Images', '1st Image', 'Last Image', 'CoAdds']
	;
	n_lines = (*a_ptr).n_groups
	;
	;	test for no information case
	;
	if(n_lines eq 0) then begin
		P7_button_sensitize, Event, 0, 0
		MSP_BDP_print_text, Event, "No Groups To Process, skipping"
		goto, P7VE_end
	endif
	;
	format = strarr(8,n_lines)
	format[0:6,*] = '(a)'
	;
	color_array = strarr(7,n_lines)
	;
	row_headers = strarr(n_lines)
	data_line = {group_id: '', type: '', UT: '', n_images: '',$
				 first: '', last: '', coadds: ''}
	data_array = REPLICATE(data_line, n_lines)			; array of structures
	;
	for i = 0, n_lines-1 do begin
			;
			row_headers[i] = 'Ave_RAW1_'+STRTRIM(STRING(format='(i4.1)',i),2)
			;
			g_text = STRTRIM(STRING(format='(i4.1)',(*a_ptr).group[i].group_id),2)
			data_array[i].group_id    = 'Gr_'+g_text
			data_array[i].type = (*a_ptr).group[i].type
			data_array[i].coadds = STRTRIM(STRING(format='(i3)',(*a_ptr).group[i].coadds),2)
			;
			;	convert UT to string
			;
			ut = (*a_ptr).group[i].ut
			;
			;print, "UT = ",ut
			;
			uth = fix(ut)
			utm = fix( 60.0*(ut - uth))
			uts = ut*3600.0d0 - double(uth)*3600.0d0 - double(utm)*60.0d0
			utsx = fix(uts/10.0)
			uts = uts - 10.0*utsx
			ut_text = STRING(format='(i2.2,":",i2.2,":",i1.1,f3.1)',uth,utm,utsx,uts)
			;
			data_array[i].UT    		= ut_text
			data_array[i].n_images    	= STRTRIM(STRING(format='(i3.1)',(*a_ptr).group[i].n_images),2)
			;
			;	get first, last image numbers in this group
			;
			this_group = (*a_ptr).group[i].group_id
			n_images = (*a_ptr).group[i].n_images
			first = (*n_ptr).groups[this_group].group_list[0]
			last = (*n_ptr).groups[this_group].group_list[n_images-1]
			;
			data_array[i].first = STRTRIM(STRING(format='(i5.1)',first),2)
			data_array[i].last = STRTRIM(STRING(format='(i5.1)',last),2)
			;
			color_array[*,i] = "white"
			;
			case data_array[i].type of
				"Bias":	color = "yellow"
				"Dark":	color = "gray"
				"**BAD**": color = 'red'
				"Skip": color = 'red'
				else: color = "white"
			endcase
			;
			color_array[*,i] = color
			;
	endfor
	;
	r_string = STRING(format='("Ave RAW1 Group List for Night ",I8.8)', (*n_ptr).night_desig)
	if( (*s_ptr).default_font ne 'None') then begin
    new_struct = Dialog_Big_Table(Title="Ave RAW1 Group Table"+'   or change group type to "Skip" ',Label=r_string, $
					Group_Leader=Event.top, Col_Headers = col_headers, $
					Format=format, Row_headers = row_headers,$
					Data_array = data_array, Font='Helvetica*16', $
					color_array=color_array)
	endif else begin
	 new_struct = Dialog_Big_Table(Title="Ave RAW1 Group Table"+'   or change group type to "Skip" ',Label=r_string, $
	    Group_Leader=Event.top, Col_Headers = col_headers, $
	    Format=format, Row_headers = row_headers,$
	    Data_array = data_array, Font='None', $
	    color_array=color_array)
	endelse
	;
	;	load new table data back into structure
	;
	nc = N_TAGS(new_struct)
	nr = N_ELEMENTS(new_struct)
	;
	;	note - deletion of groups only by changing type to '**BAD**' or skip
	;
	for ir = 0, nr-1 do begin
		if( new_struct[ir].type eq '**BAD**' or STRUPCASE( new_struct[ir].type) eq 'SKIP') then new_struct[ir].type = 'Skip'
		(*a_ptr).group[ir].type = 		new_struct[ir].type
	endfor
	;
	;	all the BPM groups built, populate, then show, table
	;
	;	at the end of the show/edit cycle, build the list of pathnames to
	;	the CDS images, resolving all redirect indices
	;
	;--------------------------------]
	;
	;	turn on button to enable processing start
	;
	P7_button_sensitize, Event, 1, 1
	;
	P7VE_end:
end
;
;---------------------------------------------------
pro P7_Start_Image_Calculation, Event
	;
	;	calculates the Ave RAW1 images as raw1-types and as
	;		raw1-types converted to FITS (for viewing, and for
	;		storing UT and other valuable variables)
	;
	MSP_BDP_print_text, Event, "Starting Ave RAW1 Image Creation"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	a_ptr = PTR_struct.PTR_Ave_RAW1
	n_ptr = PTR_struct.PTR_night
	b_ptr = PTR_struct.PTR_BPM
	;
	;	number of Ave RAW1 to create
	;
	n_groups = (*a_ptr).n_groups
	;
	;	update to non '**BAD**' groups
	;
	good_ind = WHERE( (*a_ptr).group[0:n_groups-1].type ne '**BAD**' and (*a_ptr).group[0:n_groups-1].type ne 'Skip',n_good)
	if(n_good lt 1) then begin
		MSP_BDP_print_text, Event, "Error - no good groups to average. Skipping"
		goto, P7SIC_end
	endif
	;
	;	clear LED progress bar to yellow
	;
	n_LEDs = N_ELEMENTS( (*s_ptr).P7_LED_colors )
	LED_scale = float(n_good) / float(n_LEDs)
	if(LED_scale le 1.0) then LED_scale = 1.0
	(*s_ptr).P7_LED_colors[*] = 'yellow'
	if(n_good lt n_LEDs) then (*s_ptr).P7_LED_colors[n_good:n_LEDs-1] = 'black'
	WCT_LED_stack, (*s_ptr).P7_LED_wid[0], (*s_ptr).P7_LED_colors
	;
	igood = 0L
	for ig = 0, n_groups -1 do begin
		;
		;	skip **BAD** flagged groups
		;
		if( (*a_ptr).group[ig].type eq '**BAD**' or (*a_ptr).group[ig].type eq 'Skip') then goto, group_loop_end
		;
		night_desig_text = STRING(format='(I8.8)',(*a_ptr).group[ig].night_desig)
		;
		;	announce which Ave RAW1 is begin constructed
		;
		ar1_text = STRING(format='(I8.8)',(*n_ptr).night_desig) + '_' + $
					STRTRIM(STRING(format='(i4.1)',ig),2)
		WIDGET_CONTROL, (*s_ptr).P7_text_wid[0], SET_VALUE=ar1_text
		;
		this_LED = float(igood) / LED_scale
		this_LED = 0 > (this_LED < (n_LEDs-1))
		(*s_ptr).P7_LED_colors[this_LED] = 'orange'
		WCT_LED_stack, (*s_ptr).P7_LED_wid[0], (*s_ptr).P7_LED_colors
        ;
		;	next, fetch the appropriate bad pixel mask, based on night and UT
		;	proximity
		;
		;	start by fetching BPM directory list
		;
		MSP_BDP_Read_BPM_Headers, Event
		;
		;	get night, UT for This Group's Data
		;
		this_group = (*a_ptr).group[ig].group_id
		ar1_night = (*a_ptr).group[ig].night_desig
		ar1_ut = (*a_ptr).group[ig].ut
		;
		n_bpm = (*b_ptr).bpm_n_BPM
		d_UT_min = 1.0e6
		jbpm = 0
		for ibpm = 0, n_bpm-1 do begin
			if( ar1_night eq (*b_ptr).bpm_night[ibpm]) then begin
				d_UT = abs(ar1_ut - (*b_ptr).bpm_UT[ibpm])
				if(d_UT lt d_UT_min) then begin
					d_UT_min = d_UT
					jbpm = ibpm
				endif
			endif
		endfor
		;
		;print, "Closest night-UT BPM match is number ",jbpm
		;print, "  smallest d_UT = ",d_UT_min
		;
		BPM_path = fix_path( (*b_ptr).bpm_image_name[jbpm] )
		;
		MSP_BDP_print_text, Event, "  BPM file selected = "+FILE_BASENAME(BPM_path)
		;
		;	load input path list and output path name
		;
		n_images = (*a_ptr).group[ig].n_images
		pathlist = (*a_ptr).group[ig].image_RAW1_pathname[0:n_images-1]
		;
		outpath = fix_path( (*s_ptr).root_reduced +(*a_ptr).output_directory + PATH_SEP()+ $
						(*a_ptr).group[ig].output_RAW1_name )
		;
		;print, "output RAW1 pathname = ",outpath
		;
		(*s_ptr).P7_LED_colors[this_LED] = 'blue'
		WCT_LED_stack, (*s_ptr).P7_LED_wid[0], (*s_ptr).P7_LED_colors
		;
		if((*a_ptr).group[ig].night_desig ge (*s_ptr).row513_date) then row513_flag = 0 else row513_flag = 1
		;
		colors = (*s_ptr).P7_LED_colors
		LED_wid = (*s_ptr).P7_LED_wid[0]
		;
		MSP_BDP_print_text, Event, "Starting Coadd to build "+FILE_BASENAME(outpath)
		;
		coadds = (*a_ptr).group[ig].coadds
		;
		coadd_median_with_bad_RAW1_sub, Event, BPM_path, pathlist, outpath, row513_flag,$
										LED_wid, this_LED, colors, COADDS=coadds
		;
		(*s_ptr).P7_LED_colors[this_LED] = 'violet'
		WCT_LED_stack, (*s_ptr).P7_LED_wid[0], (*s_ptr).P7_LED_colors
		;
		;	read the first input image for its FITS header, then read in the Ave RAW1 image
		;	just created and make a new output FITS image (not a CDS image, note)
		;
		FITS_filepath = fix_path( (*s_ptr).root_reduced +(*a_ptr).output_directory+PATH_SEP()+$
							(*a_ptr).group[ig].output_FITS_name )
		;
		;print, "output FITS pathname = ",FITS_filepath
		;
		;	get the first input CDS image's FITS header
		;
		CDS_filepath = (*a_ptr).group[ig].image_CDS_pathname[0]
		save_header = headfits(CDS_filepath, /SILENT)
		;
		;	update header with info
		;
		FXADDPAR, save_header, "FILENAME", FILE_BASENAME(FITS_filepath), "Computed Ave RAW1 Image"
		FXADDPAR, save_header, "HISTORY","  Created Ave RAW1 Image on "+SYSTIME()
		FXADDPAR, save_header, "HISTORY","    reduced by "+(*s_ptr).who
		FXADDPAR, save_header, "HISTORY", "            using the following images:"
		for ifiles = 0, n_images-1 do begin
			FXADDPAR, save_header, "HISTORY", "      "+FILE_BASENAME(pathlist[ifiles])
		endfor
		FXADDPAR, save_header, "BDP_NGHT", (*a_ptr).group[ig].night_desig, "MSP_BDP Night Designation"
		FXADDPAR, save_header, "BDP_GRP", (*a_ptr).group[ig].group_id, "MSP_BDP Group Number "
		FXADDPAR, save_header, "HISTORY", "  Ave RAW1 created using BPM file = "+FILE_BASENAME(BPM_path)
		if(coadds ne 1) then begin
			FXADDPAR, save_header, "HISTORY","  Respecting input COADDS="+STRING(format='(I3)',coadds)
			FXADDPAR, save_header, "COADTYPE", "AVERAGE", "Pixel values are the average, not sum of coadds"
		endif
		;
		;	read the raw1 image
		;
		FITS_Ave_RAW1_image = read_raw1(outpath)
		;
		writefits, FITS_filepath, FITS_Ave_RAW1_image, save_header
		;
		MSP_BDP_print_text, Event,"Write output image "+FITS_filepath
		(*s_ptr).P7_LED_colors[this_LED] = 'green'
		WCT_LED_stack, (*s_ptr).P7_LED_wid[0], (*s_ptr).P7_LED_colors
		;
		igood++
		;
		group_loop_end:
	endfor
	;
	P7_Save_Night_Summary, Event
	;
	P7SIC_end:
end

;-----------------------------------------------------------------
pro P7_Save_Night_Summary, Event
	;
	;	saves an updated night summary
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	;
	this_night = (*n_ptr).night_desig
	;
	;	which one in the summary?
	;
	num_nights = (*s_ptr).num_nights
	ind = WHERE(this_night eq (*s_ptr).night_date[0:num_nights-1], count)
	if(count ne 0) then night_index = ind[0] else begin
		MSP_BDP_print_text, Event, "Error - No matching night in P7..."
		goto, P7SNS_end
	endelse
	;
	(*s_ptr).night_who[night_index] = (*s_ptr).who
	(*s_ptr).night_step[night_index] = 7
	text = STRTRIM(STRING(format='(i)',(*a_ptr).n_groups),2)+" Ave RAW1s Made"
	(*s_ptr).night_comment[night_index] = text
	;
	P1_Write_Summary_File, Event
	;
	P7SNS_end:
end
;
