;
;	Part 8 Event Handler Routines - Apply Linearity Correction
;
;	[this version *also* creates FITS files for each raw1 image,
;		where useful information is saved in the FITS headers.]
;
;-----------------------------------------------------------------
;
pro P8_View_Linearity_Tables, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	;
	;	show the registered linearity tables
	;
	MSP_BDP_print_text, Event, "List Registered Linearity Tables requested"
	;
	;	if there are no tables, look to see if some exist in this
	;	reduced data filesystem
	;
	n_lin_tables = (*l_ptr).n_lin_tables
	if(n_lin_tables eq 0) then begin
		itables = 0
		search_dir = (*s_ptr).root_reduced + '6_Linearity_Tables'+PATH_SEP()
		;
		;	first, test that this directory exists
		;
		test = FILE_INFO(search_dir)
		if(test.exists and test.directory) then begin
			;
			;	exists and is a directory
			;	 look for nightly directories
			;
			night_list = FILE_SEARCH(search_dir+'20??????',/MARK_DIRECTORY,count=count)
			if(count ne 0) then n_nights = N_ELEMENTS(night_list) else n_nights=0
			;
			;print, "Number of nights, nights = ",n_nights, night_list
			;
			;	for each night, get a linearity table first file
			;
			if(n_nights ne 0) then begin
				for inight = 0, n_nights-1 do begin
					night_dir = night_list[inight]
					;
					lin_list = FILE_SEARCH(night_dir, "*_Lin_0.fits",count=count)
					if(count ne 0) then n_lin_list = N_ELEMENTS(lin_list) else n_lin_list = 0
					if(n_lin_list eq 1) then begin
						;
						;print,"Lin_0 file = ",lin_list[0]
						;
						;	add to list
						;
						(*l_ptr).lin_table_0_path[ (*l_ptr).n_lin_tables ] = lin_list[0]
						(*l_ptr).n_lin_tables++
						itables++
					endif
				endfor
			endif
			MSP_BDP_print_text, Event, "Loaded "+STRING(format='(I2.1)',itables)+" linearity tables from this run"
		endif
	endif
	;
	P6_List_All_Tables, Event
	;
	;	if no tables registered, tell user to go back to part 6
	;
	if( (*l_ptr).n_lin_tables eq 0) then begin
		MSP_BDP_print_text, Event, "ERROR - No Linearity Tables Registered. Go back to Part 6"
		goto, P8VLT_end
	endif
	;
	;	turn on droplist
	;
	WIDGET_CONTROL, (*s_ptr).P8_droplist_night_wid[0], SENSITIVE=1
	;
	P8VLT_end:
end
;
;-----------------------------------------------------------------------
pro P8_Select_Night, Event
	;
	MSP_BDP_print_text, Event, "Night Selection Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	b_ptr = PTR_struct.PTR_BPM
	lc_ptr = PTR_struct.PTR_LC
	;
	wid = (*s_ptr).P8_droplist_night_wid[0]
	;
	night_index = Event.index
	;
	;	index is not enough, as we might not have done all the part 2
	;	stuff.... So, need to get the night designation from the list
	;
	WIDGET_CONTROL, wid, GET_VALUE = vector_night_desig
	selected_night_desig = vector_night_desig[night_index]
	;
	night_desig = selected_night_desig
	night_text = STRING(format='(I8.8)',night_desig)
	;
	;	reset buttons, and LEDs
	;
	for iw = 0, N_ELEMENTS( (*s_ptr).P8_button_wid)-1 do begin
		if(iw le 0) then sense = 1 else sense = 0
		P8_button_sensitize, Event, iw, sense
		;if(iw le 1) then P5_base_sensitize, Event, iw, 0
		if(iw eq 0) then P8_LED_sensitize, Event, iw, 0
	endfor
	;
	;	read in that night's save file
	;
	MSP_BDP_print_text, Event, "Opening Night Summary for night = "+night_text
	;
	MSP_BDP_Create_Night_File_Path, Event, night_desig, night_file_path
	MSP_BDP_Restore_Night_File, Event, night_file_path, description, ierr
	;
	if(ierr ne 0) then begin
		MSP_BDP_print_text, Event, "Could not restore night "+night_text+" - skipping"
		goto, P8SN_end
	endif
	;
	;	must redo pointers
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	raw_root = (*s_ptr).root_raw
	;
	;	capture number of groups
	;
	ngroups = (*n_ptr).num_groups
	ngroups_text = STRING(format='(I)',ngroups)
	MSP_BDP_print_text, Event, "Restored "+night_text+" with "+ngroups_text+" groups"
	;
	;	turn on first button
	;
	P8_button_sensitize, Event, 1, 1
	;
	;	and reset view flag to zero
	;
	(*lc_ptr).view_flag = 0L
	;
	;WIDGET_CONTROL, (*s_ptr).P5_droplist_night_wid[1], SENSITIVE=0		; TBD off for now
	;
	P8SN_end:
end
;
;------------------------------------------------------
pro P8_View_Group_Table, Event
	;
	;	show the table of groups needing linearity correction
	;		( ie, not BIAS or Linearity groups)
	;
	MSP_BDP_print_text, Event, "Show/Edit Groups for Linearity Correction requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	r_ptr = PTR_struct.PTR_RAW1
	lc_ptr = PTR_struct.PTR_LC
	;
	night_desig = (*n_ptr).night_desig
	night_desig_text = STRING(format='(I8.8)',night_desig)
	;
	root_raw = (*s_ptr).root_raw
	;
	n_images = (*n_ptr).num_images
	file_list = (*n_ptr).image_list[0:n_images-1]
	image_type = (*n_ptr).image_type[0:n_images-1]
	image_frame = (*n_ptr).image_frame[0:n_images-1]
	coadds = (*n_ptr).image_coadds[0:n_images-1]
	;
	input_pathname_list = strarr(n_images)
	;
	;	1. Find and save the info for groups in lc structure, if view flag = 0, else skip
	;
	view_flag = (*lc_ptr).view_flag
	;
	n_groups = (*n_ptr).num_groups
	n_lc = 0L
	;
	for ig = 0, n_groups-1 do begin
		;
		;	is this group a BIAS group? (or a 0 sec Dark?)
		;
		this_group_type = (*n_ptr).groups[ig].group_type
		;
		;	is it subframed?
		;
		this_group_frame = (*n_ptr).groups[ig].group_frame
		test_3 = STRUPCASE(this_group_frame) eq 'FULL'
		;
		tgt_UC = STRUPCASE(this_group_type)
		test_1 = (tgt_UC  eq 'BIAS')					; flag bias groups
		test_2 = (STRPOS(tgt_UC, "LIN")	ne -1)			; linearity images
		;
		if(test_1 or test_2 or NOT(test_3)) then goto, group_loop_end
		;
		;	this group is not a bias group, add its info to the lc structure
		;
		group_id = (*n_ptr).groups[ig].group_id
		group_images = (*n_ptr).groups[ig].group_images
		image_list = (*n_ptr).groups[ig].group_list[0:group_images-1]
		group_coadds = (*n_ptr).groups[ig].group_coadds
		;
		;	UT
		;
		image_no = image_list[0]
		ind = WHERE( (*n_ptr).image_list[0:n_images-1] eq image_no, count)
		if(count ne 1) then begin
			MSP_BDP_print_text, Event, "ERROR - no match of image no in P8 UT load. image_no = "+STRING(format='(i)',image_no)
			goto, group_loop_end
		endif
		;
		ut = (*n_ptr).header_list[ind[0]].ut
		;
		(*lc_ptr).group[n_lc].ut = ut
		;
		;print, "first call to UT = ",ut
		;
		(*lc_ptr).n_groups = n_lc + 1
		(*lc_ptr).this_group_no = n_lc
		(*lc_ptr).group[n_lc].serial_no = n_lc
		(*lc_ptr).group[n_lc].object = (*n_ptr).groups[ig].group_object
		if(NOT(view_flag)) then (*lc_ptr).group[n_lc].type = this_group_type
		(*lc_ptr).group[n_lc].output_FITS_name = night_desig_text		; just the subdirectory
		(*lc_ptr).group[n_lc].output_RAW1_name = ''
		(*lc_ptr).group[n_lc].night_desig = night_desig
		(*lc_ptr).group[n_lc].group_id = group_id
		(*lc_ptr).group[n_lc].n_images = group_images
		(*lc_ptr).group[n_lc].coadds = group_coadds
		if(NOT(view_flag)) then (*lc_ptr).group[n_lc].reg_lin_no = 1L			; point to the first linearity table registered - default
		;
		;	image pathnames, resolved as to raw directory
		;
		for ii = 0, group_images-1 do begin
			;
			ind = WHERE( (*n_ptr).image_list[0:n_images-1] eq image_list[ii],count)
			if(count ne 1) then begin
				MSP_BDP_print_text, Event, "ERROR - no match of image no in P8 pathname load. image_no = "+STRING(format='(i)',image_list[ii])
				goto, group_loop_end
			endif
			;
			index = (*n_ptr).image_root_index[ind[0]]
			if(index eq 0) then begin
				base_dir = (*s_ptr).root_raw
			endif else begin
				base_dir = (*s_ptr).root_reduced
			endelse
			;
			cds_directory = base_dir + (*n_ptr).paths_cds_root[index]
			raw1_directory = base_dir + (*n_ptr).paths_raw1_root[index]
			;
			image_no_text = STRTRIM(STRING(format='(i5.3)',image_list[ii]),2)
			cds_path = fix_path( FILEPATH(night_desig_text+"."+image_no_text+".fits", ROOT_DIR=cds_directory) )
			raw1_path = fix_path( FILEPATH('raw1_'+image_no_text,ROOT_DIR=raw1_directory) )
			;
			;print, "CDS path = ",cds_path
			;print, "RAW1 path = ",raw1_path
			;
			(*lc_ptr).group[n_lc].image_CDS_pathname[ii] = cds_path
			(*lc_ptr).group[n_lc].image_RAW1_pathname[ii] = raw1_path
		endfor
		;
		;	increment counter
		;
		n_lc++
		;
		MSP_BDP_print_text, Event, "For Linearity Correct "+STRING(format='(i)',n_lc)+$
				" found "+STRING(format='(i)',group_images)+" images"
		;
		group_loop_end:
	endfor
	;
	(*lc_ptr).n_groups = n_lc
	;
	;	if there are no images to build, then write summary and exit
	;
	if(n_lc eq 0) then begin
		MSP_BDP_print_text, Event, "No Bias groups found to build Linearity Corrected images from."
		num_nights = (*s_ptr).num_nights
		ind = WHERE(night_desig eq (*s_ptr).night_date[0:num_nights-1], count)
		if(count ne 0) then night_index = ind[0] else begin
			MSP_BDP_print_text, Event, "Error - No matching night in P8..."
			goto, P8_ST_end
		endelse
		;
		(*s_ptr).night_step[night_index] = 8
		text = "Struct. Loaded"
		(*s_ptr).night_comment[night_index] = text
		;
		P1_Write_Summary_File, Event
		;
		;P5_Draw_Done, Event, TEXT=text
		;
		goto, P8_ST_end
	endif
	;
	;	OK, we are going to build images. Check existence of output directory
	;	and create it if needed.
	;
	(*lc_ptr).output_directory = '8_Linearity_Corrected_Images'+PATH_SEP()+night_desig_text
	lc_dir_path = fix_path( (*s_ptr).root_reduced + (*lc_ptr).output_directory )
	;
	;	see if this directory exists
	;
	test_dir = FILE_INFO( lc_dir_path )
	if( NOT(test_dir.exists) or NOT(test_dir.directory) ) then begin
		MSP_BDP_print_text, Event, "Creating new Linearity Corrected directory: "+lc_dir_path
		FILE_MKDIR, lc_dir_path
	endif
	;------------------------------------------------------------------------------------
	;
	;	build the table structure
	;
	col_headers = ['Group', '_____Object_____','__Type__', '____UT____', 'N_Images', '1st Image', $
					'Last Image','CoAdd','Lin Table No.']
	;
	n_lines = (*lc_ptr).n_groups
	format = strarr(9,n_lines)
	format[0:8,*] = '(a)'
	;
	color_array = strarr(9,n_lines)
	;
	row_headers = strarr(n_lines)
	data_line = {group_id: '', object: '', type: '', UT: '', n_images: '',$
				 first: '', last: '', coadd: '', reg_lin_no:''}
	data_array = REPLICATE(data_line, n_lines)			; array of structures
	;
	for i = 0, n_lines-1 do begin
			;
			row_headers[i] = 'LC_'+STRTRIM(STRING(format='(i4.1)',i),2)
			;
			g_text = STRTRIM(STRING(format='(i4.1)',(*lc_ptr).group[i].group_id),2)
			data_array[i].group_id    = 'Gr_'+g_text
			data_array[i].object = (*lc_ptr).group[i].object
			data_array[i].type = (*lc_ptr).group[i].type
			data_array[i].coadd = STRTRIM(STRING(format='(i3)',(*lc_ptr).group[i].coadds),2)
			;
			;	convert UT to string
			;
			ut = (*lc_ptr).group[i].ut
			;
			;print, "UT = ",ut
			;
			uth = fix(ut)
			utm = fix( 60.0*(ut - uth))
			uts = ut*3600.0d0 - double(uth)*3600.0d0 - double(utm)*60.0d0
			utsx = fix(uts/10.0)
			uts = uts - 10.0*utsx
			ut_text = STRING(format='(i2.2,":",i2.2,":",i1.1,f3.1)',uth,utm,utsx,uts)
			;
			data_array[i].UT    		= ut_text
			data_array[i].n_images    	= STRTRIM(STRING(format='(i3.1)',(*lc_ptr).group[i].n_images),2)
			;
			;	get first, last image numbers in this group
			;
			this_group = (*lc_ptr).group[i].group_id
			n_images = (*lc_ptr).group[i].n_images
			first = (*n_ptr).groups[this_group].group_list[0]
			last = (*n_ptr).groups[this_group].group_list[n_images-1]
			;
			data_array[i].first = STRTRIM(STRING(format='(i5.1)',first),2)
			data_array[i].last = STRTRIM(STRING(format='(i5.1)',last),2)
			data_array[i].reg_lin_no = STRTRIM(STRING(format='(i5.1)', (*lc_ptr).group[i].reg_lin_no),2)
			;
			color_array[*,i] = "white"
			;
			case data_array[i].type of
				"Skip": color = 'red'
				"**BAD**":	color = "red"
				"Bias":	color = "skyblue"
				"Dark": color = "grey68"
				"Pol-Img": color = "lawngreen"
				"Pol-Spec": color = "pink"
				"Spec": color = "plum"
				"Image": color = "yellow"
				"S-Argon": color = "slateblue"
				"Lin-On": color = "khaki light"
				"Lin-Off": color = "khaki dark"
				"I-Flat-On": color = "goldenrod light"
				"I-Flat-Off": color = "goldenrod dark"
				"S-Flat-On": color = "rosybrown light"
				"S-Flat-Off": color = "rosybrown dark"
				"P-Flat-On": color = "palegreen light"
				"P-Flat-Off": color = "palegreen dark"
				"PCal-On": color = "palegreen light"
				"PCal-Off": color = "palegreen dark"
				"R-Flat-Off": color = "steelblue dark"
				"R-Flat-On": color = "steelblue light"
				else: color = "white"
			endcase
			;
			color_array[*,i] = color
			;
	endfor
	;
	r_string = STRING(format='("Linearity Correct Group List for Night ",I8.8)', (*n_ptr).night_desig)
	if( (*s_ptr).default_font ne 'None') then begin
    new_struct = Dialog_Big_Table(Title="Linearity Correct Group Table"+$
    		'     can change linearity table number or group types via their entries',Label=r_string, $
					Group_Leader=Event.top, Col_Headers = col_headers, $
					Format=format, Row_headers = row_headers,$
					Data_array = data_array, Font='Helvetica*16', $
					color_array=color_array)
	endif else begin
	 new_struct = Dialog_Big_Table(Title="Linearity Correct Group Table"+$
	    '     can change linearity table number or group types via their entries',Label=r_string, $
	    Group_Leader=Event.top, Col_Headers = col_headers, $
	    Format=format, Row_headers = row_headers,$
	    Data_array = data_array, Font='None', $
	    color_array=color_array)
	endelse
	;
	;	load new table data back into structure
	;
	nc = N_TAGS(new_struct)
	nr = N_ELEMENTS(new_struct)
	;
	;	note - permit change of linearity table reg no and group type
	;
	for ir = 0, nr-1 do begin
		(*lc_ptr).group[ir].reg_lin_no = 	new_struct[ir].reg_lin_no
		(*lc_ptr).group[ir].type = new_struct[ir].type
	endfor
	;
	;	set view flag
	;
	(*lc_ptr).view_flag = 1L
	;
	P8_button_sensitize, Event, 2, 1
	;
	P8_ST_end:
end
;
;---------------------------------------------------
pro P8_Start_Linearity_Correction, Event
	;
	;	performs Linearity Correction
	;
	MSP_BDP_print_text, Event, "Starting Linearity Correction"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	lc_ptr = PTR_struct.PTR_LC
	n_ptr = PTR_struct.PTR_night
	b_ptr = PTR_struct.PTR_BPM
	a_ptr = PTR_struct.PTR_Ave_RAW1
	;
	;	number of groups to process
	;
	n_groups = (*lc_ptr).n_groups
	;
	;	determine number of images to process
	;
	n_images = 0L
	for ig = 0, n_groups-1 do begin
		if( (*lc_ptr).group[ig].type ne '**BAD**' and (*lc_ptr).group[ig].type ne "Skip") then begin
			this_group_n_images = (*lc_ptr).group[ig].n_images
			n_images = n_images + this_group_n_images
		endif
	endfor
	;
	MSP_BDP_print_text, Event, "Will Linearity Correct "+STRTRIM(STRING(format='(i5.1)',n_images),2)+" images"
	;
	;	clear LED progress bar to yellow
	;
	n_LEDs = N_ELEMENTS( (*s_ptr).P8_LED_colors )
	LED_scale = float(n_images) / float(n_LEDs)
	if(LED_scale le 1.0) then LED_scale = 1.0
	(*s_ptr).P8_LED_colors[*] = 'yellow'
	if(n_images lt n_LEDs) then (*s_ptr).P8_LED_colors[n_images:n_LEDs-1] = 'black'
	WCT_LED_stack, (*s_ptr).P8_LED_wid[0], (*s_ptr).P8_LED_colors
	;
	c_images = 0L
	night_desig_text = STRING(format='(I8.8)',(*lc_ptr).group[ig].night_desig)
	;
	;	start by fetching BPM directory list
	;
	MSP_BDP_Read_BPM_Headers, Event
	;
	;	how many did we load? If none, issue message and leave
	;
	n_bpm = (*b_ptr).bpm_n_BPM
	if(n_bpm eq 0) then begin
		MSP_BDP_print_text, Event, "ERROR - No Bad Pixel Masks found - go back to Part 5"
		goto, P8SLC_outer_end
	endif
	;
	;	and the Ave RAW1 list
	;
	MSP_BDP_Read_Ave_RAW1_Headers, Event
	n_ave_raw1 = (*a_ptr).n_groups
	if(n_ave_raw1 eq 0) then begin
		MSP_BDP_print_text, Event, "ERROR - No Ave RAW1 Images found - go back to Part 7"
		goto, P8SLC_outer_end
	endif
	;
	;	for every group
	;
	lin_table_no = 0
	for ig = 0, n_groups -1 do begin
		;
		;	if the group type is BAD, skip over it
		;
		if( (*lc_ptr).group[ig].type eq '**BAD**' or (*lc_ptr).group[ig].type eq "Skip") then goto, group_end
		;
		;	for this group, if no linearity files open, then open and read
		;	the ones indicated. If a set is open, and this next group requests
		;	same ones, then don't reopen. If the this group requests another
		;	linearity table set, open and read them.
		;
		this_lin_table_no = (*lc_ptr).group[ig].reg_lin_no
		if(lin_table_no eq 0 or (lin_table_no ne this_lin_table_no)) then begin
			;
			;	read indicated lin table into memory
			;
			P8_Read_Linearity_Tables, Event, this_lin_table_no
			;
			lin_table_no = this_lin_table_no
		endif
		;
		;	get night, UT for This Image's Data
		;
		this_group = (*lc_ptr).group[ig].group_id
		lc_night = (*lc_ptr).group[ig].night_desig
		this_group_filter = (*n_ptr).groups[this_group].group_filter
		;
		;	write group number to text window
		;
		WIDGET_CONTROL, (*s_ptr).P8_text_wid[0], SET_VALUE=STRING(format='(I5.1)',ig)
		;
		;	for every image in the group
		;
		night_desig = (*lc_ptr).group[ig].night_desig
		night_desig_text = STRTRIM(STRING(format='(I8.8)',night_desig),2)
		;
		for i_image = 0, (*lc_ptr).group[ig].n_images-1 do begin
			;
			this_LED = float(c_images) / LED_scale
			this_LED = 0 > (this_LED < (n_LEDs-1))
			(*s_ptr).P8_LED_colors[this_LED] = 'orange'
			WCT_LED_stack, (*s_ptr).P8_LED_wid[0], (*s_ptr).P8_LED_colors
	        ;
			;	next, fetch the appropriate bad pixel mask, based on night and UT
			;	proximity
			;
			image_CDS_path = fix_path( (*lc_ptr).group[ig].image_CDS_pathname[i_image] )
			;
			;	get coadd number
			;
			group_coadds = (*lc_ptr).group[ig].coadds
			;
			image_name = FILE_BASENAME(image_CDS_path)
			image_no = 0L
			c_start = STRPOS(image_name, '.')
			c_end = STRPOS(image_name,'.fits')
			;
			image_desig = STRMID(image_name, c_start+1, (c_end - c_start - 1))
			reads, image_desig, format='(i)',image_no
			;
			;print, "Image name, no = ",image_name, image_no
			;
			;	match to night's list to get to FITS header in memory
			;
			n_night_images = (*n_ptr).num_images
			ind = WHERE( (*n_ptr).image_list[0:n_night_images-1] eq image_no, count)
			if(count ne 1) then begin
				MSP_BDP_print_text, Event, "ERROR - didn't match image "+image_name
				goto, P8SLC_end
			endif
			;
			this_image_UT = (*n_ptr).header_list[ind[0]].ut
			;
			;	match date, time to BPM, AveRAW1 lists
			;		calc julian date of this observation
			;
			year = night_desig / 10000L
			month = (night_desig - 10000L * year) / 100L
			day = night_desig - 10000L * year - 100L * month
			;
			hour = FLOOR(this_image_UT)
			minute = FLOOR((this_image_UT - double(hour))*60.0d0)
			second = 3600.0d0 * (this_image_UT - double(hour) - double(minute)/60.0d0)
			;
			this_jd = JULDAY(month, day, year, hour, minute, second)
			;
			;print, "Image JD = ",this_jd
			;
			d_jd_min = 1.0e10
			jbpm = 0
			for ibpm = 0, n_bpm-1 do begin
				;
				;	get JD of BPM
				;
				bpm_jd = (*b_ptr).bpm_jd[ibpm]
				;
				;	difference
				;
				dif_jd = abs(bpm_jd - this_jd)
				;
				if(dif_jd lt d_jd_min) then begin
					d_jd_min = dif_jd
					jbpm = ibpm
				endif
			endfor
			;
			BPM_path = (*b_ptr).bpm_image_name[jbpm]
			;
			;print, "Closest night-JD BPM match is number ",jbpm," path = ",BPM_path
			;print, "  smallest d_UT = ",d_jd_min, " at JD = ",(*b_ptr).bpm_jd[jbpm]
			;
			;MSP_BDP_print_text, Event, "  BPM file selected = "+FILE_BASENAME(BPM_path)
			;
			;	same for Ave RAW1
			;
			d_jd_min = 1.0e10
			jar = 0
			for iar = 0, n_ave_raw1-1 do begin
				;
				;	get JD of ave raw1
				;
				ar_jd = (*a_ptr).group[iar].jd
				;
				;	difference
				;
				dif_jd = abs(ar_jd - this_jd)
				;
				;	but, also match coadds
				;
				raw1_coadds = (*a_ptr).group[iar].coadds
				;
				if(dif_jd lt d_jd_min and raw1_coadds eq group_coadds) then begin
					d_jd_min = dif_jd
					jar = iar
				endif
			endfor
			;
			AR_path = (*a_ptr).group[jar].image_CDS_pathname[0]
			;
			;print, "Closest night-JD Ave_RAW1 match is number ",jar," path = ",AR_path
			;print, "  smallest d_UT = ",d_jd_min, " at JD = ",(*a_ptr).group[jar].jd
			;
			;
			;MSP_BDP_print_text, Event, "  Ave RAW1 file selected = "+FILE_BASENAME(AR_path)
			;
			;	summarize input files
			;
			CDS_path = image_CDS_path									; if coadds = SUM
			RAW1_path = (*lc_ptr).group[ig].image_RAW1_pathname[i_image]; if coadds = SUM
			;
			;print, "CDS_path = ",CDS_path
			;print, "RAW1_path = ",RAW1_path
			;
			BPM_path = BPM_path
			AR_path = AR_path											; if coadds = AVERAGE
			;
			Lin_0_path = (*l_ptr).lin_table_0_path[ (*lc_ptr).group[ig].reg_lin_no -1 ]
			;
			;	output file
			;
			out_path = fix_path( (*s_ptr).root_reduced + "8_Linearity_Corrected_Images"+PATH_SEP()+$
						night_desig_text+PATH_SEP()+night_desig_text+'.'+$
						STRTRIM(STRING(format='(I5.3)',image_no),2)+'_LC.fits' )
			;
			;	linearity correction file
			;
			(*s_ptr).P8_LED_colors[this_LED] = 'blue'
			WCT_LED_stack, (*s_ptr).P8_LED_wid[0], (*s_ptr).P8_LED_colors
			;
			MSP_BDP_print_text, Event, "Starting Linearity Correction of "+FILE_BASENAME(out_path)
			;
			linearity_correct, Event, CDS_path, RAW1_path, AR_path, BPM_path, $
								 corrected_image, corrected_header, group_coadds
			;
			;	update header and write to disk after checking filter status
			;
			Test_Fix_Bad_Filter_Names, Event, this_group_filter, HEADER=corrected_header
			;
			(*s_ptr).P8_LED_colors[this_LED] = 'violet'
			WCT_LED_stack, (*s_ptr).P8_LED_wid[0], (*s_ptr).P8_LED_colors
			;
			FXADDPAR, corrected_header, "FILENAME", FILE_BASENAME(out_path), "Linearity Corrected Image"
			FXADDPAR, corrected_header, "HISTORY","  Linearity Corrected on "+SYSTIME()
			FXADDPAR, corrected_header, "HISTORY","    reduced by "+(*s_ptr).who
			FXADDPAR, corrected_header, "HISTORY", "    using Linearity Correction images starting with:"
			FXADDPAR, corrected_header, "HISTORY", "      "+FILE_BASENAME(Lin_0_path)
			FXADDPAR, corrected_header, "HISTORY", "  and Ave RAW1 image "+FILE_BASENAME(AR_path)
			FXADDPAR, corrected_header, "HISTORY", "  and Bad Pixel Mask "+FILE_BASENAME(BPM_path)
			if(group_coadds ne 1) then begin
				FXADDPAR, corrected_header, "HISTORY", "  all while respecting COADDS= "+STRING(format='(I3)',group_coadds)
				FXADDPAR, corrected_header, "COADTYPE", "SUM", "Pixel values are sum of coadded images"
			endif
			;
			writefits, out_path, corrected_image, corrected_header
			;
			MSP_BDP_print_text, Event,"Wrote output image "+out_path
			;
			(*s_ptr).P8_LED_colors[this_LED] = 'green'
			WCT_LED_stack, (*s_ptr).P8_LED_wid[0], (*s_ptr).P8_LED_colors
			;
			P8SLC_end:
			;
			c_images++
		endfor
		;
		group_end:
	endfor
	;
	;	release memory used by linearity tables
	;
	PTR_FREE, (*l_ptr).lin_table_ptr
	;
	P8_Save_Night_Summary, Event, c_images
	;
	P8SLC_outer_end:
end
;
;----------------------------------------------------------------
pro P8_Read_Linearity_Tables, Event, lin_reg_no
	;
	MSP_BDP_print_text, Event, "Read Linearity Table Set "+STRING(format='(I4.1)',lin_reg_no)+" Requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	lc_ptr = PTR_struct.PTR_LC
	n_ptr = PTR_struct.PTR_night
	b_ptr = PTR_struct.PTR_BPM
	a_ptr = PTR_struct.PTR_Ave_RAW1
	;
	;	get the lead name's path after checking that we aren't outside list
	;
	max_reg_no = (*l_ptr).n_lin_tables
	if(lin_reg_no gt max_reg_no) then begin
		MSP_BDP_print_text, Event, "ERROR - tried to read linearity table number beyond those registered."
		goto, P8RLT_end
	endif
	;
	lin_0_path = fix_path( (*l_ptr).lin_table_0_path[lin_reg_no - 1] )
	;
	;	check that it exists
	;
	test = FILE_INFO(lin_0_path)
	if(NOT(test.exists)) then begin
		MSP_BDP_print_text, Event, "ERROR - linearity table not found. path = "+lin_0_path
		goto, P8RLT_end
	endif
	;
	;	start loop of reading linearity table into memory structure
	;
	(*l_ptr).lin_table_ptr = PTR_NEW(fltarr(9,1024,1026))
	(*(*l_ptr).lin_table_ptr)[0,*,*] = readfits(lin_0_path, header, /silent)
	;
	;	read in the remaining tables
	;
	for itable = 1, 8 do begin
		;
		start = STRPOS(lin_0_path, 'Lin_0')
		lin_n_path = lin_0_path
		STRPUT, lin_n_path, STRING(format='(i1.1)',itable), (start+4)
		;
		;print, "reading table ",lin_n_path
		;
		(*(*l_ptr).lin_table_ptr)[itable,*,*] = readfits(lin_n_path, header, /silent)
	endfor
	;
	;	pointer has everything, now
	;
	P8RLT_end:
end
;-----------------------------------------------------------------
pro P8_Save_Night_Summary, Event, n_images
	;
	;	saves an updated night summary
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	;
	this_night = (*n_ptr).night_desig
	;
	;	which one in the summary?
	;
	num_nights = (*s_ptr).num_nights
	ind = WHERE(this_night eq (*s_ptr).night_date[0:num_nights-1], count)
	if(count ne 0) then night_index = ind[0] else begin
		MSP_BDP_print_text, Event, "Error - No matching night in P8..."
		goto, P8SNS_end
	endelse
	;
	(*s_ptr).night_who[night_index] = (*s_ptr).who
	(*s_ptr).night_step[night_index] = 8
	text = STRTRIM(STRING(format='(i4.1)',n_images),2)+" Lin. Corr. Done"
	(*s_ptr).night_comment[night_index] = text
	;
	P1_Write_Summary_File, Event
	;
	P8SNS_end:
end
;
