;
;	Part 9 Event Handler Routines - Make Dark Images
;
;-----------------------------------------------------------------
;
pro P9_Discover_Dark_Groups, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	n_ptr = PTR_struct.PTR_night
	s_ptr = PTR_struct.PTR_summary
	d_ptr = PTR_struct.PTR_Dark
	;
	;	from the list of possible nights with dark data, build the list
	;	of groups of dark data
	;
	MSP_BDP_print_text, Event, "Discover Dark Groups Requested"
	;
	;	how many nights?
	;
	n_nights = (*d_ptr).n_nights
	if(n_nights le 0) then begin
		MSP_BDP_print_text, Event, "ERROR - no nights with Dark Groups. Skipping."
		goto, P9DDG_end
	endif
	;
	;	for every night
	;
	n_dark_groups = 0L			; counter
	;
	for inight = 0, n_nights-1 do begin
		;
		;	fetch night's desig
		;
		this_night_desig = (*d_ptr).night_list[inight]
		this_night_desig_text = STRING(format='(I8.8)',this_night_desig)
		;
		;	load this night's SAV file
		;
		MSP_BDP_print_text, Event, "Opening Night Summary for night = "+this_night_desig_text
		;
		MSP_BDP_Create_Night_File_Path, Event, this_night_desig, night_file_path
		MSP_BDP_Restore_Night_File, Event, night_file_path, description, ierr
		;
		if(ierr ne 0) then begin
			MSP_BDP_print_text, Event, "Could not restore night "+this_night_desig_text+" - skipping"
			goto, night_loop_end
		endif
		;
		;	must redo pointers
		;
		WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
		s_ptr = PTR_struct.PTR_summary
		n_ptr = PTR_struct.PTR_night
		;
		;	capture number of groups
		;
		ngroups = (*n_ptr).num_groups
		ngroups_text = STRTRIM(STRING(format='(I3.1)',ngroups),2)
		MSP_BDP_print_text, Event, "Restored "+this_night_desig_text+" with "+ngroups_text+" groups"
		;
		;	for every group, grab every "Dark" group - note that we use Dark's to do the job
		;		of "Bias" images in removing ski-jumps, as well
		;
		if(ngroups le 0) then begin
			MSP_BDP_print_text, Event, "ERROR - no groups for this night. Skipping."
			goto, night_loop_end
		endif
		;
		;	set the directory path
		;
		dir_path = fix_path( (*s_ptr).root_reduced + '8_Linearity_Corrected_Images' + PATH_SEP() + $
					this_night_desig_text + PATH_SEP() )
		;
		for ig = 0, ngroups-1 do begin
			;
			;	get group type and compare to Dark
			;
			this_group_type = (*n_ptr).groups[ig].group_type
			tgt_UC = STRUPCASE(this_group_type)
			test_1 = tgt_UC eq 'DARK'
			;
			;	and make sure it is not subframed (for now)
			;
			test_2 = STRUPCASE((*n_ptr).groups[ig].group_frame) eq 'FULL'
			;
			if(test_1 and test_2) then begin
				;
				;	save crucial information
				;
				(*d_ptr).night_desig[n_dark_groups] = this_night_desig
				(*d_ptr).night_group_id[n_dark_groups] = (*n_ptr).groups[ig].group_id
				(*d_ptr).exptime[n_dark_groups] = (*n_ptr).groups[ig].group_exptime
				;
				;	for each image in this group, build the full pathname to the _LC image
				;
				n_images = (*n_ptr).groups[ig].group_images
				(*d_ptr).n_input_pathnames[n_dark_groups] = n_images
				;
				for i_image = 0, n_images-1 do begin
					;
					;	get the image number from the night's group list
					;
					image_no = (*n_ptr).groups[ig].group_list[i_image]
					if(i_image eq 0) then image_no_0 = image_no
					;
					;	build name of image
					;
					filename = this_night_desig_text + '.' + $
							   STRTRIM(STRING(format='(i5.3)',image_no),2) + '_LC.fits'
					;
					;	full pathname
					;
					pathname = fix_path( FILEPATH(filename, ROOT_DIR = dir_path) )
					;
					(*d_ptr).input_pathname[n_dark_groups,i_image] = pathname
					;
					;print, "input file, dir, path = ",filename, dir_path, pathname
					;
				endfor
				;
				;	build the JD for the first image
				;
				n_header_night = (*n_ptr).num_images
				ind = WHERE( (*n_ptr).image_list[0:n_header_night-1] eq image_no_0,count)
				if(count ne 1) then begin
					MSP_BDP_print_text, Event, "ERROR - did not match image no to night list"
					goto, P9DDG_end
				endif
				;
				this_image_UT = (*n_ptr).header_list[ind[0]].ut
				;
				year = this_night_desig / 10000L
				month = (this_night_desig - 10000L * year) / 100L
				day = this_night_desig - 10000L * year - 100L * month
				;
				hour = FLOOR(this_image_UT)
				minute = FLOOR((this_image_UT - double(hour))*60.0d0)
				second = 3600.0d0 * (this_image_UT - double(hour) - double(minute)/60.0d0)
				;
				this_jd = JULDAY(month, day, year, hour, minute, second)
				;
				(*d_ptr).input_first_JD[n_dark_groups] = this_jd
				;
				;	build the output pathname for the group
				;
				out_dir = (*s_ptr).root_reduced + '9_Dark_Images' + PATH_SEP()
				out_file = 'Drk_' + STRING(format='(I6.6)',1000.0*(*d_ptr).exptime[n_dark_groups]) + $
							'_' + this_night_desig_text + '_g' + STRTRIM(STRING(format='(i4.1)',$
							(*d_ptr).night_group_id[n_dark_groups]),2) + '.fits'
				;
				outpath = fix_path( FILEPATH(out_file, ROOT_DIR= out_dir) )
				(*d_ptr).output_pathname[n_dark_groups] = outpath
				;
				;print, "output file, dir, path = ",out_file, out_dir, outpath
				;
				;	bump counter
				;
				n_dark_groups++
			endif else begin
				;
				;print," Rejected group of type = ",this_group_type
				;
			endelse
			;
		endfor
		;
		night_loop_end:
		;
	endfor
	;
	;	update dark group counter
	;
	(*d_ptr).n_dark_groups = n_dark_groups
	;
	;	if no dark groups, say so and quit
	;
	if(n_dark_groups eq 0) then begin
		MSP_BDP_print_text, Event, "ERROR - No Dark Groups found. Skipping."
	endif else begin
		;
		;	turn on next button
		;
		P9_button_sensitize, Event, 1, 1
		;
	endelse
	;
	P9DDG_end:
end
;
;------------------------------------------------------
pro P9_Edit_Group_Table, Event
	;
	;	show the table of groups lined up for dark image creation
	;
	MSP_BDP_print_text, Event, "Show/Edit Groups for Dark Image Computation requested"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE = PTR_struct
	s_ptr = PTR_struct.PTR_summary
	n_ptr = PTR_struct.PTR_night
	r_ptr = PTR_struct.PTR_RAW1
	d_ptr = PTR_struct.PTR_Dark
	;
	view_flag = (*d_ptr).view_flag
	;
	n_dark_groups = (*d_ptr).n_dark_groups
	;
	;------------------------------------------------------------------------------------
	;
	;	build the table structure
	;
	col_headers = ['__Night__','Group', 'ExpT (ms)', 'N_Img', '_1st Image_', $
					'_Last Image_','___Output File Name___','Status']
	;
	ncols = N_ELEMENTS(col_headers)
	;
	n_lines = n_dark_groups
	format = strarr(ncols,n_lines)
	format[*,*] = '(a)'
	;
	color_array = strarr(ncols,n_lines)
	;
	row_headers = strarr(n_lines)
	data_line = {night_desig: '', group_id: '', exptime: '', n_images: '',$
				 first: '', last: '', output_filename:'', status:''}
	data_array = REPLICATE(data_line, n_lines)			; array of structures
	;
	for i = 0, n_lines-1 do begin
			;
			row_headers[i] = 'Drk_'+STRTRIM(STRING(format='(i4.1)',i),2)
			;
			night_desig_text = STRING(format='(I8.8)',(*d_ptr).night_desig[i])
			data_array[i].night_desig = night_desig_text
			;
			g_text = STRTRIM(STRING(format='(i4.1)',(*d_ptr).night_group_id[i]),2)
			data_array[i].group_id    = 'Gr_'+g_text
			;
			data_array[i].exptime = STRING(format='(I6.6)',1000.0*(*d_ptr).exptime[i])
			;
			data_array[i].n_images    	= STRTRIM(STRING(format='(i3.1)',(*d_ptr).n_input_pathnames[i]),2)
			;
			first = FILE_BASENAME((*d_ptr).input_pathname[i,0])
			if( (*d_ptr).n_input_pathnames[i] eq -1) then begin
				last = 'skipped'
			endif else begin
				last = FILE_BASENAME((*d_ptr).input_pathname[i,(*d_ptr).n_input_pathnames[i]-1])
			endelse
			;
			data_array[i].first = first
			data_array[i].last = last
			;
			data_array[i].output_filename = FILE_BASENAME((*d_ptr).output_pathname[i])
			;
			;	check to see if this image already exists
			;
			test = FILE_INFO((*d_ptr).output_pathname[i])
			if(test.exists) then data_array[i].status = 'Exists' else data_array[i].status = 'To Do'
			if((*d_ptr).n_input_pathnames[i] eq -1) then data_array[i].status = 'Skip'
			;
			case STRUPCASE(data_array[i].status) of
				'EXISTS': color = 'orange'
				'TO DO': color = 'green'
				'SKIP': color = 'pink'
				else: color = 'yellow'
			endcase
			;
			color_array[*,i] = color
			;
	endfor
	;
	r_string = "Dark Group List"
	if( (*s_ptr).default_font ne 'None') then begin
    new_struct = Dialog_Big_Table(Title="Dark Image Group Table"+$
    				"   to skip a group, put 'Skip' in the Status column",Label=r_string, $
					Group_Leader=Event.top, Col_Headers = col_headers, $
					Format=format, Row_headers = row_headers,$
					Data_array = data_array, Font='Helvetica*16', $
					color_array=color_array)
  endif else begin
    new_struct = Dialog_Big_Table(Title="Dark Image Group Table"+$
      "   to skip a group, put 'Skip' in the Status column",Label=r_string, $
      Group_Leader=Event.top, Col_Headers = col_headers, $
      Format=format, Row_headers = row_headers,$
      Data_array = data_array, Font='None', $
      color_array=color_array)
  endelse
	;
	;	load new table data back into structure
	;
	nc = N_TAGS(new_struct)
	nr = N_ELEMENTS(new_struct)
	;
	;	note - permit change of status
	;
	for ir = 0, nr-1 do begin
		if(STRUPCASE(new_struct[ir].status) ne 'TO DO') then begin
			(*d_ptr).n_input_pathnames[ir] = -1
		endif
	endfor
	;
	;	set view flag
	;
	(*d_ptr).view_flag = 1L
	;
	P9_button_sensitize, Event, 2, 1
	;
	P9_ST_end:
end
;
;---------------------------------------------------
pro P9_Compute_Dark_Images, Event
	;
	;
	MSP_BDP_print_text, Event, "Starting Dark Image Computations"
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	l_ptr = PTR_struct.PTR_Linearity
	d_ptr = PTR_struct.PTR_Dark
	n_ptr = PTR_struct.PTR_night
	b_ptr = PTR_struct.PTR_BPM
	a_ptr = PTR_struct.PTR_Ave_RAW1
	;
	;	number of groups to process
	;
	n_groups = (*d_ptr).n_dark_groups
	;
	n_good = 0L
	for ig = 0, n_groups -1 do begin
		;
		;	if this group is to be skipped, do so
		;
		if( (*d_ptr).n_input_pathnames[ig] ne -1) then n_good++
	endfor
	;
	;	clear LED progress bar to yellow
	;
	n_LEDs = N_ELEMENTS( (*s_ptr).P9_LED_colors )
	LED_scale = float(n_good) / float(n_LEDs)
	if(LED_scale le 1.0) then LED_scale = 1.0
	(*s_ptr).P9_LED_colors[*] = 'yellow'
	if(n_good lt n_LEDs) then (*s_ptr).P9_LED_colors[n_good:n_LEDs-1] = 'black'
	WCT_LED_stack, (*s_ptr).P9_LED_wid[0], (*s_ptr).P9_LED_colors
	;
	;	check that there are any to process
	;
	if(n_good eq 0) then begin
		MSP_BDP_print_text, Event, "No New Dark Images need to be computed. Skipping"
		goto, P9SLC_outer_end
	endif
	;
	;	start by fetching BPM directory list
	;
	MSP_BDP_Read_BPM_Headers, Event
	;
	;	how many did we load? If none, issue message and leave
	;
	n_bpm = (*b_ptr).bpm_n_BPM
	if(n_bpm eq 0) then begin
		MSP_BDP_print_text, Event, "ERROR - No Bad Pixel Masks found - go back to Part 5"
		goto, P9SLC_outer_end
	endif
	;
	;	for every group
	;
	i_good = 0L
	for ig = 0, n_groups -1 do begin
		;
		;	if this group is to be skipped, do so
		;
		if( (*d_ptr).n_input_pathnames[ig] eq -1) then goto, loop_end
		;
		;	get night, group, number of images for each group to turn into one Dark image
		;
		this_group = (*d_ptr).night_group_id[ig]
		this_night = (*d_ptr).night_desig[ig]
		n_input_images = (*d_ptr).n_input_pathnames[ig]
		;
		night_desig_text = STRING(format='(I8.8)',this_night)
		;
		this_LED = float(i_good) / LED_scale
		this_LED = 0 > (this_LED < (n_LEDs-1))
		(*s_ptr).P9_LED_colors[this_LED] = 'orange'
		WCT_LED_stack, (*s_ptr).P9_LED_wid[0], (*s_ptr).P9_LED_colors
        ;
		;	next, fetch the appropriate bad pixel mask, based on night and JD
		;	proximity
		;
		this_jd = (*d_ptr).input_first_JD[ig]
		;
		d_jd = abs(this_jd - (*b_ptr).bpm_JD[0:n_bpm-1])
		d_jd_min = MIN(d_jd)
		ind = WHERE(d_jd_min eq d_jd[*],count)
		if(count ne 1) then begin
			MSP_BDP_print_text, Event, "ERROR - bad JD match in P9"
			goto, P9SLC_outer_end
		endif
		;
		BPM_path = (*b_ptr).bpm_image_name[ind[0]]
		;
		MSP_BDP_print_text, Event, "  BPM file selected = "+FILE_BASENAME(BPM_path)
		;
		;	summarize input files
		;
		input_path = (*d_ptr).input_pathname[ig,0:n_input_images-1]
		;
		out_path = (*d_ptr).output_pathname[ig]
		;
		(*s_ptr).P9_LED_colors[this_LED] = 'blue'
		WCT_LED_stack, (*s_ptr).P9_LED_wid[0], (*s_ptr).P9_LED_colors
		;
		MSP_BDP_print_text, Event, "Starting Dark Image Computation for "+FILE_BASENAME(out_path)
		;
		;	 check that directory exists. if not, create it
		;
		out_dir = FILE_DIRNAME(out_path)
		test = FILE_INFO(out_dir)
		if(NOT(test.exists) or NOT(test.directory)) then FILE_MKDIR, out_dir
		;
		if(this_night ge (*s_ptr).row513_date) then row513_flag = 0 else row513_flag = 1
		;
		coadd_median_with_bad_sub, Event, BPM_path, input_path, out_path, $
								row513_flag, (*s_ptr).P9_LED_wid[0], this_LED, (*s_ptr).P9_LED_colors
		;
		(*s_ptr).P9_LED_colors[this_LED] = 'green'
		WCT_LED_stack, (*s_ptr).P9_LED_wid[0], (*s_ptr).P9_LED_colors
		;
		MSP_BDP_print_text, Event,"Wrote output image "+out_path
		;
		i_good++
		;
		loop_end:
	endfor
	;
	P9_Save_Night_Summary, Event
	;
	P9SLC_outer_end:
end
;

;-----------------------------------------------------------------
pro P9_Save_Night_Summary, Event, n_images
	;
	;	saves an updated night summary
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	n_ptr = PTR_struct.PTR_night
	k_ptr = PTR_struct.PTR_kokopeli
	s_ptr = PTR_struct.PTR_summary
	r_ptr = PTR_struct.PTR_RAW1
	a_ptr = PTR_struct.PTR_Ave_RAW1
	d_ptr = PTR_struct.PTR_Dark
	;
	;	for each night in the dark list, write the number of groups coadded
	;
	n_groups = (*d_ptr).n_dark_groups
	n_nights = (*d_ptr).n_nights
	;
	n_darks = intarr(n_nights)
	;
	for ig = 0, n_groups-1 do begin
		ind = WHERE( (*d_ptr).night_desig[ig] eq (*d_ptr).night_list[0:n_nights-1],count)
		if(count eq 1) then n_darks[ind[0]]++
	endfor
	;
	;	for each possible night
	;
	for inight = 0, n_nights-1 do begin
		;
		;	if there were non-zero number of dark images made, write to summary file for this night
		;
		if(n_darks[inight] ne 0) then begin
			;
			this_night = (*d_ptr).night_list[inight]
			;
			;	which one in the summary?
			;
			num_nights = (*s_ptr).num_nights
			ind = WHERE(this_night eq (*s_ptr).night_date[0:num_nights-1], count)
			if(count ne 0) then night_index = ind[0] else begin
				MSP_BDP_print_text, Event, "Error - No matching night in P9..."
				goto, P9SNS_end
			endelse
			;
			(*s_ptr).night_who[night_index] = (*s_ptr).who
			(*s_ptr).night_step[night_index] = 9
			text = STRTRIM(STRING(format='(i4.1)',n_darks[inight]),2)+" Dark Images Made"
			(*s_ptr).night_comment[night_index] = text
			;
			P1_Write_Summary_File, Event
			;
		endif
	endfor
	;
	P9SNS_end:
end