pro MSP_BDP_Ski_Jump_Detector, image, ski_jump_code
;
;	tests for the presence of ski jumps at top and bottom of
;	Mimir images, using a 5 sigma criterion
;
;	criterion
;
criterion = 5.0
;
image_size = SIZE(image)
nc = image_size[1]
nr = image_size[2]
;
;	select first half of image
;
ncx = ROUND(nc/4)
;
;	and column strip to examine
;
ncx1 = 0 > (ncx-5)
ncx2 = nc < (ncx+5)
;
dncx = abs(ncx2 - ncx1) + 1
;
;	center reference region
;
nry = ROUND(nr/2)
nr_ref_1 = 0 > (nry-6)
nr_ref_2 = nr < (nry+5)
;
dnr_ref = abs(nr_ref_2 - nr_ref_1) + 1
dnr_ref_2 = dnr_ref / 2
;
;	bottom ski jump test region
;
nr_bot_1 = 0
nr_bot_2 = 3
;
dnr_bot = abs(nr_bot_2 - nr_bot_1) + 1
;
;	top ski jump test region
;
nr_top_2 = nr-3
nr_top_1 = nr-6
;
dnr_top = abs(nr_top_2 - nr_top_1) + 1
;
;	compute median, rms for center reference region, on
;	and even-odd row basis to account for O/E readout
;
even = FLOOR(nr_ref_1/2)*2
odd = even + 1
;
center_even = fltarr(dncx,dnr_ref_2)
center_odd = fltarr(dncx,dnr_ref_2)
;
for irow = 0, dnr_ref_2-1 do begin
	jrow = even + 2*irow
	center_even[*,irow] = image[ncx1:ncx2,jrow]
endfor
for irow = 0, dnr_ref_2-1 do begin
	jrow = odd + 2*irow
	center_odd[*,irow] = image[ncx1:ncx2,jrow]
endfor
;
center_e = REFORM(center_even[*,*], dncx*dnr_ref_2 )
center_o = REFORM(center_odd[*,*], dncx*dnr_ref_2)
;help, center
ref_e = median_filtered_mean(center_e)
ref_o = median_filtered_mean(center_o)
;print, "center mean, rms = ",ref
;
;	for top, bottom, do MFM along rows first
;
bot_vec = fltarr(dnr_bot)
top_vec = fltarr(dnr_top)
;
for nr = 0, dnr_bot-1 do begin
	nrr = nr + nr_bot_1
	zz = median_filtered_mean(REFORM(image[ncx1:ncx2,nrr:nrr]))
	bot_vec[nr] = zz[0]
endfor
for nr = 0, dnr_top-1 do begin
	nrr = nr + nr_top_1
	zz = median_filtered_mean(REFORM(image[ncx1:ncx2,nrr:nrr]))
	top_vec[nr] = zz[0]
endfor
;
;	compute average top, bottom
;
bottom = MOMENT(bot_vec)
top = MOMENT(top_vec)
;
ski_jump = intarr(2)
;
t1 = (bottom[0] - ref_e[0])/ref_e[1]
t2 = (bottom[0] - ref_o[0])/ref_o[1]
t3 = (top[0] - ref_e[0])/ref_o[1]
t4 = (top[0] - ref_o[0])/ref_o[1]
;
if(t1 gt criterion or t2 gt criterion) then ski_jump[0] = 1 else ski_jump[0] = 0
if(t3 gt criterion or t4 gt criterion) then ski_jump[1] = 1 else ski_jump[1] = 0
;
;print, "ski jump array = ",ski_jump
;print, "bottom moment array = ",bottom
;print, "top moment array = ",top
;
ski_jump_code = ski_jump[0] + 10 * ski_jump[1]
;
end