function P3_detect_clocking_errors, image_pathname

;
;  Analyzes images looking for duplicated columns in each quadrent from clocking errors.  Returns
;  which quadrents are affected [UL, UR, LL, LR] and the phase of the duplicated column.
;
;  MP  20090731  Original Version
;  MP  20091022  Enforced sequential column checking to prevent false positives

;image_name = dialog_pickfile()
image = readfits(image_pathname, header, /SILENT)

result = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
;[UL, ULcol, ULcol, UR, URcol, URcol, LL, LLcol, LLcol, LR, LRcol, LRcol]
;if the quadrant (UL, UR, LL, LR) is marked as 1, then that quadrant is affected and the two following
;numbers are the phase of the duplicated columns, numbered 0-7

;Check Upper Left Quadrant
sample1 = image[64:72,600]
sample2 = image[64:72,700]
sample3 = image[64:72,800]
sample4 = image[64:72,900]

test1 = [1,0]; intentional, to prevent all zeros from going through and being treated like a true error
test2 = [0,0]
test3 = [0,0]
test4 = [0,0]

for i=0,7 do begin
  ;for j=i+1,7 do begin; check each sample for duplicates
    j = i+1
    if sample1[i] eq sample1[j] then test1 = [i,j]
    if sample2[i] eq sample2[j] then test2 = [i,j]
    if sample3[i] eq sample3[j] then test3 = [i,j]
    if sample4[i] eq sample4[j] then test4 = [i,j]
    ;for a true clocking error in this quadrant, each test array would be identical
    if n_elements(uniq([test1[0], test2[0], test3[0], test4[0]])) eq 1 then begin
      if n_elements(uniq([test1[1], test2[1], test3[1], test4[1]])) eq 1 then begin
        ;found duplicate columns in this quadrant in columns i and j, update result array
        result[0] = 1; quadrant is affected
        result[1] = test1[0]; first column
        result[2] = test1[1]; second column
      endif
    endif
  ;endfor
endfor



;Check Upper Right Quadrant
sample1 = image[576:584,600]
sample2 = image[576:584,700]
sample3 = image[576:584,800]
sample4 = image[576:584,900]

test1 = [1,0]
test2 = [0,0]
test3 = [0,0]
test4 = [0,0]

for i=0,7 do begin
  ;for j=i+1,7 do begin; check each sample for duplicates
    j=i+1
    if sample1[i] eq sample1[j] then test1 = [i,j]
    if sample2[i] eq sample2[j] then test2 = [i,j]
    if sample3[i] eq sample3[j] then test3 = [i,j]
    if sample4[i] eq sample4[j] then test4 = [i,j]
    ;for a true clocking error in this quadrant, each test array would be identical
    if n_elements(uniq([test1[0], test2[0], test3[0], test4[0]])) eq 1 then begin
      if n_elements(uniq([test1[1], test2[1], test3[1], test4[1]])) eq 1 then begin
        ;found duplicate columns in this quadrant in columns i and j, update result array
        result[3] = 1; quadrant is affected
        result[4] = test1[0]; first column
        result[5] = test1[1]; second column
      endif
    endif
  ;endfor
endfor



;Check Lower Left Quadrant
sample1 = image[64:72,200]
sample2 = image[64:72,300]
sample3 = image[64:72,400]
sample4 = image[64:72,500]

test1 = [1,0]
test2 = [0,0]
test3 = [0,0]
test4 = [0,0]

for i=0,7 do begin
  ;for j=i+1,7 do begin; check each sample for duplicates
    j= i+1
    if sample1[i] eq sample1[j] then test1 = [i,j]
    if sample2[i] eq sample2[j] then test2 = [i,j]
    if sample3[i] eq sample3[j] then test3 = [i,j]
    if sample4[i] eq sample4[j] then test4 = [i,j]
    ;
    ;for a true clocking error in this quadrant, each test array would be identical
    if n_elements(uniq([test1[0], test2[0], test3[0], test4[0]])) eq 1 then begin
      if n_elements(uniq([test1[1], test2[1], test3[1], test4[1]])) eq 1 then begin
        ;found duplicate columns in this quadrant in columns i and j, update result array
        result[6] = 1; quadrant is affected
        result[7] = test1[0]; first column
        result[8] = test1[1]; second column
      endif
    endif
  ;endfor
endfor



;Check Lower Right Quadrant
sample1 = image[576:584,200]
sample2 = image[576:584,300]
sample3 = image[576:584,400]
sample4 = image[576:584,500]

test1 = [1,0]
test2 = [0,0]
test3 = [0,0]
test4 = [0,0]

for i=0,7 do begin
  ;for j=i+1,7 do begin; check each sample for duplicates
    j= i+1
    if sample1[i] eq sample1[j] then test1 = [i,j]
    if sample2[i] eq sample2[j] then test2 = [i,j]
    if sample3[i] eq sample3[j] then test3 = [i,j]
    if sample4[i] eq sample4[j] then test4 = [i,j]
    ;for a true clocking error in this quadrant, each test array would be identical
    if n_elements(uniq([test1[0], test2[0], test3[0], test4[0]])) eq 1 then begin
      if n_elements(uniq([test1[1], test2[1], test3[1], test4[1]])) eq 1 then begin
        ;found duplicate columns in this quadrant in columns i and j, update result array
        result[9] = 1; quadrant is affected
        result[10] = test1[0]; first column
        result[11] = test1[1]; second column
      endif
    endif
  ;endfor
endfor

return, result

end