pro fix_wrong_night_name
;
;	in the case of collecting data with the wrong night name
;	this program will change the CDS file rootnames to the
;	correct night name and will also change the filename
;	contained in the FITS header in the CDS file
;
;	DPC	20130922
;	DPC 20131222 added ability to add offset image number
;
dir_in = dialog_pickfile(Title='Input CDS Directory',/dir)
dir_out = dialog_pickfile(Title='Output CDS Directory',/dir)
;
raw1_in = dialog_pickfile(Title='Input RAW1 Directory',/dir)
raw1_out = dialog_pickfile(Title='Output RAW1 Directory',/dir)
;
;	read the contents of the input directory
;
files = FILE_SEARCH(dir_in,'*.fits')
nfiles = N_ELEMENTS(files)
;
files_raw1 = FILE_SEARCH(raw1_in,'raw1_*')
nfiles_raw1 = N_ELEMENTS(files_raw1)
print, 'Found ',nfiles,' CDS files and ',nfiles_raw1,' RAW1 files'
;
;	parse the first file to get the current root name
;
base = FILE_BASENAME(files[0],'.fits')
;
;	separate at dots
;
parts = STRSPLIT(base,'.',/extract)
;
old_root = parts[0]
;
print, 'The old root file name was ',old_root
;
print, 'What should it become ? (YYYYMMDD)'
new_root = 0L
read, new_root
;
print, 'Echo - new root will become ',new_root
;
;	and first image number
;
print, 'The old first image number was ',parts[1]
print, 'What should it become? '
new_base = 0L
read, new_base
base_dif = new_base	- parts[1] ; add to old number
;
;	part1, copy old files to new directory, changing the name, only
;
new_paths = strarr(nfiles)
;
for i = 0, nfiles-1 do begin
	old_path = files[i]
	old_file = FILE_BASENAME(files[i],'.fits')
	parts = STRSPLIT(old_file,'.',/extract)
	;
	;	change number
	;
	image_number = long(parts[1])
	image_number += base_dif
	if(image_number lt 1000) then begin
		imno_str = STRING(image_number, format='(I3.3)')
	endif else begin
		imno_str = STRING(image_number, format='(I4.4)')
	endelse
	;
	new_path = dir_out + STRING(new_root,format='(I8.8)')+'.'+imno_str+'.fits'
	new_paths[i] = new_path
	;
	print, "copying ",old_path," to ",new_path
	;
	FILE_COPY, old_path, new_path
	;
	;	and RAW1 images
	;
	old_raw1_path = files_raw1[i]
	old_raw1_file = FILE_BASENAME(files_raw1[i],'.fits')
	parts = STRSPLIT(old_raw1_file,'_',/extract)
	;
	;	change number
	;
	image_number = long(parts[1])
	image_number += base_dif
	if(image_number lt 1000) then begin
		imno_str = STRING(image_number, format='(I3.3)')
	endif else begin
		imno_str = STRING(image_number, format='(I4.4)')
	endelse
	;
	new_raw1_path = raw1_out + 'raw1_' + imno_str
	;
	print, "copying ",old_raw1_path," to ",new_raw1_path
	;
	FILE_COPY, old_raw1_path, new_raw1_path
endfor
;
;	next, for each new file, read it in, change the filename and write it back out in place
;	that is, never touch the input file
;
for i = 0, nfiles-1 do begin
	image = readfits(new_paths[i],header)
	old_name = SXPAR(header,"FILENAME")
	parts = STRSPLIT(old_name,'.',/extract)
	image_number = long(parts[1])
	image_number += base_dif
	if(image_number lt 1000) then begin
		imno_str = STRING(image_number, format='(I3.3)')
	endif else begin
		imno_str = STRING(image_number, format='(I4.4)')
	endelse
	;
	new_name = STRING(new_root,format='(I8.8)')+'.'+imno_str+'.fits'
	;
	FXADDPAR,header,"FILENAME",new_name," Filename modified by <fix_wrong_night_name.pro>"
	;
	writefits, new_paths[i],image,header
endfor
;
print, 'Done!'
;
end
