pro hwp_fidelity_test, Event, files_in, check_dither, fidelity_out, this_fold_angle, $
			special_PA, pa_dith, s_pa_dith, dither_vector, debug, DITHER=class_by_dither
;
;	for a group of polarimetry images,
;	evaluates HWP offset angle and recommends what to
;	do for each image.
;
;	INPUT:
;			files_in - string array of pathnames to group CDS files
;			check_dither - integer flag (0 = skip dither checking; 1 = do dither checking)
;	OUTPUT:
;			fidelity_out - integer array of recommendations. Values
;				are: "-1" = delete image
;					 "0"  = leave image "as is"
;				     "1" = change HWP angle to HWP+45 deg
;			this_fold_ang - integer array holding the fold angle of the image (0, 45, 90, 135)
;			dither - vector of dither pointings; 1=need HWP rotation; 0=no rotation needed
;
;	DPC	20091221 - initial version
;	DPC 20091222 - changed to multiplicative scaling from additive
;--------------------------------------------------------------
;
;
debug = 1
;
n_images = N_ELEMENTS(files_in)
;
if(debug) then print, "number of images = ",n_images
;
if(check_dither eq 0) then begin
	pa_dith = -99
	s_pa_dith = -99
endif
;
pixel_value = fltarr(n_images)
s_pixel_value = fltarr(n_images)
reference_pixel_value = fltarr(n_images)
s_reference_pixel_value = fltarr(n_images)
hwp_value = fltarr(n_images)
image_number = indgen(n_images)
jd = dblarr(n_images)
file_name_list = FILE_BASENAME(files_in)
the_ind = lonarr(n_images)
;
;	for each image
;
this_u = fltarr(n_images)
this_u[*] = -99.9
this_q = fltarr(n_images)
this_q[*] = -99.9
this_p = fltarr(n_images)
this_p[*] = -99.9
this_pa = fltarr(n_images)
this_pa[*] = -99.9
this_fold_angle = intarr(n_images)
this_fold_angle[*] = -99
;
fidelity_out = intarr(n_images)
;
ix = 163				; this is a good pixel for tracing instrumental polarization
iy = 850
;
nom_hwp_angle = [0.0, 22.5, 45.0, 67.5, 90.0, 112.5, 135.0, 157.5, $
					 180.0, 202.5, 225.0, 247.5, 270.0, 292.5, 315.0, 337.5]
;
ii = 0L
for i = 0, n_images-1 do begin
	;
	the_ind[i] = -1
	;
	image = readfits(files_in[i],header,/silent)
	;
	;	get HWP angle - look for some higher-level keywords first
	;
	hwp_a = normal_angle(sxpar(header, "HWPANGLE", COUNT=count))
	if(count gt 0) then begin
		hwp_value[i] = hwp_a
	endif else begin
		;
		;	this may be a "new" header - look for HWP_ANG keyword
		;
		hwp_n = normal_angle(sxpar(header, "HWP_ANG", COUNT=n_count))
		if(n_count gt 0) then begin
			hwp_value[i] = hwp_n
		endif else begin
			;
			;	first attempt failed, look for raw count
			;
			comment = sxpar(header, "COMMENT", COUNT=count)
			ind = WHERE(STRPOS(comment,"BD1") ne -1,ngood)
			if(ngood gt 0) then begin
				;
				comment1 = STRUPCASE( STRTRIM( comment[ind[0]],2 ))
				;
				s0 = STRPOS( comment1, "GOTO")
				s1 = STRPOS( comment1, '.')
				s3 = STRPOS( comment1, 'HOME')
				;
				istep = 0L
				if(s0 ge 0) then begin
					if(s1 ge 0) then begin
						if(s3 ge 0) then begin
							angle = 0.0
						endif else begin
							hwp_string = STRMID( comment1, s0+5, (s1-s0-5))
							reads, hwp_string, format='(i)', istep
							angle = float(istep) * 0.675d0
							angle = normal_angle(angle)
						endelse
					endif else begin
						;
						;	no dots... so, parse with spaces
						;
						len = STRLEN(comment1) - s0
						remainder = STRMID(comment1, s0, len-1)
						parts = STRSPLIT(remainder, ' ',/REGEX,/EXTRACT)
						len1 = STRLEN(parts[1])			; "GOTO" is parts[0]
						if(len1 ne 0) then begin
							nstep = STRTRIM(parts[1],2)
							reads, nstep, format='(i)', istep
							angle = float(istep) * 0.675d0
							angle = normal_angle(angle)
						endif else angle = -99.0
					endelse
				endif else begin
					if(s3 ge 0) then begin
						angle = 0.0
					endif else begin
						angle = -99.9
					endelse
				endelse
				;
				hwp_value[i] = angle
				if(angle eq -99.9) then fidelity_out[i] = -1
			endif else begin
				hwp_value[i] = -99.9
				fidelity_out[i] = -1		; mark to delete
			endelse
		endelse
	endelse
	;
	if(hwp_value[i] ne -99.9) then begin
		d_hwp = abs(abs(hwp_value[i]) - nom_hwp_angle)
		d_hwp_min = MIN(d_hwp, ind_match)
		hwp_value[i] = nom_hwp_angle[ind_match[0]]
	endif
	;
	if(hwp_value[i] ne -99.9) then begin
		if(check_dither) then begin
			;-------------------------------------------------------
			;
			;	get julian date
			;
			date_obs = FXPAR(header, "DATE-OBS")
			;
			;	parse date / time
			;
			reads, date_obs, year, month, day, hour, minute, second, format='(i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f4.2)
			;
			;	convert to JD
			;
			jd[i] = JULDAY(Month, Day, Year, Hour, Minute, Second)
			;
			;-------------------------------------------------------
		endif else jd[i] = float(i)
		;
		;	get pixel value in zone
		;
		sub_image = image[ix-40:ix+40, iy-40:iy+40]
		ss = median_filtered_mean(sub_image)
		pixel_value[i] = ss[0] > (-1000.0)
		if(pixel_value[i] ne -1000.0) then begin
			s_pixel_value[i] = ss[1] / sqrt(float(N_elements(sub_image))-1)
		endif else begin
			s_pixel_value[i] = 10000.0
		endelse
		;
;		if(debug) then print, "image, hwp, pixel mean = ",FILE_BASENAME(files_in[i]),hwp_value[i],pixel_value[i]
		;
		;	get reference pixel value (this takes care of gravity wave modulations in total power)
		;
		sub_image = image[511-40:511+40,511-40:511+40]
		rr = median_filtered_mean(sub_image)
		reference_pixel_value[i] = rr[0] > (-1000.0)		; to trap bad second readouts
		if(reference_pixel_value[i] ne -1000.0) then begin
			s_reference_pixel_value[i] = rr[1] / sqrt(float(N_elements(sub_image))-1)
		endif else begin
			s_reference_pixel_value[i] = 10000.0
		endelse
		;
		the_ind[i] = i
	endif else begin
		pixel_value[i] = -1000.0
		reference_pixel_value[i] = -1000.0
	endelse
endfor
;
;	establish dither number from JD, HWP, FILENAMES
;
if(check_dither) then begin
	dither_vector = get_dither(file_name_list, jd, hwp_value)
endif else begin
	dither_vector = intarr(n_images)
	dither_vector[*] = 0
endelse
;
;	compute mean reference pixel value
;
mean_ref = median_filtered_mean(reference_pixel_value)
s_mean_ref = mean_ref[1]
;
;	divide by reference pixel value and rescale to mean reference pixel value
;
dif_pix = (pixel_value / reference_pixel_value) * mean_ref[0]
;
ind_bad = WHERE(pixel_value eq -1000.0 or reference_pixel_value eq -1000.0,nbad)
if(nbad gt 0) then begin
	dif_pix[ind_bad] = -1000.0
;	s_dif_pix[ind_bad] = 10000.0
endif
;
ind = SORT(hwp_value)
the_hwp_values = hwp_value[UNIQ(hwp_value,SORT(hwp_value))]
;
;	trim out the special case of -99.9 if present
;
ind_not_99 = WHERE(the_hwp_values ne -99.9,n_not_99)
if(n_not_99 gt 0) then begin
	the_hwp_values = the_hwp_values[ind_not_99]
endif
;
;	if not 16 HWP positions, the rest of the code won't work...
;
if(N_ELEMENTS(the_hwp_values) ne 16) then begin
	fidelity_out[*] = -1
	special_PA = -99.9
	MSP_BDP_print_text, Event, "ERROR - only "+STRING(N_ELEMENTS(the_hwp_values))+" HWP values present - skipping rest of analysis"
	goto, the_end
endif
;
;	how many dither positions?
;
the_dither_set = dither_vector(UNIQ(dither_vector,SORT(dither_vector)))
n_dithers = N_ELEMENTS(the_dither_set)
;
q = fltarr(4)
u = fltarr(4)
p = fltarr(4)
pa = fltarr(4)
;
offset = 0.0					; or mean_ref[0]
;
q_dith = dblarr(n_dithers)
s_q_dith = dblarr(n_dithers)
u_dith = dblarr(n_dithers)
s_u_dith = dblarr(n_dithers)
p_dith = dblarr(n_dithers)
s_p_dith = dblarr(n_dithers)
pa_dith = dblarr(n_dithers)
s_pa_dith = dblarr(n_dithers)
;
class_by_dither = intarr(n_dithers)
;
;	save PAs for all groups and dithers
;
n_save = n_dithers * 4
pa_save = fltarr(n_save)
time_save = fltarr(n_save)
set_save = indgen(n_save)
;
if(n_dithers gt 0) then begin
	for idith = 0, n_dithers-1 do begin
		;
		ind_d = WHERE(dither_vector eq the_dither_set[idith] and dif_pix ne -1000.0,n_in_dither)	; indices of original file list
		if(n_in_dither gt 0) then begin
			;
			;	compute P, PA values for each Q,U set in each dither
			;
			for iset = 0, 3 do begin
				;
				q[*] = -99.9
				u[*] = -99.9
				p[*] = -99.9
				pa[*] = -99.9
				;
				ioff = 4 * iset
				;
				hwp_90 = the_hwp_values[ioff+2]
				ind_90 = WHERE(hwp_value[ind_d] eq hwp_90,n_90)
				if(n_90 gt 0) then begin
					all_ind_90 = ind_d[ind_90]
					this_fold_angle[all_ind_90] = 90
				endif
				case n_90 of
					0: I_90 = -99.9
					1: I_90 = dif_pix[all_ind_90] + offset
					else: begin
							zz = median_filtered_mean(dif_pix[all_ind_90])
							I_90 = zz[0] + offset
						   end
				endcase
				;
				hwp_0 = the_hwp_values[ioff]
				ind_0 = WHERE(hwp_value[ind_d] eq hwp_0,n_0)
				if(n_0 gt 0) then begin
					all_ind_0 = ind_d[ind_0]
					this_fold_angle[all_ind_0] = 0
				endif
				case n_0 of
					0: I_0 = -99.9
					1: I_0 = dif_pix[all_ind_0] + offset
					else: begin
							zz = median_filtered_mean(dif_pix[all_ind_0])
							I_0 = zz[0] + offset
						   end
				endcase
				;
				hwp_135 = the_hwp_values[ioff+3]
				ind_135 = WHERE(hwp_value[ind_d] eq hwp_135,n_135)
				if(n_135 gt 0) then begin
					all_ind_135 = ind_d[ind_135]
					this_fold_angle[all_ind_135] = 135
				endif
				case n_135 of
					0: I_135 = -99.9
					1: I_135 = dif_pix[all_ind_135] + offset
					else: begin
							zz = median_filtered_mean(dif_pix[all_ind_135])
							I_135 = zz[0] + offset
						   end
				endcase
				;
				hwp_45 = the_hwp_values[ioff+1]
				ind_45 = WHERE(hwp_value[ind_d] eq hwp_45,n_45)
				if(n_45 gt 0) then begin
					all_ind_45 = ind_d[ind_45]
					this_fold_angle[all_ind_45] = 45
				endif
				case n_45 of
					0: I_45 = -99.9
					1: I_45 = dif_pix[all_ind_45] + offset
					else: begin
							zz = median_filtered_mean(dif_pix[all_ind_45])
							I_45 = zz[0] + offset
						   end
				endcase
				;
				if(I_90 ne -99.9 and I_0 ne -99.9) then begin
					q[iset] = (I_90 - I_0) / (I_90 + I_0)
				endif else begin
					q[iset] = -99.9
				endelse
				;
				if(I_135 ne -99.9 and I_45 ne -99.9) then begin
					u[iset] = (I_135 - I_45) / (I_135 + I_45)
				endif else begin
					u[iset] = -99.9
				endelse
				;
				if(q[iset] ne -99.9 and u[iset] ne -99.9) then begin
					p[iset] = sqrt(q[iset]^2 + u[iset]^2)
					pa[iset] = (28.65 * ATAN(u[iset],q[iset]) + 72000.0) MOD 180.0
				endif else begin
					p[iset] = -99.9
					pa[iset] = -99.9
				endelse
				;
				;	save results for each constituent image
				;
				case n_0 of
					0: 				; don't do anything, as results already deleted
					else: begin
						this_q[all_ind_0] = q[iset]
						this_u[all_ind_0] = u[iset]
						this_p[all_ind_0] = p[iset]
						this_pa[all_ind_0] = pa[iset]
					   end
				endcase
				;
				case n_90 of
					0: 				; don't do anything, as results already deleted
					else: begin
						this_q[all_ind_90] = q[iset]
						this_u[all_ind_90] = u[iset]
						this_p[all_ind_90] = p[iset]
						this_pa[all_ind_90] = pa[iset]
					   end
				endcase
				;
				case n_45 of
					0: 				; don't do anything, as results already deleted
					else: begin
						this_u[all_ind_45] = u[iset]
						this_q[all_ind_45] = q[iset]
						this_p[all_ind_45] = p[iset]
						this_pa[all_ind_45] = pa[iset]
					   end
				endcase
				;
				case n_135 of
					0: 				; don't do anything, as results already deleted
					else: begin
						this_u[all_ind_135] = u[iset]
						this_q[all_ind_135] = q[iset]
						this_p[all_ind_135] = p[iset]
						this_pa[all_ind_135] = pa[iset]
					   end
				endcase
				;
			endfor
			;
			;	compute folded p, pa for this dither position
			;
			ind_0 = WHERE(this_fold_angle[ind_d] eq 0, n_0)
			ind_45 = WHERE(this_fold_angle[ind_d] eq 45, n_45)
			ind_90 = WHERE(this_fold_angle[ind_d] eq 90, n_90)
			ind_135 = WHERE(this_fold_angle[ind_d] eq 135, n_135)
			;
			if(n_0 gt 0 and n_90 gt 0 and n_45 gt 0 and n_135 gt 0) then begin
				if(n_0 gt 1) then begin
					zz = MOMENT(dif_pix[ind_d[ind_0]])
				endif else zz = [dif_pix[ind_d[ind_0]],dif_pix[ind_d[ind_0]]]
				I_0 = zz[0]
				s_I_0 = sqrt(zz[1])
				if(n_0 gt 2) then s_I_0 /= sqrt(n_0 - 1.0)
				if(n_90 gt 1) then begin
					zz = MOMENT(dif_pix[ind_d[ind_90]])
				endif else zz = [dif_pix[ind_d[ind_90]],dif_pix[ind_d[ind_90]]]
				I_90 = zz[0]
				s_I_90 = sqrt(zz[1])
				if(n_90 gt 2) then s_I_90 /= sqrt(n_90 - 1.0)
				if(n_45 gt 1) then begin
					zz = MOMENT(dif_pix[ind_d[ind_45]])
				endif else zz = [dif_pix[ind_d[ind_45]],dif_pix[ind_d[ind_45]]]
				I_45 = zz[0]
				s_I_45 = sqrt(zz[1])
				if(n_45 gt 2) then s_I_45 /= sqrt(n_45 - 1.0)
				if(n_135 gt 1) then begin
					zz = MOMENT(dif_pix[ind_d[ind_135]])
				endif else zz = [dif_pix[ind_d[ind_135]],dif_pix[ind_d[ind_135]]]
				I_135 = zz[0]
				s_I_135 = sqrt(zz[1])
				if(n_135 gt 2) then s_I_135 /= sqrt(n_135 - 1.0)
				;
				;
				if(I_90 ne -99.9 and I_0 ne -99.9) then begin
					q_dith[idith] = (I_90 - I_0) / (I_90 + I_0)
					s_q_dith[idith] = sqrt(s_I_0^2 + s_I_90^2) / (I_90 + I_0)
				endif else begin
					q_dith[idith] = -99.9
					s_q_dith[idith] = 99.9
				endelse
				;
				if(I_135 ne -99.9 and I_45 ne -99.9) then begin
					u_dith[idith] = (I_135 - I_45) / (I_135 + I_45)
					s_u_dith[idith] = sqrt(s_I_45^2 + s_I_135^2) / (I_135 + I_45)
				endif else begin
					u_dith[idith] = -99.9
					s_u_dith[idith] = 99.9
				endelse
				;
				if(q_dith[idith] ne -99.9 and u_dith[idith] ne -99.9) then begin
					p_dith[idith] = sqrt(q_dith[idith]^2 + u_dith[idith]^2)
					pa_dith[idith] = 28.65 * ATAN(u_dith[idith],q_dith[idith])
					pa_dith[idith] = (pa_dith[idith] + 72000.0) MOD 180.0
					;
					s_p_dith[idith] = (1.0/p_dith[idith]) * sqrt( (u_dith[idith] * s_q_dith[idith])^2 $
												+ (q_dith[idith] * s_q_dith[idith])^2)
					s_pa_dith[idith] = (28.65 * s_p_dith[idith] / p_dith[idith]) < 60.0
				endif else begin
					p_dith[idith] = -99.9
					pa_dith[idith] = -99.9
					s_p_dith[idith] = 99.9
					s_pa_dith[idith] = 99.9
				endelse
;				if(debug) then begin
;					print, "For dither ",idith," P, PA = ",p_dith[idith],'+/-',s_p_dith[idith],'   ',$
;									pa_dith[idith],'+/-',s_pa_dith[idith]
;					print, "              Q, U = ",q_dith[idith],"+/-",s_q_dith[idith],"     ",u_dith[idith],"+/-",s_u_dith[idith]
;					print, 'number of data values n_0, n_90, n_45, n_135 = ',n_0, n_90, n_45, n_135
;				endif
				;
				;	classify
				;
				;		first, recenter PA about 35 deg
				;
				pa_dith[idith] = (pa_dith[idith] + 720.0) MOD 180.
				if(pa_dith[idith] gt 145.0) then pa_dith[idith] -= 180.0
				;
				test_0a = (pa_dith[idith] - 60.0 + 720.0) MOD 180.
				test_0b = test_0a - 180.0
				test_0 = abs(test_0a) < abs(test_0b)
				;
				test_1a = (pa_dith[idith] - 10.0 + 720.) MOD 180.
				test_1b = test_1a - 180.0
				test_1 = abs(test_1a) < abs(test_1b)
				;
				;
				;	compute probabilities
				;
				p_45 = exp(-0.5*( (test_1 / s_pa_dith[idith])^2))
				p_0  = exp(-0.5*( (test_0 / s_pa_dith[idith])^2))
				;
				; condition for insisting on 45 deg shift is that p_45 / p_0 > 2.0 and s_pa < 45 deg
				;
				if( (p_45 / p_0) gt 2.0 and s_pa_dith[idith] lt 45.0) then begin
					class_by_dither[idith] = 1
					fidelity_out[ind_d] = 1
					if(debug) then print, "... 45 degree phase shift needed"
				endif else begin
					class_by_dither[idith] = 0
					fidelity_out[ind_d] = 0
					if(debug) then print, "... no phase shift needed"
				endelse
				;
				;	also look for the wierd case of angles in the 110-140 deg range - DPC 20100209
				;
				if( (pa_dith[idith] gt 110.0 and pa_dith[idith] lt 140.0) or (pa_dith[idith] lt -40.0)) then begin
					class_by_dither[idith] = 2
					fidelity_out[ind_d] = 2
					if(debug) then print, "... odd angle outside normal range"
				endif
				;
			endif else begin
				p_dith[idith] = -99.9
				pa_dith[idith] = -99.9
				s_p_dith[idith] = 99.9
				s_pa_dith[idith] = 99.9
			endelse
		endif
	endfor
	;
endif
;
;	then HWP - begin by fitting line and removing
;
new_pix = dif_pix + offset
;
ind_bad = WHERE(dif_pix eq -1000.0,nbad)
if(nbad gt 0) then new_pix[ind_bad] = -1000.0
;
;	next, bin by fold angle and average to find best U, Q, P, PA values
;
the_fold_ind = UNIQ(this_fold_angle,SORT(this_fold_angle))
the_fold_angles = this_fold_angle[the_fold_ind]
ind_not_99 = WHERE(the_fold_angles ne -99,n_not_99)
;
if(n_not_99 gt 0) then begin
	the_fold_angles = the_fold_angles[ind_not_99]
	;
	n_fold = N_ELEMENTS(the_fold_angles)
	if(debug) then print,"number of HWP fold angles = ",n_fold
	;
	y_fold = dblarr(n_fold)
	s_y_fold = dblarr(n_fold)
	;
	for i_fold = 0, n_fold-1 do begin
		ind_fold = WHERE(this_fold_angle eq the_fold_angles[i_fold],n_this_fold)
		if(n_this_fold ne 0) then begin
			vec = new_pix[ind_fold]
			ind_good = WHERE(vec ne -1000.0,ngood)
			if(ngood gt 0) then vec = vec[ind_good]
			hh = median_filtered_mean(vec)
			y_fold[i_fold] = hh[0]
			s_y_fold[i_fold] = hh[1] / sqrt(float(ngood)-1)
			;
			if(debug) then print,"i_fold, y_fold, s_y_fold = ",i_fold,y_fold[i_fold],s_y_fold[i_fold]
			;
		endif else begin
			y_fold[i_fold] = -99.9
			s_y_fold[i_fold] = 99.9
		endelse
	endfor
	;
	ind_good = WHERE(y_fold ne -99.9,ngood)
	if(ngood gt 0) then begin
		yave = TOTAL(y_fold[ind_good]) / float(ngood)
		ymax = max(y_fold[ind_good])
		ymin = min(y_fold[ind_good])
		dy = ymax - ymin
		ymax = ymax + 0.1*dy
		ymin = ymin - 0.1*dy
	endif else begin
		;
		;	this is really bad, just jump out
		;
		fidelity_out[*] = -1
		goto, the_end
	endelse
endif else begin
	;
	;	no good results, jump out
	;
	if(debug) then print,"whoops! no fold angles found - error"
	fidelity_out[*] = -1
	goto, the_end
endelse
;
;	compute P, PA values of the folded set
;
I_90 = y_fold[2]
I_0 = y_fold[0]
s_I_90 = s_y_fold[2]
s_I_0 = s_y_fold[0]
;
I_135 = y_fold[3]
I_45 = y_fold[1]
s_I_135 = s_y_fold[3]
s_I_45 = s_y_fold[1]
;
if(I_0 ne -99.9 and I_90 ne -99.9) then begin
	q_fold = (I_90 - I_0) / (I_90 + I_0)
	N = I_90 - I_0
	D = I_90 + I_0
	s_q_fold = sqrt(s_I_90 * s_I_90 + s_I_0 * s_I_0) / D
endif else begin
	q_fold = -99.9
	s_q_fold = 99.9
endelse
;
if(I_45 ne -99.9 and I_135 ne -99.9) then begin
	u_fold = (I_135 - I_45) / (I_135 + I_45)
	N = I_135 - I_45
	D = I_135 + I_45
	s_u_fold = sqrt(s_I_135 * s_I_135 + s_I_45 * s_I_45) / D
endif else begin
	u_fold = -99.9
	s_u_fold = 99.9
endelse
;
if(q_fold ne -99.9 and u_fold ne -99.9) then begin
	p_fold = sqrt(q_fold^2 + u_fold^2)
	pa_fold = (28.65 * ATAN(u_fold,q_fold) + 72000.0) MOD 180
	s_p_fold = sqrt( (u_fold * s_u_fold)^2 + (q_fold * s_q_fold)^2) / p_fold
	s_pa_fold = 28.65 * s_p_fold / p_fold
endif else begin
	p_fold = -99.9
	pa_fold = -99.9
	s_p_fold = 99.9
	s_pa_fold = 99.9
endelse
;
if(debug) then begin
	print, "Folded Q, U = ",q_fold,"+/-",s_q_fold,"    ",u_fold,"+/-",s_u_fold
	print, "Folded  Pol, PA = ",p_fold,"+/-",s_p_fold,'   ',pa_fold,"+/-",s_pa_fold
endif
;
special_PA = pa_fold
;
;	set all fidelity_out flags equal to their dither flag
;
the_end:
;
;if(special_PA eq -99.9) then stop
;
end
