function kokepeli_data_extract, array, xsub1, ysub1
;
;   extracts image data from array
;
;   filling result array with pixel data
;
;   extracts columns of data, centered on where kokepeli crack should
;   be in each quadrant
;
;   DPC     20050621 initial program
;           20050726 modified to use positive signals from convolved images
;           20050527 modified to tune extraction regions based on use
;           20050728 modified to use smaller central array with xsub1, ysub1 offsets
;
sz = size(array)
;if (sz[0] ne 2 and sz[1] ne 1024 and sz[2] ne 1026) then begin
;    print, "Wrong array size for Kokepeli_Data_Extract"
;    print, "  Current array has ",sz[0]," dimensions"
;    print, "  First dimension has size ",sz[1]," instead of 1024"
;    if(sz[0] gt 1) then print, "   Second dimension has size ",sz[2]," instead of 1026"
;    print, " PROGRAM STOP"
;    stop
;endif
;
;   declare result array size, then start loading it
;
result = fltarr(3,4,21,21)  ; (Z:X:Y,quad, IX, IY)
;
;   quadrant extraction constants
;
;   UL = 0
;      x1 = 487;       version 2 = 463
;      m  = 1.759259
;      b = -172.0
;
;   UR = 1
;      x1 = 519;       version 2 = 517
;      m  = 1.759259
;      b = -172.0
;
;   LL = 2
;      x1 = 492;       version 2 = 368
;      m  = 1.759259
;      b = -453.9
;
;   LR = 3
;      x1 = 513;       version 2 = 524
;      m  = 1.759259
;      b = -453.9
;
;x1 = [487, 519, 492, 513]      ; DS9 column number (version 1)
x1 = [463, 517, 368, 524]     ; version 2 20050727
;
m  = [1.759259, 1.759259, 1.759259, 1.759259]
b  = [-172.0, -172.0, -453.9, -453.9]
;
;   for each quad
;
for iquad = 0, 3 do begin
    ;
    ;print, "Quad = ",iquad
    ;
    ;  for each column, compute first, last row to capture
    ;
    for icol = x1[iquad]-1, x1[iquad]+19 do begin
       ;
       ; compute center row
       ;
       ycen = floor(float(icol) * m[iquad] + b[iquad])
       ;
       ; go up, down 10 rows
       ;
       y1 = ycen - 10
       y2 = ycen + 10
       ;
       ;print, "Original Y1, Y2 = ",y1, y2
       ;
       ;       correct for selection of central sub-image
       ;
       y1 = y1 - ysub1
       y2 = y2 - ysub1
       ;
       ;print, "Corrected Y1, Y2 = ",y1, y2
       ;
       ;print, "for icol = ",icol
       ;print, "y1, ycen, y2 = ",y1, ycen, y2
       ;
       ; for each pixel, save to results array
       ;
       ;    trap large negative values to -50 counts  *** turn this off for convolved data
       ;
       result[0,iquad,icol-x1[iquad]+1,0:20] = array[icol-xsub1,y1:y2]       ; -50.0 > array[icol,y1:y2]
       result[1,iquad,icol-x1[iquad]+1,0:20] = icol ; x index value
       for iy = 0, 20 do begin
        result[2,iquad,icol-x1[iquad]+1,iy] = y1+iy+ysub1 ; y index value
       endfor
       ;
       ;print, format="(21(I7,1x))",result[0,iquad,icol-x1[iquad]+1,0:20]
       ;
    endfor
endfor
;
;   done, return
;
return, result
end
