pro retard_flat_phase, flat_list_in, flat_list_out, offset_angle, ier
;
;	takes a set of 16 pol flats and changes effective HWP phase
;	by "offset_angle".
;
;	to retard phase by 50 deg, angle should = -50.0
;
;	reads flat_list_in and writes flat_list_out
;
;	DPC	20091222	new code
;
;------------------------------------------------------------------------
;
obs_order = [0, 2, 1, 3, 4, 6, 5, 7, 8, 10, 9, 11, 12, 14, 13, 15]
ind_obs = SORT(obs_order)
;
debug = 0
bad_pix_value = -1.0d6
;
;	fetch input flats
;
n_in = N_ELEMENTS(flat_list_in)
if(n_in eq 0) then begin
	print, "RETARD_FLAT_PHASE: Error - no images on input flat list"
	ier = -1
	goto, the_end
endif
;
for i_in = 0, n_in-1 do begin
	image = readfits(flat_list_in[i_in],header,/silent)
	;
	;	determine image type
	;
	if(i_in eq 0) then begin
		info = SIZE(image,/TYPE)
		nxy = SIZE(image)
		case info of
			 3: big_flat_in = lonarr(nxy[1],nxy[2],n_in)
			 4: big_flat_in = fltarr(nxy[1],nxy[2],n_in)
			 5: big_flat_in = dblarr(nxy[1],nxy[2],n_in)
		endcase
		hwp_flat_in = fltarr(n_in)
		big_header = strarr(500, n_in)
		big_header_lines = lonarr(n_in)
	endif
	;
	;	save header
	;
	nlines = N_ELEMENTS(header)
	big_header[0:nlines-1,i_in] = header[*]
	big_header_lines[i_in] = nlines
	;
	;	load data
	;
	big_flat_in[*,*,i_in] = image[*,*]
	;
	;	extract HWP angle
	;
	file_name = FILE_BASENAME(STRUPCASE(flat_list_in[i_in]),'.FITS')
	last = STRLEN(file_name)
	first = last - 4
	num_str = STRMID(file_name,first,4)
	reads, num_str, angle, format='(I)'
	hwp_flat_in[i_in] = float(num_str) / 10.0
	;
	if(debug) then print, "read image with HWP = ",hwp_flat_in[i_in],' deg'
endfor
;
big_flat_out = big_flat_in * 0.0d0 + bad_pix_value
;
;	sort by HWP angle
;
ind_hwp = SORT(hwp_flat_in)
;
hwp_flat_in = hwp_flat_in[ind_hwp]
big_flat_in = big_flat_in[*,*,ind_hwp]
big_header = big_header[*,ind_hwp]
big_header_lines = big_header_lines[ind_hwp]
;
;	for each pixel, compute sinusoidal phase
;
;debug = 1
if(debug) then begin
	device,decompose=0
	color_names = ["black", "white", "red", "orange", "yellow", "green", "blue", "violet", "gray"]
	color_values= [10,11,12,13,14,15,16,17,18,19]
	black = 10 & white = 11 & red = 12 & orange = 13 & yellow = 14 & green = 15 & blue = 16 & violet = 17 & gray = 18
	device, decomposed=0
	loadct, 13, /silent
	tvlct,[  0,255,255,255,255, 50,  0,148,233], $
	      [  0,255,  0,165,200,185,  0,  0,233], $
	      [  0,255,  0,  0,  0, 50,255,211,233], 10
	;
	xmin = MIN(hwp_flat_in)
	xmax = MAX(hwp_flat_in)
	dx = xmax - xmin

	xmax = xmax + 0.025*dx
	xmin = xmin - 0.025*dx
	;
endif
;debug = 0
;
A = dblarr(3)
;
for icol = 0, nxy[1]-1 do begin
	;
	for irow = 0, nxy[2]-1 do begin
		;
		yvec = REFORM(big_flat_in[icol, irow, *])
		ind_good = WHERE(yvec ne bad_pix_value and yvec gt 0.05 and yvec lt 2.0, ngood)
		;
		if(debug) then print, "Icol, Irow, Ngood = ",icol, irow, ngood
		;
		if(ngood gt 4) then begin
			if(debug) then begin
				;
				ymax = MAX(yvec[ind_good])
				ymin = MIN(yvec[ind_good])
				dy = ymax - ymin
				ymax = ymax + 0.1*dy
				ymin = ymin - 0.1*dy
				window, 16, title="(C,R)=("+STRING(icol, irow,format='(I4,",",I4)')+') pixel value vs hwp angle (input)'
				plot, hwp_flat_in, yvec, xrange=[xmin,xmax], yrange=[ymin, ymax], background=black,$
						color=white,/NODATA,$
						xtitle='HWP angle (orig, in deg.)',$
						ytitle='pixel value', $
						xstyle=1, ystyle=1
				oplot, hwp_flat_in, yvec, psym=4, symsize=3, thick=3, color=red
				oplot, hwp_flat_in, yvec, psym=-4, symsize=0.05, thick=3, linestyle=2, color=blue
			endif
			;
			;	do a sinusoidal fit
			;
			weights = fltarr(ngood)
			weights[*] = 1.0d0
			;
			;	set up initial guesses, if not done, else use previous
			;
			if(A[0] eq 0.0) then begin
				zmin = Min(yvec[ind_good])
				zmax = Max(yvec[ind_good])
				dz = zmax - zmin
				;
				A = [dz/2.0,0.0, (zmax+zmin)/2.]		; amplitude, phase, offset, initial guesses
			endif
			;
			angle = hwp_flat_in[ind_good] * 4.0d0
			;
			yfit = CURVEFIT(angle, yvec[ind_good], weights, A, SIGMA, CHISQ=chisq, ITER=iter, ITMAX=50, STATUS=status, /DOUBLE, FUNCTION_NAME='gfunct')
			if(status ne 0) then begin
				print, "status =/ 0"
				stop
				goto, loop_end
			endif
			;
			;	fix phase offset
			;
			a[1] = (a[1] + 36000.0d0) MOD 360.0d0
			;
			if(debug) then begin
				xmodel = findgen(101) * 3.5
				ymodel = a[2] + a[0] * sin(4.0*( ((xmodel+a[1]/4.)+36000.0) MOD 360.0d0 )*0.017453)
				oplot, xmodel, ymodel, psym=-3, color=red
				;
				print, "after first fit, A = ",A
				print, "... iteration number = ",iter,"  ChiSq = ",chisq
				;
				print, "P_inst = ",abs(a[0])*100.0,"%   PA = ",(90.0 - a[1])/2.0," deg"
			endif
	    	;
	    	;	advance phase by angle
	    	;
	    	new_angle = ((hwp_flat_in[ind_good] - offset_angle) + 720.0d0) MOD 360.0d0
	    	new_model = a[2] + a[0] * sin(4.0d0*(new_angle+(a[1]/4.0d0))*0.017453d0)
	    	;
	    	;	save in output array
	    	;
	    	big_flat_out[icol, irow, ind_good] = new_model[*]
	    	;
	    	if(debug) then begin
	    		;
	    		;	and replot
	    		;
	    		oplot, new_angle, new_model, psym=-3, color=white, thick=3
	    	    ;
	    	    ;
	    		;	and obs order
		    	window, 14, title='(C,R)=('+STRING(icol,irow,format='(I4,",",I4)')+')  pixel value by obs order'
		    	order = indgen(16)
		    	plot, order, yvec[ind_good[obs_order]], yrange=[ymin, ymax], background=black, color=white, /nodata
		    	oplot, order, yvec[ind_good[obs_order]], psym=4, color=orange, thick=5
		    	oplot, order, new_model[obs_order], psym=-3, color=blue, thick=2
			    ;
			    ;	refit
			    ;
			    angle = new_angle * 4.
				yfit = CURVEFIT(angle, yvec[ind_good], weights, A, SIGMA, STATUS=status, /DOUBLE, FUNCTION_NAME='gfunct')
				a[1] = (a[1] + 36000.0) MOD 360.0d0
				print, "After retardation, A = ",A
				;
				stop
				;
			endif
		endif
		loop_end:
	endfor
endfor
;
; save results, if not debugging
;
if(debug eq 0) then begin
	for i_out = 0, n_in-1 do begin
		;
		;	fetch header and add comment
		;
		this_header = big_header[0:big_header_lines[i_out]-1,i_out]
		sxaddpar, this_header, 'COMMENT', 'Rotated Polar Flat Field by '+STRING(offset_angle)+' deg.'
		sxaddpar, this_header, 'COMMENT', '  On '+SYSTIME()
		sxaddpar, this_header, 'MB_PHASE', offset_angle,'Phase offset angle added to data'
		;
		this_image = REFORM(big_flat_out[*,*,i_out])
		;
		this_path = flat_list_out[i_out]
		;
		writefits, this_path, this_image, this_header
	endfor
endif
;
ier = 0
the_end:
end
